package eduni.simanim;

import java.awt.*;
import java.applet.Applet;

/**
 * An animation parameter, parameters are used to display
 * data relating to its parent entity during an animation.
 * @version     1.0, 4 September
 * @author      Ross McNab
 */

public class Anim_param {
  private Anim_entity parent;
  private int type;
  private String old_value;
  private String value;
  private String name;
  private Point parpos;
  private Point relpos;

  /** Display type: Don't display. */
  public static final int HIDDEN     = 0;
  /** Display type: Show as the entity's bitmap. */
  public static final int STATE      = 1;
  /** Display type: Show value only. */
  public static final int VALUE      = 2;
  /** Display type: Show parameter's name only. */
  public static final int NAME       = 3;
  /** Display type: Show both the parameter's name and its value. */
  public static final int NAME_VALUE = 4;
  public static final String[] enum_type = {
    "HIDDEN", "STATE", "VALUE", "NAME", "NAME_VALUE"
  };

  /** Constructor, with (x, y) co-ordinate.
   * @param name The parameter's name
   * @param type One of the display types above
   * @param value The initial value of the parameter
   * @param x The X co-ordinate of the parameter relative to the left of
   *          its parent entity.
   * @param y The Y co-ordinate of the parameter relative to the top of
   *          its parent entity.
   */
  public Anim_param(String name, int type, String value, int x, int y) {
    this.type = type;
    this.name = name;
    this.value = value;
    old_value = value;
    relpos = new Point(x,y);
  }

  /** Constructor, with co-ordinates defaulting to (0, 0).
   * @param name The parameter's name
   * @param type One of the display types above
   * @param value The initial value of the parameter
   */
  public Anim_param(String name, int type, String value) {
    this.type = type;
    this.name = name;
    this.value = value;
    old_value = value;
    relpos = new Point(0,0);
  }

  String get_value() { return(value); }
  int get_type() { return(type); }
  String get_name() { return(name); }

  void set_value(String val) { old_value = value; value = val; }
  void set_type(int type) { this.type = type; }
  void set_position(int x, int y) { relpos = new Point(x,y); }
  void set_parent(Anim_entity ent) { parent = ent; }

  // Convert the string 'type' to the enum value (or -1 if not in enum)
  static int parseType(String type) {
    for(int i=0; i<enum_type.length; i++)
      if(enum_type[i].equals(type)) return i;
    return(-1);
  }

  void update_position() {
    parpos = parent.get_position();
  }

  /*
  void drawString(Graphics g, String s, int x, int y) {
    FontMetrics fm = Sim_anim.applet.getFontMetrics(g.getFont());
    if (fm == null) System.out.println("Could not get font metrics");
    int a = fm.getMaxAscent();
    int d = fm.getMaxDescent();
    g.setColor(Color.lightGray);
    g.fillRect(x,y-a,fm.stringWidth(s), a+d);
    g.setColor(Color.black);
    g.drawString(s, x, y);
  }
  */

  void draw(Graphics g) {
    switch(type) {
      case(HIDDEN):
        break;
      case(STATE):
        break;
      case(VALUE):
        g.setColor(Color.lightGray);
        g.drawString(old_value, parpos.x+relpos.x, parpos.y+relpos.y);
        g.setColor(Color.black);
        g.drawString(value, parpos.x+relpos.x, parpos.y+relpos.y);
        break;
      case(NAME):
        g.drawString(name, parpos.x+relpos.x, parpos.y+relpos.y);
        break;
      case(NAME_VALUE):
        g.setColor(Color.lightGray);
        g.drawString(name+" = "+old_value, parpos.x+relpos.x, parpos.y+relpos.y);
        g.setColor(Color.black);
        g.drawString(name+" = "+value, parpos.x+relpos.x, parpos.y+relpos.y);
        break;
    }
  }
}
