
/*
** Author : Steven Ramsay <smr@dcs.ed.ac.uk>, 15/03/97
** Files  : Choices.java, Choices.class
** Package: simdcp
** Update : 11/06/97 
**
** Description:
** The structure representing a choice in protocol.
*/

package eduni.simdcp;

/* Methods defined in final class Choices
** PUBLIC
** Choices(int)
**
** PROTECTED
**        String  Name()
**        int     Total_Actions()
**        int     Number()
**        void    Set_Next_State(int)
**        int     Next_State()
**        void    Add_Action(Actable)
**        Actable Action(int)
**        String  Display()
**
** PRIVATE
** static void    Error(String)
*/
final class Choices implements Constants {
  // This class represents the choice of a set of actions in a state depending
  // on the first action of each choice.  Each choice consists of a set of 
  // actions and ends with the next state to make the transition to.

  private int       choice_num;                   // Number of choice in state
  private int       next_state;           // Can't tell if state coord or part
  private int       total_actions;        // Total number of actions available
  private Actable[] actions;                       // list of actions possible

  public Choices(int number) {
      this.choice_num    = number;
      this.actions       = new Actable[MAX_ACTIONS + 1];
      this.total_actions = 0;
  }                                                  // constructor Choices(1)

  protected int Total_Actions() {
      return this.total_actions;
  }                                                    // int Total_Actions(0)

  protected int Number() {
      return this.choice_num;
  }                                                           // int Number(0)

  protected void Set_Next_State(int next) {
      this.next_state = next;
  }                                                  // void Set_Next_State(1)

  protected int Next_State() {
      return this.next_state;
  }                                                       // int Next_State(0)

  protected void Add_Action(Actable new_action) {
      if (this.total_actions < MAX_ACTIONS) {
          this.total_actions++;
          this.actions[this.total_actions] = new_action;
      }
      else {
          Error("Cannot add another action to choice");
      }
  }                                                       // int Add_Action(1)

  protected Actable Action(int number) {
      Actable tmp_action = null;

      if ((number > 0) && (number <= this.Total_Actions())) {
          tmp_action = this.actions[number];
      }
      else {
          Error("No action exists at "+number);
      }
      return tmp_action;
  }                                                       // Actable Action(1)

  protected String Display() {
      int     i;
      String  note1 = new String();
      String  note2 = new String();
      Actable tmp_action;
      
      for (i=1; i <= this.Total_Actions(); i++) {
           tmp_action = this.Action(i);
           if (tmp_action != null) {
               note2 = new String(note1+tmp_action.Display()+" . ");
               note1 = note2;
           }
      }
      return (note1+"S"+this.Next_State());
  }                                                         // void Display(0)

  private static void Error(String note) {
      // Method to output error message if required
      Scheduler.Error_Pub(note+" (class Choice)");
  }                                                           // void Error(1)

}                                                             // class Choices

/*------------------------------ End of Choices.java ------------------------*/

