
/*
** Author : Steven Ramsay <smr@dcs.ed.ac.uk>, 15/03/97
** Files  : Constants.java, Constants.class
** Update : 11/06/97 
** Package: simdcp
**
** Description:
** All global values set on initialisation
*/
 
package eduni.simdcp;

/* Methods defined in interface Constants
** -- none defined: interface is abstract --
** PUBLIC
** ABSTRACT
*/
interface Constants {
  // This interface defines the constants used in several classes
  // Mainly associated with messages

  // Initial options
  public static final int MAX_SITES   = 10;         // max no. of sites allowed
  
  public static final int MAX_TABLES  = 3;        // max no. of tables allowed
  public static final int MAX_STATES  = 10;       // max no. of states allowed
  public static final int MAX_CHOICES = 10;      // max no. of choices allowed
  public static final int MAX_ACTIONS = 6;       // max no. of actions allowed

  public static final int MAX_MSG_NAMES   = 15;   // max message names allowed
  public static final int MAX_CTRL_NAMES  = 5;    // max control names allowed
  public static final int MAX_LOG_NAMES   = 6;        // max log names allowed
  public static final int MAX_LOG_ENTRIES = 10;     // max log entries allowed

  public static final int MAX_INTERVAL = 10;          // max messsage interval

  public static final int DETECT_MESSAGE_LOOP = 5;     // msgs to detect cycle
  public static final int DETECT_FAILURE_LOOP = 5; // failures to detect cycle
  
  // Transfer times of comms links - must have only one delay
  // Different delays cause messages to be delayed and look lost
  public static final double LINK_DELAY = 1.0;

  public static final double CRASH_DELAY = 0.01;            // Delay for crash
  public static final double RECOVER_DELAY = 0.001;      // Delay for recovery
   
  public static final double VERSION = 1.1;                 // Current version
  
  public static final boolean RELEASEv = false;     // Is it a release version 

  public static final boolean SIMANIM = true;      // Does package use SimAnim 
  // DO NOT CHANGE

  public static final int SITE  = 1;                         // Entity Type ID
  public static final int COORD = 2;  
  public static final int PART  = 3;
  public static final int BOTH  = 4;        // Both a Coord and a Part at site
  public static final int SCHED = -1;

  public static final int OK    =  100;                                // Okay
  public static final int RETRY = -100;                               // Retry
  public static final int FAIL  = -200;                                // Fail


  public static final int NONE = 0, UNKNOWN = 0;     // None/Unknown value = 0
  public static final int COMMIT = 1;                                // Commit
  public static final int ABORT  = -1;                                // Abort


  public static final int SLEEPING  = -1;         // Possible states a process
  public static final int RUN_OK    =  5;            // can be in - its status
  public static final int FAILED    = -5;


  public static final int NEVER    = 19;           // Alternatives for setting 
  public static final int ALWAYS   = 20;        // failure and recovery points
  public static final int PROMPT   = 21;
  public static final int INTERVAL = 22;
  public static final int STATES   = 23;
  public static final int RANDOM   = 24;
 

  public static final int UNIFORM = 1;            // Probability distributions
  public static final int NORMAL  = 2;              // These are examples only
  public static final int NUM_PR_DISTS = 2;   // Number of prob. distributions


  public static final int NO  = 0;
  public static final int YES = 1;

  public static final int R = 1;                              // Read from log
  public static final int W = -1;                              // Write to log

  public static final int SND = 1;                             // Send message
  public static final int REC = 2;                          // Receive message
  public static final int UNI = 1;                       // Message is Unicast
  public static final int MLT = 2;                     // Message is Multicast


  // Sim Anim Constants

  public static final int SCHED_w  = 32;
  public static final int SCHED_h  = 32;
  public static final int FILTER_w = 16;
  public static final int FILTER_h = 16;
  public static final int COORD_h  = 20;
  public static final int COORD_w  = 20;
  public static final int PART_h   = 20;
  public static final int PART_w   = 20;
  public static final int PORT1_h  = 8;
  public static final int PORT1_w  = 8;

  public static final int CP_gap   = 10;         // Gap between Coord and Part
  public static final int FCP_pgap = 15;  // Gap between Filter and C, P ports

  public static final int NAME_LENGTH = 70;           // Length of state names

}                                                       // interface Constants

/*----------------------------- End of Constants.java -----------------------*/

