
/*
** Author : Steven Ramsay <smr@dcs.ed.ac.uk>, 15/03/97
** Files  : Ctrl.java, Ctrl.class
** Package: simdcp
** Update : 11/06/97 
**
** Description:
** Structure of control actions in protocol table.
*/

package eduni.simdcp;

/* Methods defined in final class Ctrl
** PUBLIC
** Ctrl(String, int)
** Ctrl(String, int, int, int, int)
**        String  Name()
**        String  Display()
**        int     Info()
**
** PROTECTED
**        boolean Is_Msg()
**        Msg     Msg()
**
** PRIVATE
*/
final class Ctrl implements Actable {
  // This class represents the control actions which are seperate from the 
  // actual messages.  These might be to indicate a decision must be made,
  // e.g. if participant must commit or abort txn, or it could be control
  // messages from coordinator to scheduler e.g. to indicate transaction has
  // terminated and the result i.e. was it aborted.
  private String  ctrl_name;                            // control action name
  private int     info;                 // Extra information e.g. Commit/Abort
  private boolean is_msg;                                 // Is ctrl a message
  private Msg     ctrl_msg;

  public Ctrl(String name, int info) {
      // Constructor for class
      this.ctrl_name = name;
      this.info      = info;
      this.is_msg    = false;
      this.ctrl_msg  = null;
  }                                                     // constructor Ctrl(2)

  public Ctrl(String name, int msg_pair, int msg_sr, int msg_umb, int info) {
      // Constructor for class
      this(name, info);
      this.ctrl_msg = new Msg(name, msg_pair, msg_sr, msg_umb, info);
      this.is_msg   = true;
  }                                                     // constructor Ctrl(5)

  public String Name() {
      return this.ctrl_name;
  }                                                          // String Name(0)

  public String Display() {
      // Method used in displaying the table
      String note = null;
      if (Option.Show_Debug()) {
          if (this.Is_Msg()) {             // Control action is also a message
              note = new String("Ctrl-"+this.ctrl_msg.Display());
          }
          else {             // Control action indicates a decision to be made
              note = new String("Cdec:"+this.Name()+"("+this.info+")");
          }
      }
      else {
          note = new String("C:"+this.Name());
      } 
      return note;
  }                                                       // String Display(0)

  public int Info() {
      // Public method to return the extra info in ctrl.
      return this.info;
  }                                                             // int Info(0)

  protected boolean Is_Msg() {
      // Method to indicate if the control action is also a message 
      return this.is_msg;
  }                                                       // boolean Is_Msg(0)

  protected Msg Msg() {
      // Method to return message 
      Msg msg = this.ctrl_msg;
      return msg;
  }                                                              // Msg Msg(0)
}                                                                // class Ctrl

/*------------------------------ End of Ctrl.java ---------------------------*/

