
/*
** Author : Steven Ramsay <smr@dcs.ed.ac.uk>, 15/11/96
** Files  : DCPsim.java, DCPsim.class, DCPsim.html, tracefile
** Update : 11/06/97 
** Package: simdcp
**
** Description:
** Simulation model for Distributed Commit Protocols in SimJava
** The coordinator and participants communicate via a scheduler,
** with all messages being filtered through an interface at each Site.
** The scheduler is used to control the message passing.
**
** This package has been updated for JDK 1.1 ONLY
** It is NOT compatable with JDK1.0.2
**
** GUI windows can be displayed with SimAnim but there is a delay in 
** destroying them at the end - and only after a relayout is done.
**
** This will execute as an applet only
*/

package eduni.simdcp;

import eduni.simjava.*;
import eduni.simanim.*;
import java.awt.*;
import java.awt.event.*;

/* Methods defined in class DCPsim
** PUBLIC
**        void    anim_init()
**        void    anim_layout()
**        void    actionPerformed(ActionEvent)
**        void    itemStateChanged(ItemEvent)
**        void    focusGained(FocusEvent)
**        void    focusLost(FocusEvent)
**
** PROTECTED
**        void    finalize()
**        void    Do_ReLayout()
**
** PRIVATE
**        void    constrain(6)
**        void    constrain(14)
*/
public class DCPsim extends Anim_applet 
                    implements FocusListener {

  //  private static boolean SIM_RUNNING    = false;

  private static O_Box option_window = null;
  private static I_Box info_window    = null;

  Label     sites_l, coord_l, vote_l, table_l, fail_l, rcvr_l;
  TextField sites_tf, coord_tf, vote_tf, table_tf, fail_tf, rcvr_tf;
  Button    option_b, info_b;

  Panel     display_p1, display_p2, button_p, applet_p;

  GridBagLayout gridbag = new GridBagLayout();

  public void anim_init() {
      // Method to setup the options and GUI inputs 

      this.addFocusListener(this);

      display_p1 = new Panel();
      display_p2 = new Panel();
      button_p   = new Panel();
      applet_p   = new Panel();

      display_p1.setLayout(gridbag);
      display_p2.setLayout(gridbag);
      button_p.setLayout(new FlowLayout(FlowLayout.CENTER));
      applet_p.setLayout(gridbag);

      sites_tf = new TextField(""+Option.Num_Sites(), 2);
      coord_tf = new TextField(""+Option.Coord_Site(), 2);
      vote_tf  = new TextField(""+Option.Num_Abort(), 2);
      table_tf = new TextField(Option.Table().Name(), 20);
      fail_tf  = new TextField(Global.Name_FR_Method(Option.F_Method()), 20);
      rcvr_tf  = new TextField(Global.Name_FR_Method(Option.R_Method()), 20);

      sites_tf.setEditable(false); 
      coord_tf.setEditable(false); 
      vote_tf.setEditable(false); 
      table_tf.setEditable(false); 
      fail_tf.setEditable(false); 
      rcvr_tf.setEditable(false); 

      constrain(display_p1, new Label("Number of sites:"), 0, 0, 1, 1);
      constrain(display_p1, sites_tf, 1, 0, 1, 1, 
                GridBagConstraints.NONE, 
                GridBagConstraints.NORTHEAST, 1.0, 1.0, 2, 0, 2, 5);
      constrain(display_p1, new Label("Voting to Abort:"), 0, 1, 1, 1);
      constrain(display_p1, vote_tf, 1, 1, 1, 1, 
                GridBagConstraints.NONE, 
                GridBagConstraints.NORTHEAST, 1.0, 1.0, 2, 0, 2, 5);
      constrain(display_p1, new Label("Coordinator at site:"), 0, 2, 1, 1);
      constrain(display_p1, coord_tf, 1, 2, 1, 1, 
                GridBagConstraints.NONE, 
                GridBagConstraints.NORTHEAST, 1.0, 1.0, 2, 0, 2, 5);

      constrain(display_p2, new Label("Table to use:"), 2, 0, 1, 1);
      constrain(display_p2, table_tf, 3, 0, 1, 1, 
                GridBagConstraints.NONE, 
                GridBagConstraints.NORTHEAST, 1.0, 1.0, 2, 0, 2, 5);
      constrain(display_p2, new Label("Failure method:"), 2, 1, 1, 1);
      constrain(display_p2, fail_tf, 3, 1, 1, 1, 
                GridBagConstraints.NONE, 
                GridBagConstraints.NORTHEAST, 1.0, 1.0, 2, 0, 2, 5);
      constrain(display_p2, new Label("Recovery method:"), 2, 2, 1, 1);
      constrain(display_p2, rcvr_tf, 3, 2, 1, 1, 
                GridBagConstraints.NONE, 
                GridBagConstraints.NORTHEAST, 1.0, 1.0, 2, 0, 2, 5);

      option_b = new Button("Options Window");
      info_b   = new Button("About");

      button_p.add(option_b);
      button_p.add(info_b);
 

      constrain(applet_p, display_p1, 0, 0, 1, 1, GridBagConstraints.BOTH, 
                GridBagConstraints.NORTHWEST, 1.0, 1.0, 10, 10, 2, 10);
      constrain(applet_p, display_p2, 1, 0, 1, 1, GridBagConstraints.BOTH, 
                GridBagConstraints.NORTHWEST, 1.0, 1.0, 10, 20, 2, 10);
      constrain(applet_p, button_p, 0, 1, 2, 1, GridBagConstraints.HORIZONTAL, 
                GridBagConstraints.CENTER, 1.0, 0.0, 2, 10, 5, 10);

      this.add("North", applet_p);

      option_window = new O_Box("DCPsim Option", option_b, this);
      option_b.addActionListener(option_window);
      info_window    = new I_Box("About DCPsim", info_b);
      info_b.addActionListener(info_window);
  }

  public void anim_layout() {
      // This method sets up the entities to begin the simulation.

      // Set up entities for a Scheduler and the required number of
      // Participants, one of which will be the Coordinator.
      Scheduler scheduler;
      Filter    filter;
      Coord     coordinator;
      Part      participant;

      Global.Init_Variables();

      Global.Window_Visible(option_window, false);
      Global.Window_Visible(info_window, false);

      int i;

      sites_tf.setText(""+Option.Num_Sites());
      coord_tf.setText(""+Option.Coord_Site());
      vote_tf.setText(""+Option.Num_Abort());
      table_tf.setText(Option.Table().Name());
      fail_tf.setText(Global.Name_FR_Method(Option.F_Method()));
      rcvr_tf.setText(Global.Name_FR_Method(Option.R_Method()));
      
      scheduler = new Scheduler("SCHED", Option.Sched_x(), 
                                         Option.Sched_y(),
                                          "sched", "port1");
      Sim_system.add(scheduler);

      scheduler.Reset_Window();

      Scheduler.Info_Pub("Laying out simulation");

      for (i=1; i <= Option.Num_Sites(); i++) {
  	  // For each site create the entities; Filter, Coordinator 
	  // and Participant.
          filter      = new Filter("SITE"+i, i, Option.Filter_x(i),
                                   Option.Filter_y(i), "filter", "port1");
          coordinator = new Coord("COOR"+i, i, Option.Coord_x(i), 
                                  Option.Coord_y(i), "coord", "port1");
          participant = new Part("PART"+i, i, Option.Part_x(i), 
                                  Option.Part_y(i), "part", "port1");
          Sim_system.add(filter);
          Sim_system.add(coordinator);
          Sim_system.add(participant);

	  filter.Reset_Window();         
	  coordinator.Reset_Window();         
          participant.Reset_Window();         

	  // Link each entities ports to form a logical site, via the filter
          // and link the filter to the Scheduler.
          Sim_system.link_ports("SCHED", "port_Site"+i, 
                                "SITE"+i, "port_Sched");
          Sim_system.link_ports("SITE"+i, "port_Coord"+i, 
                                "COOR"+i, "port_Site"+i); 
          Sim_system.link_ports("SITE"+i, "port_Part"+i, 
                                "PART"+i, "port_Site"+i);
      }
      Sim_system.link_ports("SCHED", "port_Self", 
                            "SCHED", "port_Self");

  }                                                     // void anim_layout(0)

  protected void Do_Relayout() {
      anim_relayout();
  }                                                     // void Do_Relayout(0)

  public void focusGained(FocusEvent event) {
      // Method to handle event of getting focus
      // This uses JDK 1.1 methods
      option_b.requestFocus();
  }                                                     // void focusGained(1)

  public void focusLost(FocusEvent event) {
      // Method to handle event of losing focus
      // This uses JDK 1.1 methods
      // Do nothing
  }                                                       // void focusLost(1)

  protected void finalize() throws Throwable {
      // Finalize method to clean up windows etc on termination
      option_window.dispose();
      info_window.dispose();
      super.finalize();
  }                                                        // void finalize(0)

  /* The following constrain methods are from "Java in a Nutshell" Section 5
  ** examples on GUIs.  Used to position components in containers.
  */  
  private void constrain(Container container, Component component, 
                        int grid_x, int grid_y, int grid_w, int grid_h,
                        int fill, int anchor, 
                        double weight_x, double weight_y,
                        int top, int left, int bottom, int right) 
  {

      GridBagConstraints c = new GridBagConstraints();
      c.gridx = grid_x; 
      c.gridy = grid_y;
      c.gridwidth = grid_w; 
      c.gridheight = grid_h;
      c.fill = fill; 
      c.anchor = anchor;
      c.weightx = weight_x; 
      c.weighty = weight_y;

      if (top + bottom + left + right > 0) {
          c.insets = new Insets(top, left, bottom, right);
      }
      ((GridBagLayout) container.getLayout()).setConstraints(component, c);
      container.add(component);
  }                                                      // void constrain(14)
    
  private void constrain(Container container, Component component, 
                         int grid_x, int grid_y, int grid_w, int grid_h) 
  {
      constrain(container, component, grid_x, grid_y, grid_w, grid_h, 
                GridBagConstraints.NONE, GridBagConstraints.NORTHWEST, 
                0.0, 0.0, 0, 0, 0, 0);
  }                                                       // void constrain(6)

}                                                              // class DCPsim

/*---------------------------- End of DCPsim.java --------------------------*/

