
/*
** Author : Steven Ramsay <smr@dcs.ed.ac.uk>, 07/02/97
** Files  : Global.java, Global.class
** Update : 11/06/97 
** Package: simdcp
**
** Description:
** All global data and methods accessible to all classes in package
** Mainly options and display routines.
*/

package eduni.simdcp;

import java.awt.*;

/* Methods defined in class Global
** PUBLIC
** -- no constructor -- 
**
** PROTECTED
** static void    Init_Variables()
** static void    Inc_Msg_Round()
** static void    Inc_Messages_Sent()
** static void    Inc_Messages_Rec()
** static void    Inc_Real_Msgs_Sent()
** static void    Inc_Real_Msgs_Rec()
** static void    Inc_Ctrl_Msgs_Sent()
** static void    Inc_Ctrl_Msgs_Rec()
** static void    Inc_DT_Log_Writes()
** static void    Inc_DT_Log_Reads()
** static void    Inc_Failed_PART()
** static void    Inc_Recovered_PART()
** static void    Inc_Timeouts_COORD()
** static void    Inc_Timeout_Cycles()
** static void    Inc_FailR_Cycles()
** static void    Reset_Sent_Msgs()                            // Msg Interval
** static void    Reset_Rec_Msgs()                             // Msg Interval
** static void    Inc_Num_Have_Failed(int)                  // Specific States
** static void    Reset_Num_Have_Failed(int)                // Specific States
** static int     Message_Round()
** static int     Messages_Sent()
** static int     Messages_Rec()
** static int     Real_Msgs_Sent()
** static int     Real_Msgs_Rec()
** static int     Ctrl_Msgs_Sent()
** static int     Ctrl_Msgs_Rec()
** static int     DT_Log_Writes()
** static int     DT_Log_Reads()
** static int     Failed_PART()
** static int     Recovered_PART()
** static int     Timeouts_COORD()
** static int     Timeout_Cycles()
** static int     FailR_Cycles()
** static int     Num_Errors()
** static int     Num_Sent_Msgs()                              // Msg Interval
** static int     Num_Rec_Msgs()                               // Msg Interval
** static int     Num_Have_Failed(int)                      // Specific States
** static int     Total_Have_Failed(int)                    // Specific States
** static void    Debug_CLI(String)
** static void    Error_CLI(String)
** static void    Info_CLI(String)
** static boolean In_List(int[], int, int)
** static String  Name_CorA(int)
** static String  Name_RorW(int)
** static String  True_False(boolean)
** static String  Yes_No(boolean)
** static String  Name_Status(int)
** static String  Name_FR_Method(int)
** static String  Short_Type(int)
** static String  Full_Type(int)
** static void    Window_Visible(Component, boolean)
** static void    Enable_Event(Component, boolean)
** static int     Choice_Index_Count(Choice)
**        void    Create_Choice_Index(Choice, int, int)
**        void    Update_Choice_Index(Choice, int, int)
**
** PRIVATE
** static void    Inc_Total_Have_Failed(int)                // Specific States
** static void    Inc_Sent_Msgs()                              // Msg Interval
** static void    Inc_Rec_Msgs()                               // Msg Interval
*/
class Global implements Constants {
  // Class for global methods and variables


  // *********** DO NOT CHANGE BELOW THIS LINE ***********
  
  private static int num_of_errors  = 0;             // Total number of errors 
  private static int message_rounds = 0;               // Total message rounds
  private static int total_msg_sent = 0;      // Total number of messages sent
  private static int total_msg_rec  = 0;  // Total number of messages received
  private static int real_msg_sent  = 0;        // No. of "REAL" messages sent
  private static int real_msg_rec   = 0;    // No. of "REAL" messages received
  private static int ctrl_msgs_sent = 0;    // Number of control messages sent
  private static int ctrl_msgs_rec  = 0;   // No. of control messages received
  private static int dt_log_writes  = 0;   // No. of write accesses to DT logs
  private static int dt_log_reads   = 0;    // No. of read accesses to DT logs
  private static int PART_failed    = 0;        // No. of participants failing
  private static int PART_recovered = 0;     // No. of participants recovering
  private static int COORD_timeouts = 0;        // No. of coordinator timeouts
  private static int timeout_cycles = 0;     // No. of timeout cycles detected
  private static int failr_cycles   = 0;   // No. of fail/rcvr cycles detected

  private static int sent_msg_count = 0;    // count messages sent (not total)
  private static int rec_msg_count  = 0;     // count messages rec (not total)

  private static int[] num_have_failed   = new int[MAX_STATES];
  private static int[] total_have_failed = new int[MAX_STATES];
  

  protected static void Init_Variables() {
      // Method to initialise all variables used as counters during 
      // simulation.  Also required to reset them after each execution.
      num_of_errors  = 0;                            // Total number of errors 
      message_rounds = 0;                              // Total message rounds
      total_msg_sent = 0;                     // Total number of messages sent
      total_msg_rec  = 0;                 // Total number of messages received
      real_msg_sent  = 0;                       // No. of "REAL" messages sent
      real_msg_rec   = 0;                   // No. of "REAL" messages received
      ctrl_msgs_sent = 0;                   // Number of control messages sent
      ctrl_msgs_rec  = 0;                  // No. of control messages received
      dt_log_writes  = 0;                  // No. of write accesses to DT logs
      dt_log_reads   = 0;                   // No. of read accesses to DT logs
      PART_failed    = 0;                       // No. of participants failing
      PART_recovered = 0;                    // No. of participants recovering
      COORD_timeouts = 0;                       // No. of coordinator timeouts
      timeout_cycles = 0;                    // No. of timeout cycles detected
      failr_cycles   = 0;                  // No. of fail/rcvr cycles detected

      sent_msg_count = 0;                   // count messages sent (not total)
      rec_msg_count  = 0;                    // count messages rec (not total)

      num_have_failed   = new int[MAX_STATES];
      total_have_failed = new int[MAX_STATES];
  }                                                  // void Init_Variables(0)

  protected static void Inc_Msg_Round() {
      // Method to increment the message rounds
      message_rounds++;
      Scheduler.Info_Pub("\t\t\t\tMessage Round "+Message_Round());
  }                                               // void Inc_Message_Round(0)

  protected static void Inc_Messages_Sent() {
      // Method to increment the messages sent
      total_msg_sent++;
  }                                               // void Inc_Messages_Sent(0)

  protected static void Inc_Messages_Rec() {
      // Method to increment the messages received
      total_msg_rec++;
  }                                                // void Inc_Messages_Rec(0)

  protected static void Inc_Real_Msgs_Sent() {
      // Method to increment the real observable messages sent
      real_msg_sent++;
      Inc_Messages_Sent();
      Inc_Sent_Msgs();                               // Message Interval count
      Debug_CLI("*** Sent real one");
  }                                              // void Inc_Real_Msgs_Sent(0)

  protected static void Inc_Real_Msgs_Rec() {
      // Method to increment the real observable messages received
      real_msg_rec++; 
      Inc_Messages_Rec();
      Inc_Rec_Msgs();                                // Message Interval count
      Debug_CLI("*** Rec real one");
  }                                               // void Inc_Real_Msgs_Rec(0)

  protected static void Inc_Ctrl_Msgs_Sent() {
      // Method to increment the control messages sent
      ctrl_msgs_sent++;
      Inc_Messages_Sent();
      Debug_CLI("^^^ Sent control one");
  }                                              // void Inc_Ctrl_Msgs_Sent(0)

  protected static void Inc_Ctrl_Msgs_Rec() {
      // Method to increment the control messages received
      ctrl_msgs_rec++;
      Inc_Messages_Rec();
      Debug_CLI("^^^ Rec control one");
  }                                               // void Inc_Ctrl_Msgs_Rec(0)

  protected static void Inc_DT_Log_Writes() {
      // Method to increment the total writes to DT logs
      dt_log_writes++;
  }                                               // void Inc_DT_Log_Writes(0)

  protected static void Inc_DT_Log_Reads() {
      // Method to increment the total read accesses to DT logs
      dt_log_reads++;
  }                                                // void Inc_DT_Log_Reads(0)

  protected static void Inc_Failed_PART() {
      // Method to increment the total failed participants
      PART_failed++;
  }                                                 // void Inc_Failed_PART(0)

  protected static void Inc_Recovered_PART() {
      // Method to increment the total recovered participants
      PART_recovered++;
  }                                              // void Inc_Recovered_PART(0)

  protected static void Inc_Timeouts_COORD() {
      // Method to increment the total coordinator timeouts
      COORD_timeouts++;
  }                                              // void Inc_Timeouts_COORD(0)

  protected static void Inc_Timeout_Cycles() {
      // Method to increment the total timeout cycles detected
      timeout_cycles++;
  }                                              // void Inc_Timeout_Cycles(0)

  protected static void Inc_FailR_Cycles() {
      // Method to increment the total fail/recover cycles detected
      failr_cycles++;
  }                                                // void Inc_FailR_Cycles(0)

  private static void Inc_Sent_Msgs() {
      // Method to increment the number of messages sent (interval)
      sent_msg_count++;
  }                                                   // void Inc_Sent_Msgs(0)

  protected static void Reset_Sent_Msgs() {
      // Method to reset the number of messages sent (interval)
      sent_msg_count = 0;
  }                                                 // void Reset_Sent_Msgs(0)

  private static void Inc_Rec_Msgs() {
      // Method to increment the number of messages received (interval)
      rec_msg_count++;
  }                                                    // void Inc_Rec_Msgs(0)

  protected static void Reset_Rec_Msgs() {
      // Method to reset the number of messages received (interval)
      rec_msg_count = 0;
  }                                                  // void Reset_Rec_Msgs(0)

  protected static void Inc_Num_Have_Failed(int state) {
      // Method to increment the number of parts failed in a state
      if ((state > 0) && (state <= Option.Table().Total_States(PART))) {
  	  if (num_have_failed[state] < Option.Num_To_Fail(state)) {
              num_have_failed[state]++;
              Inc_Total_Have_Failed(state);
          }
      }
  }                                             // void Inc_Num_Have_Failed(1)

  protected static void Reset_Num_Have_Failed(int state) {
      // Method to reset the number of parts failed in a state
      if ((state > 0) && (state <= Option.Table().Total_States(PART))) {
          num_have_failed[state] = 0;
      }
  }                                           // void Reset_Num_Have_Failed(1)

  private static void Inc_Total_Have_Failed(int state) {
      // Method to increment the total number of parts failed in a state
      if ((state > 0) && (state <= Option.Table().Total_States(PART))) {
          total_have_failed[state]++;
      }
  }                                           // void Inc_Total_Have_Failed(1)
  
  protected static int Message_Round() {
      // Method to return the message rounds
      return message_rounds;
  }                                                    // int Message_Round(0)

  protected static int Messages_Sent() {
      // Method to return the total messages sent
      return total_msg_sent;
  }                                                    // int Messages_Sent(0)

  protected static int Messages_Rec() {
      // Method to return the total messages received
      return total_msg_rec;
  }                                                     // int Messages_Rec(0)

  protected static int Real_Msgs_Sent() {
      // Method to return the total real observable  messages sent
      return real_msg_sent;
  }                                                   // int Real_Msgs_Sent(0)

  protected static int Real_Msgs_Rec() {
      // Method to return the total real observable messages received
      return real_msg_rec;
  }                                                    // int Real_Msgs_Rec(0)

  protected static int Ctrl_Msgs_Sent() {
      // Method to return the total control messages sent
      return ctrl_msgs_sent;
  }                                                   // int Ctrl_Msgs_Sent(0)

  protected static int Ctrl_Msgs_Rec() {
      // Method to return the total control messages received
      return ctrl_msgs_rec;
  }                                                    // int Ctrl_Msgs_Rec(0)

  protected static int DT_Log_Writes() {
      // Method to return the total writes to DT logs
      return dt_log_writes;
  }                                                    // int DT_Log_Writes(0)

  protected static int DT_Log_Reads() {
      // Method to return the total read accesses to DT logs
      return dt_log_reads;
  }                                                     // int DT_Log_Reads(0)

  protected static int Failed_PART() {
      // Method to return the total failed participants
      return PART_failed;
  }                                                      // int Failed_PART(0)

  protected static int Recovered_PART() {
      // Method to return the total recovered participants
      return PART_recovered;
  }                                                   // int Recovered_PART(0)

  protected static int Timeouts_COORD() {
      // Method to return the total coordinator timeouts
      return COORD_timeouts;
  }                                                   // int Timeouts_COORD(0)

  protected static int Timeout_Cycles() {
      // Method to return the total timeout cycles detected
      return timeout_cycles;
  }                                                   // int Timeout_Cycles(0)

  protected static int FailR_Cycles() {
      // Method to return the total fail/recover cycles detected
      return failr_cycles;
  }                                                     // int FailR_Cycles(0)

  protected static int Num_Errors() {
      // Method to return the total number of errors during txn
      return num_of_errors;
  }                                                       // int Num_Errors(0)

  protected static int Num_Sent_Msgs() {
      // Method to return the number of messages sent (interval)
      return sent_msg_count;
  }                                                    // int Num_Sent_Msgs(0)

  protected static int Num_Rec_Msgs() {
      // Method to return the number of messages rec (interval)
      return rec_msg_count;
  }                                                     // int Num_Rec_Msgs(0)

  protected static int Num_Have_Failed(int state) {
      // Method to return the number of parts failed in a state
      int num = 0;

      if ((state > 0) && (state <= Option.Table().Total_States(PART))) {
	 if (num_have_failed[state] <= Option.Num_To_Fail(state)) {
             num = num_have_failed[state];
         }
      }
      return num;
  }                                                  // int Num_Have_Failed(1)

  protected static int Total_Have_Failed(int state) {
      // Method to return the number of parts failed in a state
      int num = 0;

      if ((state > 0) && (state <= Option.Table().Total_States(PART))) {
          num = total_have_failed[state];
      }
      return num;
  }                                                // int Total_Have_Failed(1)

  protected static void Debug_CLI(String note) {
      // Method to output debugging comments via command line interface
      if ((Option.Show_Debug()) && (Option.Display_CLI())) {
          System.out.println(note);
      }
  }                                                       // void Debug_CLI(1)

  protected static void Error_CLI(String note) {
      // Method to output error messages via command line interface
      num_of_errors++;                           // Increment number of Errors
      if ((Option.Show_Error()) && (Option.Display_CLI())) {
          System.out.println("ERROR: "+note);
      }
  }                                                       // void Error_CLI(1)

  protected static void Info_CLI(String note) {
      // Method to output error messages via command line interface
      if ((Option.Show_Info()) && (Option.Display_CLI())) {
          System.out.println(note);
      }
  }                                                        // void Info_CLI(1)



  protected static boolean In_List(int[] list, int id, int value) {
      // Method which returns a boolean indicating if a given value
      // is present in a specified list at the given index.
      boolean reply = false;

      if (list[id] == value) {
          reply = true;
      }
      return reply;
  }                                                      // boolean In_List(3)

  protected static String Name_CorA(int decision) {
      // Method to return a string with the name COMMIT or ABORT depending
      // on decision/outcome given. 
      String name = null;
 
      if (decision == COMMIT) {
          name = new String("COMMIT");
      }
      else if (decision == ABORT) {
          name = new String("ABORT");
      }
      else if (decision == FAILED) {
          name = new String("FAILED");
      }
      else if (decision == UNKNOWN) {
          name = new String("UNKNOWN");
      }
      else {
          name = new String(""+decision);
      }
      return name;
  }                                                     // String Name_CorA(1)

  protected static String Name_RorW(int rw) {
      // Method to return a string with R for read or W for abort depending
      // on data given. 
      String name = null;
 
      if (rw == R) {
          name = new String("R");
      }
      else if (rw == W) {
          name = new String("W");
      }
      else {
          name = new String("E");
      }
      return name;
  }                                                     // String Name_RorW(1)

  protected static String True_False(boolean truefalse) {
      // Method to return a string with the name True or False depending
      // on value given. 
      String name = null;
 
      if (truefalse == true) {
          name = new String("True");
      }
      else if (truefalse == false) {
          name = new String("False");
      }
      else {
          name = new String("E");
      }
      return name;
  }                                                    // String True_False(1)

  protected static String Yes_No(boolean truefalse) {
      // Method to return a string with the name YES or NO depending
      // on value given. 
      String name = null;
 
      if (truefalse == true) {
          name = new String("Yes");
      }
      else if (truefalse == false) {
          name = new String("No");
      }
      else {
          name = new String("E");
      }
      return name;
  }                                                        // String Yes_No(1)

  protected static String Name_Status(int status) {
      // Method to return a string with the name of the status given.
      String name = null;
 
      if (status == SLEEPING) {
          name = new String("Inactive");
      }
      else if (status == RUN_OK) {
          name = new String("Operational");
      }
      else if (status == FAILED) {
          name = new String("FAILED");
      }
      else {
          name = new String("Error");
      }
      return name;
  }                                                    // String Name_Status(1)

  protected static String Name_FR_Method(int method) {
      // Method to return a string with the name of the failure or 
      // recovery method given.
      String name = null;

      if (method == NEVER) {
          name = new String("Never");
      }
      else if (method == ALWAYS) {
          name = new String("Always");
      }
      else if (method == PROMPT) {
          name = new String("Prompt User");
      }
      else if (method == INTERVAL) {
          name = new String("Message Interval");
      }
      else if (method == STATES) {
          name = new String("Specified States");
      }
      else if (method == RANDOM) {
          name = new String("Prob. Distribution");
      }
      else {
          name = new String("Error");
      }
      return name;
  }                                                 // String Name_FR_Method(1)

  protected static String Short_Type(int type) {
      // Method to return the name of a site's type 
      String note = null;

      if (type == COORD) {
          note = new String("Coord");
      }
      else if (type == PART) {
          note = new String("Part ");
      }
      else if (type == SITE) {
          note = new String("Site ");
      }
      else if (type == BOTH) {
          note = new String("Both");
      }
      else if (type == SCHED) {
          note = new String("Sched");
      }
      else {
          note = new String(""+type);
      }
      return note;
  }                                                    // String Short_Type(1)

  protected static String Full_Type(int type) {
      // Method to return the name of a site's type 
      String note = null;

      if (type == COORD) {
          note = new String("Coordinator ");
      }
      else if (type == PART) {
          note = new String("Participant ");
      }
      else if (type == SITE) {
          note = new String("Site ");
      }
      else if (type == BOTH) {
          note = new String("Coord&Part");
      }
      else if (type == SCHED) {
          note = new String("Scheduler ");
      }
      else {
          note = new String(""+type);
      }
      return note;
  }                                                     // String Full_Type(1)

  protected static void Window_Visible(Component comp, boolean bool) {
      // Method to show or hide a window depending on bool.
      // It did use try/catch to allow 1.1 and 1.02 code as deprecated 
      // features are involved. 
      if (comp != null) {
          comp.setVisible(bool);       // JDK 1.1 code

          /* For JDK 1.0.2 compatability of deprecated methods - smr
	  ** try {
	  **     comp.setVisible(bool);         // JDK 1.1 code   
          ** } catch (NoSuchMethodError e) {
	  **     if (bool) {
	  **         comp.show();               // JDK 1.0.2 code
	  **     } else {
	  **         comp.hide();               // JDK 1.0.2 code
	  **     }
	  ** }
          */
      }
  }                                                  // void Window_Visible(2)

  protected static void Enable_Event(Component comp, boolean bool) {
      // This method enables or disables an event depending on the boolean.
      // It did use try/catch to allow 1.1 and 1.02 code as deprecated 
      // features are involved. 
      if (comp != null) {
          comp.setEnabled(bool);          // JDK 1.1 code

          /* For JDK 1.0.2 compatability of deprecated methods - smr
	  ** try {
          **     comp.setEnabled(bool);         // JDK 1.1 code
	  ** } catch (NoSuchMethodError e) {
	  **     if (bool) {
          **         comp.enable();             // JDK 1.0.2 code
          **     } else {
	  **         comp.disable();            // JDK 1.0.2 code
          **     }
          ** }
          */
      }
  }                                                    // void Enable_Event(2)

  protected static int Choice_Index_Count(Choice choice) {
      // This method returns the index count of a choice
      // It did use try/catch to allow 1.1 and 1.02 code as deprecated 
      // features are involved. 
      int num = 0;
      num = choice.getItemCount();          // JDK 1.1 code

      /* For JDK 1.0.2 compatability of deprecated methods - smr
      ** try {
      **     num = choice.getItemCount();          // JDK 1.1 code
      ** } catch (NoSuchMethodError e) {
      **     num = choice.countItems();            // JDK 1.0.2 code
      ** } 
      */
      return num;
  }                                              // void Choice_Index_Count(1)

  protected static void Create_Choice_Index(Choice choice, int from, int to) {
      // Method to create a choice index given the first and last values
      int i, count, value;

      if (to > from) {
          count = to - from;
          value = from;
      }
      else {
          count = from - to;
          value = to;
      }

      for (i=0; i <= count; i++, value++) {
          choice.addItem(""+value);
      }
  }                                             // void Create_Choice_Index(3)

  protected static void Update_Choice_Index(Choice choice, int num, int off) {
      // Method to update a choice selector to reflect a change in value
      // e.g. number of sites in txn.
      // num is value to be upper limit
      // off is the offset assuming 0 is default starting value
      // if list starts at 1, offset is 1
      int choice_indices, i, choice_upper_value;
      int count;

      /* DO NOT CHANGE AS THIS WORKS CORRECTLY - smr */

      choice_indices = Choice_Index_Count(choice);

      choice_upper_value = (choice_indices + off) - 1;

      Debug_CLI("Update Choice: upvalue "+num+", indices "
                +choice_indices+", off "+off);
      Debug_CLI("Update choice: chupvalue "+choice_upper_value); 


      if (choice_upper_value > num) {
          i = choice_indices+off;
          for (count=(choice_upper_value - num); count > 0; i--, count--) {
	      choice.remove(i-off-1);
              Debug_CLI("Removing value "+(i-1)+" at index "+(i-off-1));
          }
      }
      else if (choice_upper_value < num) {
          i = choice_indices+off;
          for (count=(num - choice_upper_value); count > 0; i++, count--) {
              choice.addItem(""+i);
              Debug_CLI("Adding value "+i+" at index "+(i-off));
          }
      }
  }                                             // void Update_Choice_Index(3)


}                                                              // class Global

/*----------------------------- End of Global.java -------------------------*/

