
/*
** Author : Steven Ramsay <smr@dcs.ed.ac.uk>, 15/03/97
** Files  : Log.java, Log.class
** Package: simdcp
** Update : 11/06/97 
**
** Description:
** Structure of log actions in protocol table.
*/

package eduni.simdcp;

/* Methods defined in final class Log
** PUBLIC
** Log(String, int, int)
**        String  Name()
**        String  Display()
**        int     Info()
**
** PROTECTED
**        int     Round()
**
** PRIVATE
*/
final class Log implements Actable {
  // This method implements the Actable interface and so it can be thought of
  // as a reference type.  This will allow the class Log to be used 
  // interchangeably with other classes implementing this interface.
  // A log consists of the name to be entered into the DT log and the 
  // relevant message round as well as some information.  At present the
  // information gives an idea as to whether the log was written during the
  // Commit or Abort path.
  private String log_name;                        // The name of the log entry
  private int    rw;                        // Is log name to be read/written?
  private int    info;                  // Extra information e.g. Commit/Abort

  public Log(String name, int rw, int info) {
      this.log_name = name;
      this.rw       = rw;
      this.info     = info;
  }                                                      // constructor Log(3)

  public String Name() {
      return this.log_name;
  }                                                          // String Name(0)

  public String Display() {
      // Method used in displaying the table
      String note = null;
      if (Option.Show_Debug()) {
          note = new String("L("+Global.Name_RorW(this.RW())
                            +","+this.Info()+"):"+this.Name());
      }
      else {
          note = new String("L:"+this.Name());
      }
      return note;
  }                                                         // void Display(0)

  public int Info() {
      // Public method to return the extra info in log.
      return this.info;
  }                                                             // int Info(0)

  protected int RW() {
      // Method to return the read/write intention of log
      return this.rw;
  }                                                               // int RW(0)

}                                                                 // class Log

/*------------------------------- End of Log.java ---------------------------*/

