
/*
** Author : Steven Ramsay <smr@dcs.ed.ac.uk>, 10/03/97
** Files  : O_Box.java, O_Box.class
** Update : 11/06/97 
** Package: simdcp
**
** Description:
** Maintains all startup option methods 
*/

package eduni.simdcp;

import java.awt.*;
import java.awt.event.*;

/* Methods defined in class O_Box
** PUBLIC
** O_Box(String, Button, DCPsim)
**        void    actionPerformed(ActionEvent)
**        void    itemStateChanged(ItemEvent)
**        void    focusGained(FocusEvent)
**        void    focusLost(FocusEvent)
**
** PROTECTED
**
** PRIVATE
**        void    Enable_F_Panel(int)
**        void    Enable_R_Panel(int)
**        void    Debug(String)
**        void    constrain(6)
**        void    constrain(14)
*/
final class O_Box extends Frame 
                  implements Constants, ActionListener, 
                             ItemListener, FocusListener    {

  String   name;

  DCPsim   sim_applet;

  Button   done_b,  applet_b;

  Checkbox show_debug_cb, show_error_cb, show_info_cb,
           show_route_cb, show_message_cb, show_ctrl_msg_cb,
           display_cli_cb, display_gui_cb;

  Checkbox f_never_cb, f_always_cb,
           f_prompt_cb, f_inter_cb, f_states_cb, f_prob_cb;

  Checkbox r_always_cb, r_never_cb,
           r_prompt_cb, r_inter_cb, r_prob_cb;

  Choice   num_sites_choice, coord_site_choice, 
           num_abort_choice, table_choice;

  Choice   f_inter_choice, r_inter_choice, 
           f_states_choice, f_num_choice, 
           f_dist_choice, r_dist_choice;

  Panel    panel1, panel2, panel3, panel4, panel5, panel6,
           panelb;

  CheckboxGroup fail_gp, rcvr_gp;

  GridBagLayout gridbag = new GridBagLayout();


  public O_Box(String name, Button applet_b, DCPsim sim) {
      super(name+" Window");
      this.name = name;
      this.Setup_Options();
      this.pack();
      this.applet_b = applet_b;
      this.sim_applet = sim;
  }                                                    // constructor O_Box(3) 

  private void Setup_Options() {
      int i;

      addFocusListener(this);

      done_b = new Button("Done");
      done_b.addActionListener(this);

      num_sites_choice  = new Choice(); // starts at 1
      coord_site_choice = new Choice(); // starts at 1
      num_abort_choice  = new Choice(); // starts at 0
      table_choice      = new Choice(); 
  
      f_inter_choice    = new Choice(); // starts at MAX
      r_inter_choice    = new Choice(); // starts at 0
      f_states_choice   = new Choice();
      f_num_choice      = new Choice(); // starts at 0
      f_dist_choice     = new Choice();
      r_dist_choice     = new Choice();
 
      num_sites_choice.addItemListener(this);
      coord_site_choice.addItemListener(this);
      num_abort_choice.addItemListener(this);
      table_choice.addItemListener(this);
      f_inter_choice.addItemListener(this);
      r_inter_choice.addItemListener(this);
      f_states_choice.addItemListener(this);
      f_num_choice.addItemListener(this);
      f_dist_choice.addItemListener(this);
      r_dist_choice.addItemListener(this);

      panel1 = new Panel();
      panel2 = new Panel();
      panel3 = new Panel();
      panel4 = new Panel();
      panel5 = new Panel();
      panel6 = new Panel(); 
      panelb = new Panel();

      panel1.setLayout(gridbag);
      panel2.setLayout(gridbag);
      panel3.setLayout(gridbag);
      panel4.setLayout(gridbag);
      panel5.setLayout(gridbag);
      panel6.setLayout(gridbag);
      panelb.setLayout(gridbag);

      this.setLayout(gridbag);

      Global.Create_Choice_Index(num_sites_choice,  1, MAX_SITES);
      Global.Create_Choice_Index(coord_site_choice, 1, Option.Num_Sites());
      Global.Create_Choice_Index(num_abort_choice,  0, Option.Num_Sites());

      Global.Create_Choice_Index(f_inter_choice, 0, MAX_INTERVAL);
      Global.Create_Choice_Index(r_inter_choice, 0, MAX_INTERVAL);
      Global.Create_Choice_Index(f_num_choice,   0, Option.Num_Sites());

      for (i=1; i <= Option.Total_Tables(); i++) { 
          table_choice.addItem(""+Option.Table(i).Name());
      }
   
      for (i=0; i <= NUM_PR_DISTS; i++) {
          f_dist_choice.addItem(Pr_Dist.Name_Pr_Dist(i));
          r_dist_choice.addItem(Pr_Dist.Name_Pr_Dist(i));
      }
     
      this.Update_States_Choice();            // Set states depending on table

      try {
        num_sites_choice.select(  Option.Num_Sites()    - 1 );
        coord_site_choice.select( Option.Coord_Site()   - 1 );
        num_abort_choice.select(  Option.Num_Abort()        );
        table_choice.select(      Option.Table_To_Use() - 1 );

        f_inter_choice.select(    Option.F_Interval()       );
        r_inter_choice.select(    Option.R_Interval()       );
        f_states_choice.select( Option.First_State_Choice() - 1 );
        f_num_choice.select( Option.Num_To_Fail(Option.First_State_Choice()) );
        f_dist_choice.select(     Pr_Dist.Pr_Dist_Failure()  );
        r_dist_choice.select(     Pr_Dist.Pr_Dist_Recovery() );

      } catch (IllegalArgumentException e) {
          Scheduler.Error_Pub("Option selection threw exception");
      }

      show_debug_cb = new Checkbox("Show Trace");
      show_debug_cb.setState(Option.Show_Debug());
      show_error_cb = new Checkbox("Show Errors");
      show_error_cb.setState(Option.Show_Error());
      show_info_cb  = new Checkbox("Show Info");
      show_info_cb.setState(Option.Show_Info());
      show_route_cb = new Checkbox("Show Routing");
      show_route_cb.setState(Option.Show_Routing());
      show_message_cb  = new Checkbox("Display Protocol Messages");
      show_message_cb.setState(Option.Show_Messages());
      show_ctrl_msg_cb  = new Checkbox("Display Control Messages");
      show_ctrl_msg_cb.setState(Option.Show_Ctrl_Messages());

      display_cli_cb = new Checkbox("Display via CLI");
      display_cli_cb.setState(Option.Display_CLI());
      display_gui_cb = new Checkbox("Display via GUI");
      display_gui_cb.setState(Option.Display_GUI());

      show_debug_cb.addItemListener(this);
      show_error_cb.addItemListener(this);
      show_info_cb.addItemListener(this);
      show_route_cb.addItemListener(this);
      show_message_cb.addItemListener(this);
      show_ctrl_msg_cb.addItemListener(this);

      display_cli_cb.addItemListener(this);
      display_gui_cb.addItemListener(this);

      fail_gp = new CheckboxGroup();
      rcvr_gp = new CheckboxGroup();

      f_never_cb  = new Checkbox("Never",   fail_gp, Option.Fail(NEVER));
      f_always_cb = new Checkbox("Always",  fail_gp, Option.Fail(ALWAYS));
      f_prompt_cb = new Checkbox("Prompt",  fail_gp, Option.Fail(PROMPT));
      f_inter_cb  = new Checkbox("Interval",fail_gp, Option.Fail(INTERVAL));
      f_states_cb = new Checkbox("States",  fail_gp, Option.Fail(STATES));
      f_prob_cb   = new Checkbox("Random",  fail_gp, Option.Fail(RANDOM));

      r_always_cb = new Checkbox("Always",  rcvr_gp, Option.Recover(ALWAYS));
      r_never_cb  = new Checkbox("Never",   rcvr_gp, Option.Recover(NEVER));
      r_prompt_cb = new Checkbox("Prompt",  rcvr_gp, Option.Recover(PROMPT));
      r_inter_cb  = new Checkbox("Interval",rcvr_gp, Option.Recover(INTERVAL));
      r_prob_cb   = new Checkbox("Random",  rcvr_gp, Option.Recover(RANDOM));

      f_never_cb.addItemListener(this);
      f_always_cb.addItemListener(this);
      f_prompt_cb.addItemListener(this);
      f_inter_cb.addItemListener(this);
      f_states_cb.addItemListener(this);
      f_prob_cb.addItemListener(this);

      r_always_cb.addItemListener(this);
      r_never_cb.addItemListener(this);
      r_prompt_cb.addItemListener(this);
      r_inter_cb.addItemListener(this);
      r_prob_cb.addItemListener(this);


      constrain(panel1, new Label("Number of Sites (max "+MAX_SITES+")"),
                0, 0, 1, 1);
      constrain(panel1, num_sites_choice, 1, 0, 1, 1, 
                GridBagConstraints.NONE, 
                GridBagConstraints.NORTHEAST, 1.0, 1.0, 2, 0, 10, 5);

      constrain(panel1, new Label("Participants to ABORT"), 0, 1, 1, 1);
      constrain(panel1, num_abort_choice, 1, 1, 1, 1, 
                GridBagConstraints.NONE,
                GridBagConstraints.NORTHEAST, 1.0, 1.0, 2, 0, 10, 5);

      constrain(panel1, new Label("Coordinator at Site"), 0, 2, 1, 1);
      constrain(panel1, coord_site_choice, 1, 2, 1, 1, 
                GridBagConstraints.NONE,
                GridBagConstraints.NORTHEAST, 1.0, 1.0, 2, 0, 10, 5);

      constrain(panel1, new Label("Table to use (total "+Option.Num_Tables()
                +")"), 0, 3, 1, 1);
      constrain(panel1, table_choice, 0, 4, 2, 1, 
                GridBagConstraints.NONE, 
                GridBagConstraints.NORTHEAST, 1.0, 1.0, 2, 0, 5, 5);


      constrain(panel2, show_info_cb, 0, 0, 1, 1, GridBagConstraints.NONE, 
                GridBagConstraints.NORTHWEST, 1.0, 0.0, 5, 5, 5, 5);
      constrain(panel2, show_error_cb, 0, 1, 1, 1, GridBagConstraints.NONE,
                GridBagConstraints.NORTHWEST, 1.0, 0.0, 5, 5, 5, 5);
      constrain(panel2, show_debug_cb, 0, 2, 1, 1, GridBagConstraints.NONE, 
                GridBagConstraints.NORTHWEST, 1.0, 0.0, 5, 5, 5, 5);
      constrain(panel2, show_route_cb, 0, 3, 1, 1, GridBagConstraints.NONE, 
                GridBagConstraints.NORTHWEST, 1.0, 0.0, 5, 5, 5, 5);

      constrain(panel2, show_message_cb, 1, 0, 1, 1, GridBagConstraints.NONE, 
                GridBagConstraints.NORTHWEST, 1.0, 0.0, 5, 5, 5, 5);
      constrain(panel2, show_ctrl_msg_cb, 1, 1, 1, 1, GridBagConstraints.NONE, 
                GridBagConstraints.NORTHWEST, 1.0, 0.0, 5, 5, 5, 5);
      constrain(panel2, display_cli_cb, 1, 2, 1, 1, GridBagConstraints.NONE, 
                GridBagConstraints.NORTHEAST, 1.0, 0.0, 5, 5, 5, 5);
      constrain(panel2, display_gui_cb, 1, 3, 1, 1, GridBagConstraints.NONE,
                GridBagConstraints.NORTHEAST, 1.0, 0.0, 5, 5, 5, 5);

      constrain(panel3, new Label("Failures: "), 0, 0, 1, 1);
      constrain(panel3, f_never_cb, 0, 1, 1, 1, 
                GridBagConstraints.NONE,
                GridBagConstraints.NORTHWEST, 0.0, 1.0, 2, 5, 2, 0);
      constrain(panel3, f_always_cb, 0, 2, 1, 1, 
                GridBagConstraints.NONE,
                GridBagConstraints.NORTHWEST, 0.0, 1.0, 2, 5, 2, 0);
      constrain(panel3, f_prompt_cb, 0, 3, 1, 1, 
                GridBagConstraints.NONE,
                GridBagConstraints.NORTHWEST, 0.0, 1.0, 2, 5, 2, 0);
      constrain(panel3, f_inter_cb, 0, 4, 1, 1, 
                GridBagConstraints.NONE,
                GridBagConstraints.NORTHWEST, 0.0, 1.0, 2, 5, 2, 0);
      constrain(panel3, f_states_cb, 0, 5, 1, 1, 
                GridBagConstraints.NONE,
                GridBagConstraints.NORTHWEST, 0.0, 1.0, 2, 5, 2, 0);
      constrain(panel3, f_prob_cb, 0, 6, 1, 1, 
                GridBagConstraints.NONE,
                GridBagConstraints.NORTHWEST, 0.0, 1.0, 2, 5, 2, 0);

      constrain(panel4, new Label("Select:"), 0, 0, 1, 1);
      constrain(panel4, new Label("Msg Interval"), 0, 1, 1, 1);
      constrain(panel4, f_inter_choice, 1, 1, 1, 1, 
                GridBagConstraints.NONE,
                GridBagConstraints.NORTHEAST, 1.0, 1.0, 5, 5, 10, 5);
      constrain(panel4, new Label("In State"), 0, 2, 1, 1);
      constrain(panel4, new Label("No. To Fail"), 1, 2, 1, 1,
                GridBagConstraints.NONE,
                GridBagConstraints.NORTHEAST, 0.0, 0.0, 0, 0, 0, 0);
      constrain(panel4, f_states_choice, 0, 3, 1, 1, 
                GridBagConstraints.NONE,
                GridBagConstraints.NORTHWEST, 1.1, 1.0, 0, 0, 10, 5);
      constrain(panel4, f_num_choice, 1, 3, 1, 1, 
                GridBagConstraints.NONE,
                GridBagConstraints.NORTHEAST, 1.1, 1.0, 0, 5, 10, 5);
      constrain(panel4, new Label("Prob. Dist."), 0, 4, 1, 1);
      constrain(panel4, f_dist_choice, 1, 4, 1, 1, 
                GridBagConstraints.NONE,
                GridBagConstraints.NORTHEAST, 1.0, 1.0, 5, 5, 5, 5);


      constrain(panel5, new Label("Recovery: "), 0, 0, 1, 1);
      constrain(panel5, r_always_cb, 0, 1, 1, 1, 
                GridBagConstraints.NONE,
                GridBagConstraints.NORTHWEST, 0.0, 1.0, 2, 5, 2, 0);
      constrain(panel5, r_never_cb, 0, 2, 1, 1, 
                GridBagConstraints.NONE,
                GridBagConstraints.NORTHWEST, 0.0, 1.0, 2, 5, 2, 0);
      constrain(panel5, r_prompt_cb, 0, 3, 1, 1, 
                GridBagConstraints.NONE,
                GridBagConstraints.NORTHWEST, 0.0, 1.0, 2, 5, 2, 0);
      constrain(panel5, r_inter_cb, 0, 4, 1, 1, 
                GridBagConstraints.NONE,
                GridBagConstraints.NORTHWEST, 0.0, 1.0, 2, 5, 2, 0);
      constrain(panel5, r_prob_cb, 0, 5, 1, 1, 
                GridBagConstraints.NONE,
                GridBagConstraints.NORTHWEST, 0.0, 1.0, 2, 5, 2, 0);

      constrain(panel6, new Label("Select:"), 0, 0, 1, 1);
      constrain(panel6, new Label("Msg Interval"), 0, 1, 1, 1);
      constrain(panel6, r_inter_choice, 1, 1, 1, 1, 
                GridBagConstraints.NONE,
                GridBagConstraints.NORTHEAST, 1.0, 1.0, 5, 5, 10, 5);
      constrain(panel6, new Label("Prob. Dist."), 0, 2, 1, 1);
      constrain(panel6, r_dist_choice, 1, 2, 1, 1, 
                GridBagConstraints.NONE,
                GridBagConstraints.NORTHEAST, 1.0, 1.0, 5, 5, 10, 5);


      constrain(panelb, done_b, 0, 0, 1, 1, 
                GridBagConstraints.NONE, 
                GridBagConstraints.CENTER, 0.3, 0.0, 0, 0, 0, 0);


      constrain(this, panel1, 0, 0, 1, 1, GridBagConstraints.BOTH, 
                GridBagConstraints.NORTHWEST, 1.0, 1.0, 10, 10, 10, 20);
      constrain(this, panel2, 0, 1, 1, 1, GridBagConstraints.HORIZONTAL, 
                GridBagConstraints.NORTHWEST, 1.0, 0.0, 10, 10, 10, 20);
      constrain(this, panel3, 1, 0, 1, 1, GridBagConstraints.BOTH, 
                GridBagConstraints.NORTHWEST, 1.0, 1.0, 10, 10, 10, 10);
      constrain(this, panel4, 2, 0, 1, 1, GridBagConstraints.BOTH, 
                GridBagConstraints.NORTHWEST, 1.0, 1.0, 10, 10, 10, 10);
      constrain(this, panel5, 1, 1, 1, 2, GridBagConstraints.BOTH, 
                GridBagConstraints.NORTHWEST, 1.0, 1.0, 10, 10, 10, 10);
      constrain(this, panel6, 2, 1, 1, 2, GridBagConstraints.BOTH, 
                GridBagConstraints.NORTHWEST, 1.0, 1.0, 10, 10, 10, 10);
      constrain(this, panelb, 0, 2, 1, 1, GridBagConstraints.HORIZONTAL, 
                GridBagConstraints.CENTER, 1.0, 0.0, 10, 10, 10, 10);

      if (!SIMANIM) {
          Global.Enable_Event(show_message_cb, false);
          Global.Enable_Event(show_ctrl_msg_cb, false);
      }

      if (RELEASEv) {
          Global.Enable_Event(show_debug_cb, false);
          Global.Enable_Event(show_route_cb, false);
      }

      Global.Enable_Event(f_prompt_cb, false);
      Global.Enable_Event(r_prompt_cb, false);
      Global.Enable_Event(f_prob_cb, false);
      Global.Enable_Event(r_prob_cb, false);

      this.Enable_F_Panel(Option.F_Method());
      this.Enable_R_Panel(Option.R_Method());

  }                                                   // void Setup_Options(0)


  public void actionPerformed(ActionEvent event) {
      // Method to handle Action events i.e. buttons, lists
      // This uses JDK 1.1 methods
      Object source = event.getSource();
 
      if (source == done_b) {
          Debug("Done button");
          Global.Window_Visible(this, false);
          this.sim_applet.Do_Relayout();
      }
      else if (source == applet_b) {
          Debug("Applet Option button");
          Global.Window_Visible(this, true);
      }
  }                                                 // void actionPerformed(1)

  public void itemStateChanged(ItemEvent event) {
      // Method to handle Item events i.e. checkboxes, choices
      // This uses JDK 1.1 methods
      int    pos, num, state;
      Object source = event.getSource();

      if (source == num_sites_choice) {                          // Num Sites

          num = 1 + num_sites_choice.getSelectedIndex();
          Option.Set_Num_Sites(num);
          pos = coord_site_choice.getSelectedIndex();
          Global.Update_Choice_Index(coord_site_choice, num, 1);
          if ((pos+1) > num) {
               Option.Set_Coord_Site(num);
               coord_site_choice.select(Option.Coord_Site() - 1);
               Debug("Coord site: "+Option.Coord_Site());
          }
          pos = num_abort_choice.getSelectedIndex();
          Global.Update_Choice_Index(num_abort_choice, num, 0);
          if (pos > num) {
              Option.Set_Num_Abort(num);
              num_abort_choice.select(Option.Num_Abort());
              Debug("Num abort: "+Option.Num_Abort());
          }
          Debug("Num sites choice - now "+Option.Num_Sites());
          pos = f_num_choice.getSelectedIndex();
          Global.Update_Choice_Index(f_num_choice, num, 0);
          if (pos > num) {
              state = 1 + f_states_choice.getSelectedIndex();
              Debug("          before - state "+state
                    +", num "+Option.Num_To_Fail(state));
              Option.Set_Num_To_Fail(num, state);
              f_num_choice.select(Option.Num_To_Fail(state));
              Debug("           after - state "+state
                    +", num "+Option.Num_To_Fail(state));
          }
      }
      else if (source == coord_site_choice) {                   // Coord site
          num = 1 + coord_site_choice.getSelectedIndex();
          Option.Set_Coord_Site(num);
          Debug("Coord site choice - now "+Option.Coord_Site());
      }
      else if (source == num_abort_choice) {                     // Num abort
          num = num_abort_choice.getSelectedIndex();
          Option.Set_Num_Abort(num);
          Debug("Num abort choice - now "+Option.Num_Abort());
      }
      else if (source == table_choice) {                             // Table
          num = 1 + table_choice.getSelectedIndex();
          Option.Set_Table_To_Use(num);
          this.Update_States_Choice();     
          Debug("Table choice - now "+Option.Table().Name());
      }
      else if (source == f_inter_choice) {                   // fail Interval
          num = f_inter_choice.getSelectedIndex();
          Option.Set_F_Interval(num);
          Debug("Fail INTERVAL choice - now "+Option.F_Interval());
      }
      else if (source == f_states_choice) {                    // fail States
          state = 1 + f_states_choice.getSelectedIndex();
          num = Option.Num_To_Fail(state);
          f_num_choice.select(num);
          if (Option.Table().Fail_Possible(PART, state)) {
              Global.Enable_Event(f_num_choice, true);
              Debug("Fail STATES choice - state "+state
                    +", num "+num+" - possible");
          }
          else {
              Global.Enable_Event(f_num_choice, false);
              Debug("Fail STATES choice - state "+state
                    +", num "+num+" - not possible");
          } 
      }
      else if (source == f_num_choice) {                          // fail Num
          state = 1 + f_states_choice.getSelectedIndex();
          num = f_num_choice.getSelectedIndex();
          Option.Set_Num_To_Fail(num, state);
          Debug("Fail NUM choice - now "+Option.Num_To_Fail(state)
                +", state "+state);
      }
      else if (source == f_dist_choice) {                        // fail Dist 
          num = f_dist_choice.getSelectedIndex();
          Pr_Dist.Set_Pr_Dist_Failure(num);
          Debug("Fail DIST choice");
      }
      else if (source == r_inter_choice) {                // recover Interval
          num = r_inter_choice.getSelectedIndex();
          Option.Set_R_Interval(num);
          Debug("Recover INTERVAL choice - now "+Option.R_Interval());
      }
      else if (source == r_dist_choice) {                     // recover Dist
          num = r_dist_choice.getSelectedIndex();
          Pr_Dist.Set_Pr_Dist_Recovery(num);
          Debug("Recover DIST choice");
      }
      else if (source == show_info_cb) {                         // show INFO
          Debug("Show INFO checkbox");
          Option.Set_Show_Info(show_info_cb.getState());
          if (!Option.Show_Info()) {
              Option.Set_Show_Debug(false);
              Option.Set_Show_Routing(false);
              show_debug_cb.setState(false);
              show_route_cb.setState(false);
          }
      }
      else if (source == show_error_cb) {                       // show ERROR
          Debug("Show ERROR checkbox");
          if (RELEASEv) {
              Option.Set_Show_Error(true);
              show_error_cb.setState(true);
	  }
          Option.Set_Show_Error(show_error_cb.getState());
      }
      else if (source == show_debug_cb) {                       // show DEBUG
          Debug("Show DEBUG checkbox");
          Option.Set_Show_Debug(show_debug_cb.getState());
          if (show_debug_cb.getState()) {
              show_info_cb.setState(true);
              show_error_cb.setState(true);
              Option.Set_Show_Info(true);
              Option.Set_Show_Error(true);
          }
          else {
              show_route_cb.setState(false);
              Option.Set_Show_Routing(false);
          }
      }
      else if (source == show_route_cb) {                       // show ROUTE
          Debug("Show ROUTE checkbox");
          Option.Set_Show_Routing(show_route_cb.getState());
          if (show_route_cb.getState()) {
              show_debug_cb.setState(true);
              show_info_cb.setState(true);
              show_error_cb.setState(true);
              Option.Set_Show_Debug(true);
              Option.Set_Show_Info(true);
              Option.Set_Show_Error(true);
          }
      }
      else if (source == show_message_cb) {                   // show MESSAGE
          Debug("Show MESSAGE checkbox");
          Option.Set_Show_Messages(show_message_cb.getState());
          if (!Option.Show_Messages()) {
              Option.Set_Show_Ctrl_Messages(false);
              show_ctrl_msg_cb.setState(false);
          }
      }
      else if (source == show_ctrl_msg_cb) {              // show CONTROL MSG
          Debug("Show CONTROL MESSAGE checkbox");
          Option.Set_Show_Ctrl_Messages(show_ctrl_msg_cb.getState());
          if (Option.Show_Ctrl_Messages()) {
              Option.Set_Show_Messages(true);
              show_message_cb.setState(true);
          }
      }
      else if (source == display_cli_cb) {                     // display CLI
          Debug("Display CLI checkbox");
          Option.Set_Display_CLI(display_cli_cb.getState());
      }
      else if (source == display_gui_cb) {                     // display GUI
          Debug("Display GUI checkbox");
          Option.Set_Display_GUI(display_gui_cb.getState());
      }
      else if (source == f_never_cb) {                          // fail NEVER
          Debug("Fail NEVER checkbox");
          Option.Set_F_Method(NEVER);
          this.Enable_F_Panel(NEVER);
      }
      else if (source == f_always_cb) {                        // fail ALWAYS
          Debug("Fail ALWAYS checkbox");
          Option.Set_F_Method(ALWAYS);
          this.Enable_F_Panel(ALWAYS);
      }
      else if (source == f_prompt_cb) {                        // fail PROMPT
          Debug("Fail PROMPT checkbox");
          Option.Set_F_Method(PROMPT);
          this.Enable_F_Panel(PROMPT);
      }
      else if (source == f_inter_cb) {                       // fail INTERVAL
          Debug("Fail INTERVAL checkbox");
          Option.Set_F_Method(INTERVAL);
          this.Enable_F_Panel(INTERVAL);
      }
      else if (source == f_states_cb) {                        // fail STATES
          Debug("Fail STATES checkbox");
          Option.Set_F_Method(STATES);
          this.Enable_F_Panel(STATES);
      }
      else if (source == f_prob_cb) {                          // fail RANDOM
          Debug("Fail RANDOM checkbox");
          Option.Set_F_Method(RANDOM);
          this.Enable_F_Panel(RANDOM);
      }
      else if (source == r_always_cb) {                     // recover ALWAYS
          Debug("Recover ALWAYS checkbox");
          Option.Set_R_Method(ALWAYS);
          this.Enable_R_Panel(ALWAYS);
      }
      else if (source == r_never_cb) {                       // recover NEVER
          Debug("Recover NEVER checkbox");
          Option.Set_R_Method(NEVER);
          this.Enable_R_Panel(NEVER);
      }
      else if (source == r_prompt_cb) {                     // recover PROMPT
          Debug("Recover PROMPT checkbox");
          Option.Set_R_Method(PROMPT);
          this.Enable_R_Panel(PROMPT);
      }
      else if (source == r_inter_cb) {                    // recover INTERVAL
          Debug("Recover INTERVAL checkbox");
          Option.Set_R_Method(INTERVAL);
          this.Enable_R_Panel(INTERVAL);
      }
      else if (source == r_prob_cb) {                       // recover RANDOM
          Debug("Recover RANDOM checkbox");
          Option.Set_R_Method(RANDOM);
          this.Enable_R_Panel(RANDOM);
      }
  }                                                // void itemStateChanged(1)


  public void focusGained(FocusEvent event) {
      // Method to handle event of getting focus
      // This uses JDK 1.1 methods
      Debug("Gained FOCUS");
      panelb.requestFocus();
  }                                                     // void focusGained(1)

  public void focusLost(FocusEvent event) {
      // Method to handle event of losing focus
      // This uses JDK 1.1 methods
      // Do nothing
      Debug("Lost FOCUS");
  }                                                       // void focusLost(1)
  

  private void Update_States_Choice() {
      // Method to update the choice of states depending on table currently 
      // selected
      int i, num, state = 0;
      int old_indices = 0;

      old_indices = Global.Choice_Index_Count(f_states_choice);

      for (i=old_indices; i > 0; i--) {
  	  // Remove existing state name items
          Debug("indices: "+old_indices+", remove: "+(i-1));   
          f_states_choice.remove(i-1);
      }

      for (i=1; i <= Option.Table().Total_States(PART); i++) {
  	  // Add new state names
          this.f_states_choice.addItem(i+": "
                                       +Option.Table().State_Name(PART, i));
      }

      Option.Reset_Num_To_Fail();
      state = Option.First_State_Choice();
      num = Option.Num_To_Fail( state );
      try {
        f_states_choice.select( state - 1);
        f_num_choice.select( num );
      } catch (IllegalArgumentException e) {
          Scheduler.Error_Pub("Option selection threw exception");
      }
      if (Option.Table().Fail_Possible(PART, state)) {
          Global.Enable_Event(f_num_choice, true);
          Debug("Fail STATES choice - state "+state
                 +", num "+num+" - possible");
      }
      else {
          Global.Enable_Event(f_num_choice, false);
          Debug("Fail STATES choice - state "+state
                +", num "+num+" - not possible");
      }      
  }                                            // void Update_States_Choice(0)


  private void Enable_F_Panel(int method) {
      // Method to enable the events in the Failure panel required
      int state;

      if ((method == NEVER) || (method == ALWAYS) || (method == PROMPT)) {
          Global.Enable_Event(panel4, false);
          Global.Enable_Event(f_inter_choice, false);
          Global.Enable_Event(f_states_choice, false);
          Global.Enable_Event(f_num_choice, false);
          Global.Enable_Event(f_dist_choice, false);
          Global.Enable_Event(panel4, false);
      }
      else if (method == INTERVAL) {
          Global.Enable_Event(panel4, true);
          Global.Enable_Event(f_inter_choice, true);
          Global.Enable_Event(f_states_choice, false);
          Global.Enable_Event(f_num_choice, false);
          Global.Enable_Event(f_dist_choice, false);
      }
      else if (method == STATES) {
          Global.Enable_Event(panel4, true);
          Global.Enable_Event(f_inter_choice, false);
          Global.Enable_Event(f_states_choice, true);
          state = 1 + f_states_choice.getSelectedIndex();
          if (Option.Table().Fail_Possible(PART, state)) {
              Global.Enable_Event(f_num_choice, true);
          }
          else {
              Global.Enable_Event(f_num_choice, false);
	  }
          Global.Enable_Event(f_dist_choice, false);
      }
      else if (method == RANDOM) {
          Global.Enable_Event(panel4, true);
          Global.Enable_Event(f_inter_choice, false);
          Global.Enable_Event(f_states_choice, false);
          Global.Enable_Event(f_num_choice, false);
          Global.Enable_Event(f_dist_choice, true);
      }
  }                                                  // void Enable_F_Panel(1)

  private void Enable_R_Panel(int method) {
      // Method to enable the events in the Recover panel required
      if ((method == ALWAYS) || (method == NEVER) || (method == PROMPT)) {
          Global.Enable_Event(r_inter_choice, false);
          Global.Enable_Event(r_dist_choice, false);
          Global.Enable_Event(panel6, false);
      }
      else if (method == INTERVAL) {
          Global.Enable_Event(panel6, true);
          Global.Enable_Event(r_inter_choice, true);
          Global.Enable_Event(r_dist_choice, false);
      }
      else if (method == RANDOM) {
          Global.Enable_Event(panel6, true);
          Global.Enable_Event(r_inter_choice, false);
          Global.Enable_Event(r_dist_choice, true);
      }
  }                                                  // void Enable_R_Panel(1)

  private void Debug(String note) {
      // Method to output debugging info to CLI
      Global.Debug_CLI("Event - "+note);
  }                                                           // void Debug(1)

  /* The following constrain methods are from "Java in a Nutshell" Section 5
  ** examples on GUIs.  Used to position components in containers.
  */  
  private void constrain(Container container, Component component, 
                        int grid_x, int grid_y, int grid_w, int grid_h,
                        int fill, int anchor, 
                        double weight_x, double weight_y,
                        int top, int left, int bottom, int right) 
  {

      GridBagConstraints c = new GridBagConstraints();
      c.gridx = grid_x; 
      c.gridy = grid_y;
      c.gridwidth = grid_w; 
      c.gridheight = grid_h;
      c.fill = fill; 
      c.anchor = anchor;
      c.weightx = weight_x; 
      c.weighty = weight_y;

      if (top + bottom + left + right > 0) {
          c.insets = new Insets(top, left, bottom, right);
      }
      ((GridBagLayout) container.getLayout()).setConstraints(component, c);
      container.add(component);
  }                                                      // void constrain(14)
    
  private void constrain(Container container, Component component, 
                         int grid_x, int grid_y, int grid_w, int grid_h) 
  {
      constrain(container, component, grid_x, grid_y, grid_w, grid_h, 
                GridBagConstraints.NONE, GridBagConstraints.NORTHWEST, 
                0.0, 0.0, 0, 0, 0, 0);
  }                                                       // void constrain(6)
    
}                                                               // class O_Box

/*------------------------------- End of O_Box.java -------------------------*/

