
/*
** Author : Steven Ramsay <smr@dcs.ed.ac.uk>, 01/06/97
** Files  : Option.java, Option.class
** Update : 11/06/97 
** Package: simdcp
**
** Description:
** All options which can be set and are required in all classes.
*/

package eduni.simdcp;

import eduni.simanim.*;

/* Methods defined in class Global
** PUBLIC
** -- no constructor -- 
**
** PROTECTED
** static void    Set_Num_Sites(int)
** static int     Num_Sites()
** static void    Set_Num_Abort(int)
** static int     Num_Abort()
** static void    Set_Coord_Site(int)
** static int     Coord_Site()
** static void    Set_Table_To_Use(int)
** static int     Table_To_Use()
** static int     Num_Tables()
** static void    Set_Display_CLI(boolean)
** static boolean Display_CLI()
** static void    Set_Display_GUI(boolean)
** static boolean Display_GUI()
** static void    Set_Show_Debug(boolean)
** static boolean Show_Debug()
** static void    Set_Show_Error(boolean)
** static boolean Show_Error()
** static void    Set_Show_Info(boolean)
** static boolean Show_Info()
** static boolean Show_Routing()
** static void    Set_Show_Routing(boolean)
** static boolean Show_Messages()
** static void    Set_Show_Messages(boolean)
** static boolean Show_Ctrl_Messages()
** static void    Set_Show_Ctrl_Messages(boolean)
** static boolean Show_DTLog()
** static boolean Show_Table()
** static boolean Show_Trace()
** static void    Set_F_Method(int)
** static int     F_Method()
** static void    Set_R_Method(int)
** static int     R_Method()
** static boolean Fail(int)
** static boolean Recover(int)
** static void    Set_F_Interval(int)
** static int     F_Interval()
** static void    Set_R_Interval(int)
** static int     R_Interval()
** static void    Set_Num_To_Fail(int, int)
** static void    Reset_Num_To_Fail()
** static int     Num_To_Fail(int)
** static int     First_State_Choice()
** static Table   Table()
** static Table   Table(int)
** static int     Total_Tables()
**
** SIM_ANIM methods
** PROTECTED
** static int     Sched_x()
** static int     Sched_y()
** static int     Filter_x(int)
** static int     Filter_y(int)
** static int     Coord_x(int)
** static int     Coord_y(int)
** static int     Part_x(int)
** static int     Part_y(int)
** static int     Port_Side(int, int, int) 
** static int     Port_Pos(int, int)   
** static void    Set_Radius(int)
** static int     Radius()
** static int     Off_x(int)
** static int     Off_y(int)
** static int     State_Pos_x(int, int)
** static int     State_Pos_y(int, int)
** static int     Id_Pos_x(int)
** static int     Id_Pos_y(int)
**
** PRIVATE
** SIM_ANIM methods
** static int     Center_x()
** static int     Center_y()
** static double  Angle(int)
** static int     Site_Origin_x()
** static int     Site_Origin_y()
** static int     Ports_On_Side_SCHED(int)
** static int     Port_Number_SCHED(int)
*/
class Option implements Constants {
  // Class for options methods and variables

  // Initial options - DEFAULT VALUES - MAY be changed
  private static int NUM_OF_SITES = 2;                 // No of sites in model
  private static int COORD_SITE   = 1;                  // Site of first Coord
  private static int NUM_TO_ABORT = 0;            // No of sites to vote abort
  private static int TABLE_TO_USE = 1;               // Number of table to use

  private static boolean DISPLAY_CLI = true;
  private static boolean DISPLAY_GUI = false;

  private static boolean SHOW_INFO   = true;
  private static boolean SHOW_ERROR  = true;
  private static boolean SHOW_DEBUG  = false;
  private static boolean SHOW_ROUTE  = false;

  private static boolean SHOW_MESSAGES  = true;
  private static boolean SHOW_CTRL_MSGS = true;

  private static int     FAILURE_METHOD = NEVER;
  private static int     RECOVER_METHOD = ALWAYS;

  private static int     FAILURE_INTERVAL = MAX_INTERVAL;
  private static int     RECOVER_INTERVAL = 0;
  

  // Sim Anim Options ...
  private static int     RADIUS = 75 + (SCHED_h / 2);

  // ... End of Sim Anim Options


  // The following are not available from options window
  private static boolean SHOW_DTLOG         = true;
  private static boolean SHOW_TABLE         = false;
  private static boolean SHOW_TRACE         = true;
  private static int     FIRST_STATE_CHOICE = 1;


  // *********** DO NOT CHANGE BELOW THIS LINE ***********
  private static Table[] tables;                       // Current table in use
  private static int     total_tables = 0;           // Total number of tables 

  // Only used if method is to fail in specified States
  private static int[]   NUM_TO_FAIL     = new int[MAX_STATES];

  static {
      int i=0; 
      // Add tables to simulation
      tables    = new Table[MAX_TABLES + 1];
      tables[1] = new Table_2PC("2-Phase Commit (2PC)"); 
      tables[2] = new Table_3PC("3-Phase Commit (3PC)");
      total_tables = 2;

      if (Show_Table()) {
  	  while (i <= MAX_TABLES) {
  	      if (tables[i] != null) {
                  tables[i].Display();
	      }
              i++;
          }
      }
  }                                                                  // static


  protected static void Set_Num_Sites(int num) {
      // Method to set the number of sites in txn.
      if (num >= 1) {
	  if (num <= MAX_SITES) {
              NUM_OF_SITES = num;
          }
          else { 
              NUM_OF_SITES = MAX_SITES;
          }
      }
      else {
          NUM_OF_SITES = 1;
      }     
  }                                                   // void Set_Num_Sites(1)

  protected static int Num_Sites() {  
      // Method to return the no. of sites being modelled.
      // Each site will have a participant and one will also share a 
      // coordinator.
      int site_num = NUM_OF_SITES;

      if (site_num < 1) {
          site_num = 1;
      }
      else if (site_num > MAX_SITES) {
          site_num = MAX_SITES;
      }
      return site_num;
  }                                                        // int Num_Sites(0)

  protected static void Set_Num_Abort(int num) {
      // Method to set the number of participants to abort in txn.
      if (num >= 0) {
  	  if (num <= Num_Sites()) {
              NUM_TO_ABORT = num;
          }
          else {
              NUM_TO_ABORT = Num_Sites();
          }
      }
      else {
          NUM_TO_ABORT = 0;
      }
  }                                                   // void Set_Num_Abort(1)

  protected static int Num_Abort() {  
      // Method to return the no. of Participants who will abort.
      int num_abort = NUM_TO_ABORT;
      if (num_abort < 0) {
          num_abort = 0;
      }
      else if (num_abort > Num_Sites()) {
          num_abort = Num_Sites();
      }
      return num_abort;
  }                                                        // int Num_Abort(0)

  protected static void Set_Coord_Site(int id) {  
      // Method to set the site id of first Coordinator
      if (id >= 1) {
          if (id <= Num_Sites()) {
              COORD_SITE = id;
          }
          else {
              COORD_SITE = Num_Sites();
	  }
      }
      else {
          COORD_SITE = 1;
      }
  }                                                  // void Set_Coord_Site(1)

  protected static int Coord_Site() {  
      // Method to return the site id of first Coordinator
      int coord_id = COORD_SITE;
      if (coord_id < 1) {
          coord_id = 1;
      }
      else if (coord_id > Num_Sites()) {
          coord_id = Num_Sites();
      }
      return coord_id;
  }                                                       // int Coord_Site(0)

  protected static void Set_Table_To_Use(int num) {  
      // Method to set the number of table to use
      if ((num >= 1) && (num <= total_tables)) {
          TABLE_TO_USE = num;
      }
      else {
          TABLE_TO_USE = 1;
      }
  }                                                    // void Table_To_Use(1)

  protected static int Table_To_Use() {  
      // Method to return the number of table to use
      int table_num = TABLE_TO_USE;
      if ((TABLE_TO_USE < 1) && (TABLE_TO_USE > total_tables)) {
          table_num = 1;
      }
      return table_num;
  }                                                     // int Table_To_Use(0)


  protected static int Num_Tables() {
      // Method to return the total number of tables to use
      return total_tables;
  }                                                       // int Num_Tables(0)

  protected static void Set_Display_CLI(boolean bool) {  
      // Method to set whether the display is sent to CLI
      DISPLAY_CLI = bool;
  }                                                 // void Set_Display_CLI(1)

  protected static boolean Display_CLI() {  
      // Method to return true to display to command line interface
      return DISPLAY_CLI;
  }                                                  // boolean Display_CLI(0)

  protected static void Set_Display_GUI(boolean bool) {  
      // Method to set whether the display is sent to GUI
      DISPLAY_GUI = bool;
  }                                                 // void Set_Display_GUI(1)

  protected static boolean Display_GUI() {  
      // Method to return true to display graphical user interfaces
      return DISPLAY_GUI;
  }                                                  // boolean Display_GUI(0)

  protected static void Set_Show_Debug(boolean bool) {  
      // Method to set whether the debugging info is displayed
      SHOW_DEBUG = bool;
  }                                                  // void Set_Show_Debug(1)

  protected static boolean Show_Debug() {  
      // Method to return true to output debugging information
      return SHOW_DEBUG;
  }                                                   // boolean Show_Debug(0)

  protected static void Set_Show_Error(boolean bool) {  
      // Method to set whether the error info is displayed
      SHOW_ERROR = bool;
  }                                                  // void Set_Show_Error(1)

  protected static boolean Show_Error() {  
      // Method to return true to output error warnings
      return SHOW_ERROR;
  }                                                   // boolean Show_Error(0)

  protected static void Set_Show_Info(boolean bool) {  
      // Method to set whether the necessary info is displayed
      SHOW_INFO = bool;
  }                                                   // void Set_Show_Info(1)

  protected static boolean Show_Info() {  
      // Method to return true to output general information
      return SHOW_INFO;
  }                                                    // boolean Show_Info(0)

  protected static void Set_Show_Routing(boolean bool) {  
      // Method to set whether the routing info is displayed
      SHOW_ROUTE = bool;
  }                                                // void Set_Show_Routing(1)

  protected static boolean Show_Routing() {  
      // Method to return true to output routing debugging info
      boolean routing = SHOW_ROUTE;
      if (!Show_Debug()) {
          routing = false;
      } 
      return routing;
  }                                                 // boolean Show_Routing(0)

  protected static void Set_Show_Messages(boolean bool) {  
      // Method to set if messages should be displayed in SimAnim
      SHOW_MESSAGES = bool;
  }                                               // void Set_Show_Messages(1)

  protected static boolean Show_Messages() {  
      // Method to return true to display messages in SimAnim
      boolean show = SHOW_MESSAGES;
      if (!SIMANIM) {
          show = false;
      }
      return show;
  }                                                // boolean Show_Messages(0)

  protected static void Set_Show_Ctrl_Messages(boolean bool) {  
      // Method to set if control messages should be displayed in SimAnim
      SHOW_CTRL_MSGS = bool;
  }                                          // void Set_Show_Ctrl_Messages(1)

  protected static boolean Show_Ctrl_Messages() {  
      // Method to return true to display control messages in SimAnim
      boolean show = SHOW_CTRL_MSGS;
      if ((!SIMANIM) || (!Show_Messages())) {
          show = false;
      }
      return show;
  }                                           // boolean Show_Ctrl_Messages(0)

  protected static boolean Show_DTLog() {  
      // Method to return true to output the log entries
      return SHOW_DTLOG;
  }                                                   // boolean Show_DTLog(0)

  protected static boolean Show_Table() {  
      // Method to return true to output the tables
      return SHOW_TABLE;
  }                                                   // boolean Show_Table(0)

  protected static boolean Show_Trace() {  
      // Method to return true to output trace information
      return SHOW_TRACE;
  }                                                   // boolean Show_Trace(0)

 
  protected static void Set_F_Method(int method) {
      // Method to set the failure method
      if (   (method == NEVER) || (method == ALWAYS) 
          || (method == PROMPT) || (method == INTERVAL) 
          || (method == STATES) || (method == RANDOM)) {
         FAILURE_METHOD = method;
      }
  }                                                    // void Set_F_Method(1)

  protected static int F_Method() {
      // Method to return the failure method
      return FAILURE_METHOD;
  }                                                         // int F_Method(0)

  protected static void Set_R_Method(int method) {
      // Method to set the recovery method
      if (   (method == ALWAYS) || (method == NEVER) 
          || (method == PROMPT) || (method == INTERVAL) 
          || (method == RANDOM)) {
         RECOVER_METHOD = method;
      }
  }                                                    // void Set_R_Method(1)

  protected static int R_Method() {
      // Method to return the recovery method
      return RECOVER_METHOD;
  }                                                         // int R_Method(0)

  protected static boolean Fail(int method) {
      // Method to return true if the method is the failure method
      boolean bool = false;

      if (FAILURE_METHOD == method) {
          bool = true;
      }
      return bool;
  }                                                         // boolean Fail(1)

  protected static boolean Recover(int method) {
      // Method to return true if the method is the recover method
      boolean bool = false;

      if (RECOVER_METHOD == method) {
          bool = true;
      }
      return bool;
  }                                                      // boolean Recover(1)

  protected static void Set_F_Interval(int num) {
      // Method to set the interval in messages between failures.
      if ((num >= 0) && (num <= MAX_INTERVAL)) {
          FAILURE_INTERVAL = num;
      }
      else {
          FAILURE_INTERVAL = MAX_INTERVAL;
      }
  }                                                  // void Set_F_Interval(1)

  protected static int F_Interval() {
      // Method to return the interval in messages between failures.
      int f_inter = FAILURE_INTERVAL;
      if ((f_inter < 0) || (f_inter > MAX_INTERVAL)) {
          f_inter = MAX_INTERVAL;
      }
      return f_inter;
  }                                                       // int F_Interval(0)

  protected static void Set_R_Interval(int num) {
      // Method to set the interval in messages between recovery.
      if ((num >= 0) && (num <= MAX_INTERVAL)) {
          RECOVER_INTERVAL = num;
      }
      else {
          RECOVER_INTERVAL = 0;
      }
  }                                                  // void Set_R_Interval(1)

  protected static int R_Interval() {
      // Method to return the interval in messages between recovery.
      int r_inter = RECOVER_INTERVAL;
      if ((r_inter < 0) || (r_inter > MAX_INTERVAL)) {
          r_inter = 0;
      }
      return r_inter;
  }                                                       // int R_Interval(0)


  protected static void Set_Num_To_Fail(int num, int state) {
      // Method to set the number of participants to fail in a state.

      if ((state > 0) && (state <= Table().Total_States(PART))) {
          if (Table().Fail_Possible(PART, state)) {
             if (num >= 0) {
                 NUM_TO_FAIL[state] = num;
             }
          }
          else {
             NUM_TO_FAIL[state] = 0;
          }
      }
  }                                                 // void Set_Num_To_Fail(2)

  protected static void Reset_Num_To_Fail() {
      // Method to reset the number of participants to fail in all states.
      NUM_TO_FAIL = new int[MAX_STATES];
  }                                               // void Reset_Num_To_Fail(0)

  protected static int Num_To_Fail(int state) {
      // Method to return the number of participants to fail in a state.
      int num = 0;

      if ((state > 0) && (state <= Table().Total_States(PART))) {
          if (Table().Fail_Possible(PART, state)) {
              num = NUM_TO_FAIL[state];
          }
      }
      if (num < 0) {
          num = 0;
      }
      else if (num > Num_Sites()) {
          num = Num_Sites();
      }
      return num;
  }                                                      // int Num_To_Fail(1)

  protected static int First_State_Choice() {
      // Method to return the first state used in options choice
      int state = FIRST_STATE_CHOICE;

      if ((state < 0) || (state > Option.Table().Total_States(PART))) {
          state = 1;
      }
      return state;
  }                                               // int First_State_Choice(0)

  protected static Table Table() {
      // Method to return the table of the current protocol
      Table table;
      table = tables[Table_To_Use()];

      return table;
  }                                                          // Table Table(0)

  protected static Table Table(int i) {
      // Method to return the table for the protocol specified
      Table table = null;
      if ((i >= 1) && (i <= Total_Tables())) {
          table = tables[i];
      }
      return table;
  }                                                          // Table Table(1)

  protected static int Total_Tables() {
      // Method to return the total number of tables (protocols) available 
      return total_tables;
  }                                                     // int Total_Tables(0)


/* -- Sim Anim methods ------------------------------------------------------*/

  protected static int Sched_x() {
      // Method to return the x position of Scheduler
      int x;

      x = 0 - (SCHED_w / 2);

      return Off_x(x);
  }                                                          // int Sched_x(0)

  protected static int Sched_y() {
      // Method to return the y position of Scheduler
      int y;

      y = 0 - (SCHED_h / 2);

      return Off_y(y);
  }                                                          // int Sched_y(0)

  protected static int Filter_x(int id) {
      // Method to return the x position of Filter with id
      int x = 0;
      int origin_x = Site_Origin_x(id);

      int side = Port_Side(SITE, id, SCHED);

      if ((side == Anim_port.TOP) || (side == Anim_port.BOTTOM)) {
          x = origin_x - (FILTER_w / 2);
      }
      else if (side == Anim_port.RIGHT) {
          x = origin_x - FILTER_w; 
      }
      else if (side == Anim_port.LEFT) {
          x = origin_x;
      }

      return Off_x(x);
  }                                                         // int Filter_x(1)

  protected static int Filter_y(int id) {
      // Method to return the y position of Filter with id
      int y = 0;

      int origin_y = Site_Origin_y(id);

      int side = Port_Side(SITE, id, SCHED);

      if ((side == Anim_port.RIGHT) || (side == Anim_port.LEFT)) {
          y = origin_y - (FILTER_h / 2);
      }
      else if (side == Anim_port.TOP) {
          y = origin_y; 
      }
      else if (side == Anim_port.BOTTOM) {
          y = origin_y - FILTER_h;
      }

      return Off_y(y);
  }                                                         // int Filter_y(1)

  protected static int Coord_x(int id) {
      // Method to return the x position of Coordinator with id
      int x = 0;
      int origin_x;
      int c_side, s_side;

      origin_x = Site_Origin_x(id);

      c_side = Port_Side(COORD, id, SITE);
      s_side = Port_Side(SITE, id, SCHED);

      if (c_side == Anim_port.TOP) {
  	  if (c_side == s_side) {
              x = origin_x + (CP_gap / 2);
	  }
	  else if (s_side == Anim_port.RIGHT) {
              x = origin_x - (FILTER_w / 2) + (CP_gap / 2);
          }
	  else if (s_side == Anim_port.LEFT) {
              x = origin_x + (FILTER_w / 2) + (CP_gap / 2);
          }
      }
      else if (c_side == Anim_port.BOTTOM) {
  	  if (c_side == s_side) {
              x = origin_x - ((CP_gap / 2) + COORD_w);
	  }
	  else if (s_side == Anim_port.RIGHT) {
              x = origin_x - ((FILTER_w / 2) + (CP_gap / 2) + COORD_w);
          }
	  else if (s_side == Anim_port.LEFT) {
              x = origin_x + (FILTER_w / 2) - ((CP_gap / 2) + COORD_w);
          }
      }
      else if (c_side == Anim_port.RIGHT) {
  	  if (c_side == s_side) {
              x = origin_x - (FILTER_w + FCP_pgap + (2 * PORT1_w) + COORD_w); 
	  }
	  else {
              x = origin_x - ((FILTER_w / 2) + FCP_pgap 
                                                  + (2 * PORT1_w) + COORD_w); 
          }
      }
      else if (c_side == Anim_port.LEFT) {
  	  if (c_side == s_side) {
              x = origin_x + (FILTER_w + FCP_pgap + (2 * PORT1_w));
	  }
	  else {
              x = origin_x + ((FILTER_w / 2) + FCP_pgap + (2 * PORT1_w));
	  }
      }

      return Off_x(x);
  }                                                          // int Coord_x(1)

  protected static int Coord_y(int id) {
      // Method to return the y position of Coordinator with id
      int y = 0;
      int origin_y;
      int c_side, s_side;

      origin_y = Site_Origin_y(id);

      c_side = Port_Side(COORD, id, SITE);
      s_side = Port_Side(SITE, id, SCHED);

      if (c_side == Anim_port.RIGHT) {
  	  if (c_side == s_side) {
              y = origin_y + (CP_gap / 2);
	  }
	  else if (s_side == Anim_port.TOP) {
              y = origin_y + (FILTER_h / 2) + (CP_gap / 2);
          }
	  else if (s_side == Anim_port.BOTTOM) {
              y = origin_y - (FILTER_h / 2) + (CP_gap / 2);
          }
      }
      else if (c_side == Anim_port.LEFT) { 
  	  if (c_side == s_side) {
              y = origin_y - ((CP_gap / 2) + COORD_h);
	  }
	  else if (s_side == Anim_port.TOP) {
              y = origin_y + (FILTER_h / 2) - ((CP_gap / 2) + COORD_h);
          }
	  else if (s_side == Anim_port.BOTTOM) {
              y = origin_y - ((FILTER_h / 2) + (CP_gap / 2) + COORD_h);
          }
      }
      else if (c_side == Anim_port.TOP) {
  	  if (c_side == s_side) {
              y = origin_y + (FILTER_h + FCP_pgap + (2 * PORT1_h)); 
	  }
          else {
              y = origin_y + ((FILTER_h / 2) + FCP_pgap + (2 * PORT1_h)); 
          }
      }
      else if (c_side == Anim_port.BOTTOM) {
  	  if (c_side == s_side) {
              y = origin_y - (FILTER_h + FCP_pgap + (2 * PORT1_h) + COORD_h);
	  }
	  else{
              y = origin_y - ((FILTER_h / 2) + FCP_pgap 
                                                  + (2 * PORT1_h) + COORD_h);
          }
      }

      return Off_y(y);
  }                                                          // int Coord_y(1)

  protected static int Part_x(int id) {
      // Method to return the x position of Participant with id
      int x = 0;
      int origin_x;
      int p_side, s_side;

      origin_x = Site_Origin_x(id);

      p_side = Port_Side(PART, id, SITE);
      s_side = Port_Side(SITE, id, SCHED);

      if (p_side == Anim_port.TOP) {
   	  if (p_side == s_side) {
              x = origin_x - ((CP_gap / 2) + PART_w);
	  }
	  else if (s_side == Anim_port.RIGHT) {
              x = origin_x - ((FILTER_w / 2) + (CP_gap / 2) + PART_w);
          }
	  else if (s_side == Anim_port.LEFT) {
              x = origin_x + (FILTER_w / 2) - ((CP_gap / 2) + PART_w);
          }
      }
      else if (p_side == Anim_port.BOTTOM) {
  	  if (p_side == s_side) {
              x = origin_x + (CP_gap / 2);
	  }
	  else if (s_side == Anim_port.RIGHT) {
              x = origin_x - (FILTER_w / 2) + (CP_gap / 2);
          }
	  else if (s_side == Anim_port.LEFT) {
              x = origin_x + (FILTER_w / 2) + (CP_gap / 2);
          }
      }
      else if (p_side == Anim_port.RIGHT) {
  	  if (p_side == s_side) {
              x = origin_x - (FILTER_w + FCP_pgap + (2 * PORT1_w) + PART_w); 
          }
          else {
              x = origin_x - ((FILTER_w / 2) + FCP_pgap 
                                                  + (2 * PORT1_w) + PART_w); 
          }
      }
      else if (p_side == Anim_port.LEFT) {
  	  if (p_side == s_side) {
              x = origin_x + (FILTER_w + FCP_pgap + (2 * PORT1_w));
	  }
	  else{
              x = origin_x + ((FILTER_w / 2) + FCP_pgap + (2 * PORT1_w));
	  }
      }

      return Off_x(x);
  }                                                           // int Part_x(1)

  protected static int Part_y(int id) {
      // Method to return the y position of Participant with id
      int y = 0;
      int origin_y;
      int p_side, s_side;
 
      origin_y = Site_Origin_y(id);

      p_side = Port_Side(PART, id, SITE);
      s_side = Port_Side(SITE, id, SCHED);

      if (p_side == Anim_port.RIGHT) {
  	  if (p_side == s_side) {
              y = origin_y - ((CP_gap / 2) + PART_h);
	  }
	  else if (s_side == Anim_port.TOP) {
              y = origin_y + (FILTER_h / 2) - ((CP_gap / 2) + PART_h);
          }
	  else if (s_side == Anim_port.BOTTOM) {
              y = origin_y - ((FILTER_h /2) + (CP_gap / 2) + PART_h);
          }
      }
      else if (p_side == Anim_port.LEFT) { 
  	  if (p_side == s_side) {
              y = origin_y + (CP_gap / 2);
	  }
	  else if (s_side == Anim_port.TOP) {
              y = origin_y + (FILTER_h / 2) + (CP_gap / 2);
          }
	  else if (s_side == Anim_port.BOTTOM) {
              y = origin_y - (FILTER_h / 2) +  (CP_gap / 2);
          }
      }
      else if (p_side == Anim_port.TOP) {
  	  if (p_side == s_side) {
              y = origin_y + (FILTER_h + FCP_pgap + (2 * PORT1_h)); 
	  }
	  else {
              y = origin_y + ((FILTER_h / 2) + FCP_pgap + (2 * PORT1_h)); 
	  } 
      }
      else if (p_side == Anim_port.BOTTOM) {
  	  if (p_side == s_side) {
              y = origin_y - (FILTER_h + FCP_pgap + (2 * PORT1_h) + PART_h);
	  }
	  else {
              y = origin_y - ((FILTER_h / 2) + FCP_pgap 
                                                  + (2 * PORT1_h) + PART_h);
	  } 
      }

      return Off_y(y);
  }                                                           // int Part_y(1)

  protected static int Port_Side(int s_type, int id, int d_type) {
      // Method to return an enumerated value indicating the side of an 
      // entity, given by src type, which a port is to be attached to.
      // The dest type is the entity which the port connects to (i.e. for the 
      // direction).  The id will identify the location of the site.
      int enum_v = Anim_port.TOP;
      double angle;
      double angle_0, angle_15, angle_30, angle_45, angle_75, angle_105,
             angle_135, angle_165, angle_195, angle_225, angle_255, 
             angle_285, angle_315, angle_345, angle_360;

      angle = Angle(id);
      angle_30  = (Math.PI / 6);
      angle_360 = (2 * Math.PI);

      angle_45  = (Math.PI / 4);
      angle_135 = (3 * Math.PI / 4);
      angle_225 = (5 * Math.PI / 4);
      angle_315 = (7 * Math.PI / 4);

      angle_15  = angle_45  - angle_30;
      angle_75  = angle_45  + angle_30;
      angle_105 = angle_135 - angle_30;
      angle_165 = angle_135 + angle_30;
      angle_195 = angle_225 - angle_30;
      angle_255 = angle_225 + angle_30;
      angle_285 = angle_315 - angle_30;
      angle_345 = angle_315 + angle_30;
  

      if (   (s_type == SCHED) 
          || ( (s_type == SITE) && ((d_type == COORD) || (d_type == PART)) )
         ) {
  	  // Scheduler, or Filter connected to Part or Coord

	  if ((angle > angle_315) || (angle < angle_45)) {
              enum_v = Anim_port.TOP;
          }
          else if ((angle >= angle_45) && (angle <= angle_135)) {
              enum_v = Anim_port.RIGHT;
	  }
          else if ((angle > angle_135) && (angle < angle_225)) {
              enum_v = Anim_port.BOTTOM;
	  }
          else if ((angle >= angle_225) && (angle <= angle_315)) {
              enum_v = Anim_port.LEFT;
	  }
      }
      else if ( (s_type == COORD) || (s_type == PART) ) {
	  // Coord or Part connected to a Filter

	  if ((angle > angle_315) || (angle < angle_45)) {
              enum_v = Anim_port.BOTTOM;
          }
          else if ((angle >= angle_45) && (angle <= angle_135)) {
              enum_v = Anim_port.LEFT;
	  }
          else if ((angle > angle_135) && (angle < angle_225)) {
              enum_v = Anim_port.TOP;
	  }
          else if ((angle >= angle_225) && (angle <= angle_315)) {
              enum_v = Anim_port.RIGHT;
	  }
      }
      else if ( (s_type == SITE) && (d_type == SCHED) ) {
	  // Site connected to scheduler

	  if ((angle > angle_315) || (angle < angle_45)) {
              if ((angle <= angle_345) && (angle > angle_315)) {
                  enum_v = Anim_port.RIGHT;
              }
	      else if (angle >= angle_15) {
                  enum_v = Anim_port.LEFT;
	      }
              else {
                  enum_v = Anim_port.BOTTOM;
	      }
          }
          else if ((angle >= angle_45) && (angle <= angle_135)) {
	      if (angle <= angle_75) {
                  enum_v = Anim_port.BOTTOM;
	      }
              else if (angle >= angle_105) {
                  enum_v = Anim_port.TOP;
              }
              else {
                  enum_v = Anim_port.LEFT;
	      }
	  }
          else if ((angle > angle_135) && (angle < angle_225)) {
	      if (angle <= angle_165) {
                  enum_v = Anim_port.LEFT;
	      }
              else if (angle >= angle_195) {
                  enum_v = Anim_port.RIGHT;
              }
              else { 
                  enum_v = Anim_port.TOP;
	      }
	  }
          else if ((angle >= angle_225) && (angle <= angle_315)) {
	      if (angle <= angle_255) {
                  enum_v = Anim_port.TOP;
	      }
              else if (angle >= angle_285) {
                  enum_v = Anim_port.BOTTOM;
              }
              else {
                  enum_v = Anim_port.RIGHT;
	      }
	  }
      }
      return enum_v;
  }                                                        // int Port_Side(3)

  protected static int Port_Pos(int s_type, int id) {
      // The number of pixels down the side of the entity with src type to 
      // attach the port at.
      // This assumes that the entity icons are square

      int pos = 0;
      int gap, ports, port_num;

      if (s_type == SCHED) {
          ports = Ports_On_Side_SCHED(id);
          port_num = Port_Number_SCHED(id);

          gap = ((SCHED_h - (ports * PORT1_h)) / (ports + 1));
          pos = (gap + ((PORT1_h + gap) * (port_num - 1))); 
      }
      else if (s_type == COORD) {
	  pos = ((COORD_h - PORT1_h) / 2);
      }
      else if (s_type == PART) {
	  pos = ((PART_h - PORT1_h) / 2);
      }
      else if (s_type == SITE) {
          pos = ((FILTER_h - PORT1_h) / 2);
      }
      return pos;
  }                                                         // int Port_Pos(2)

  protected static void Set_Radius(int r) {
      // Method to set the radius - dist between Sched and site
      RADIUS = r + (SCHED_h / 2);
  }                                                      // void Set_Radius(1)

  protected static int Radius() {
      // Method to return the radius - dist between Sched and site
      return RADIUS;
  }                                                           // int Radius(0)

  protected static int Off_x(int x) {
      // Method to return the offset x coord.
      // Translates from origin at center to origin at top left corner.

      return (x + Center_x());
  }                                                            // int Off_x(1)

  protected static int Off_y(int y) {
      // Method to return the offset y coord.
      // Translates from origin at center to origin at top left corner.
      return (y + Center_y());
  }                                                            // int Off_y(1)

  private static int Center_x() {
      // Method to return the x center of the display area
      int center_x;

      center_x = 250;

      return center_x;
  }                                                         // int Center_x(0)

  private static int Center_y() {
      // Method to return the y center of the display area
      int center_y;

      center_y = 175;

      return center_y;
  }                                                         // int Center_y(0)

  private static double Angle(int id) {
      // Method to return the angle (from vertical) of a site given
      // the id.
      // All angles in RADIANS
      double angle = 0.0;

      if ((id > 0) && (id <= Num_Sites())) {
          angle = ( (2 * Math.PI / Num_Sites()) * (id - 1) );
      }
      return angle;
  }                                                         // double Angle(1)

  private static int Site_Origin_x(int id) {
      // Method to return the x coord of origin of the site with id
      // The origin is the point where the port connection is
      int site_x = 0;

      site_x = (int) (Radius() * Math.sin(Angle(id)));

      return site_x;
  }                                                    // int Site_Origin_x(1)

  private static int Site_Origin_y(int id) {
      // Method to return the y coord of origin of the site with id
      // The origin is the point where the port connection is
      int site_y = 0;

      site_y = (int) (Radius() * (-1.0) * Math.cos(Angle(id)));

      return site_y;
  }                                                    // int Site_Origin_y(1)

  private static int Ports_On_Side_SCHED(int id) {
      // Returns the number of ports on a given side if the Scheduler
      int ports = 0;
      int i;
      int sched_side, port_side;

      port_side = Port_Side(SCHED, id, SITE);

      for (i=1; i <= Num_Sites(); i++) {
           sched_side = Port_Side(SCHED, i, SITE);
           if (sched_side == port_side) {
               ports++;
	   }
      }
      return ports;     
  }                                                    // int Ports_On_Side(1) 

  private static int Port_Number_SCHED(int id) {
      // Returns the port number on a given side if the Scheduler
      int port = 0;
      int i;
      int cur_side, last_side, port_side;

      port_side = Port_Side(SCHED, id, SITE);

      port = ((Ports_On_Side_SCHED(1) + 1) / 2);
      last_side = Port_Side(SCHED, 1, SITE);
    
      for (i=2; i <= id; i++) {
           cur_side = Port_Side(SCHED, i, SITE);

           if (cur_side == last_side) {
               port++;  
           }
           else {
               port = 1;
	   }
           last_side = cur_side;
      }
      if ((last_side == Anim_port.BOTTOM) || (last_side == Anim_port.LEFT)) {
          port = Ports_On_Side_SCHED(id) - (port - 1);
      }

      return port;     
  }                                                // int Port_Number_SCHED(1) 

  protected static int State_Pos_x(int type, int id) {
      // Returns the x coordinate for a coord or part state message
      int x, cp_side;

      x = 0;
      cp_side = Port_Side(COORD, id, SITE);

      if (type == COORD) {
  	  if (cp_side == Anim_port.TOP) {
              x = 0;
          }  
	  else if (cp_side == Anim_port.BOTTOM) {
              x = 0 - NAME_LENGTH + COORD_w;
          }
	  else if (cp_side == Anim_port.RIGHT) {
              x = 0 - NAME_LENGTH;
          }
	  else if (cp_side == Anim_port.LEFT) {
              x = COORD_w + 2;
          }
      }
      else if (type == PART) {
	  if (cp_side == Anim_port.TOP) {
              x = 0 - NAME_LENGTH + PART_w;
          }  
	  else if (cp_side == Anim_port.BOTTOM) {
              x = 0;
          }
	  else if (cp_side == Anim_port.RIGHT) {
              x = 0 - NAME_LENGTH;
          }
	  else if (cp_side == Anim_port.LEFT) {
              x = PART_w + 2;
          }
      }      

      return x;
  }                                                      // int State_Pos_x(2)

  protected static int State_Pos_y(int type, int id) {
      // Returns the y coordinate for a coord or part state message
      int y, cp_side;

      y = 0;
      cp_side = Port_Side(COORD, id, SITE);

      if (type == COORD) {
  	  if (cp_side == Anim_port.TOP) {
              y = COORD_h + 12;
          }  
	  else if (cp_side == Anim_port.BOTTOM) {
              y = 0 - 2;
          }
	  else if ((cp_side == Anim_port.RIGHT) 
                               || (cp_side == Anim_port.LEFT)) {
              y = COORD_h - 6;
          }
      }
      else if (type == PART) {
	  if (cp_side == Anim_port.TOP) {
              y = PART_h + 12;
          }  
	  else if (cp_side == Anim_port.BOTTOM) {
              y = 0 - 2;
          }
	  else if ((cp_side == Anim_port.RIGHT) 
                               || (cp_side == Anim_port.LEFT)) {
              y = PART_h - 6;
          }
      }      
      return y;
  }                                                      // int State_Pos_y(2)

  protected static int Id_Pos_x(int id) {
      // Returns the x coordinate for the filter id 
      int x, s_side, cp_side;

      x = 0;
      s_side = Port_Side(SITE, id, SCHED);
      cp_side = Port_Side(SITE, id, COORD);
     
      if ((s_side != Anim_port.RIGHT) && (cp_side != Anim_port.RIGHT)) {
          x = FILTER_w;
      }
      else if ( ((s_side != Anim_port.TOP) && (cp_side != Anim_port.TOP)) 
       || ((s_side != Anim_port.BOTTOM) && (cp_side != Anim_port.BOTTOM)) ) {
          x = 4;
      }
      else if ((s_side != Anim_port.LEFT) && (cp_side != Anim_port.LEFT)) {
          x = 0 - 15;
      }
      return x;
  }                                                         // int Id_Pos_x(1)

  protected static int Id_Pos_y(int id) {
      // Returns the y coordinate for the filter id 
      int y, s_side, cp_side;

      y = 0;
      s_side = Port_Side(SITE, id, SCHED);
      cp_side = Port_Side(SITE, id, COORD);
     
      if ((s_side != Anim_port.RIGHT) && (cp_side != Anim_port.RIGHT)) { 
          y = 12;
      }
      else if ((s_side != Anim_port.TOP) && (cp_side != Anim_port.TOP)) {
          y = 0 - 2;
      }
      else if ((s_side != Anim_port.BOTTOM) 
                                          && (cp_side != Anim_port.BOTTOM)) {
          y = FILTER_h + 12;
      }
      else if ((s_side != Anim_port.LEFT) && (cp_side != Anim_port.LEFT)) {
          y = 12;
      }
      return y;
  }                                                         // int Id_Pos_y(1)


}                                                              // class Option

/*----------------------------- End of Option.java --------------------------*/

