
/*
** Author : Steven Ramsay <smr@dcs.ed.ac.uk>, 15/02/97
** Files  : Part.java, Part.class
** Update : 11/06/97 
** Package: simdcp
**
** Description:
** The participant communicates with other sites via the scheduler.
** All messages are filtered through the Site.
*/

package eduni.simdcp;

import eduni.simjava.*;
import eduni.simanim.*;

/* Methods defined in final class Part
** PUBLIC
** Part(String, int, int, int, String)
**        void    body() 
**
** PROTECTED
**
** PRIVATE
**        void    Init_Variables()
**        void    Create_Lists() 
**        boolean Handle_Start_Txn()
**        boolean Handle_End_Txn()
**        void    Begin_Txn_PART(int)
**        int     Decide_Choice_PART(States)
**        int     Handle_Choice_PART(Choices)
**        int     Handle_Action_PART(Actable)
**        int     Handle_Action_Msg_PART(Msg)
**        int     Handle_Action_Ctrl_PART(Ctrl)
**        int     Handle_Action_Log_PART(Log)
**        int     Handle_Failure_PART()
**        Message Receive_Message()
**        void    Send_Message(Msg)
**        void    Schedule_Message(Message)
*/
final class Part extends Site implements Constants {

  private Sim_port   port_site;                         // Port to site filter

  private Site_Lists Lists;

  public Part(String name, int id, int x, int y, 
                                   String i_gif, String p_gif) {
      // Constructor for Participant at site
      super(name, id, PART, x, y, i_gif, p_gif);

      Debug("Initiating Part at site "+this.Id());

      this.port_site = new Sim_port("port_Site"+id, p_gif,
                                    Option.Port_Side(PART, id, SITE), 
                                    Option.Port_Pos(PART, id));
      add_port(port_site);
      add_param(new Anim_param("State", Anim_param.VALUE, "", 
                 Option.State_Pos_x(PART, id), Option.State_Pos_y(PART, id)));
      add_param(new Anim_param("Status", Anim_param.STATE, "sleep", 0, 0));

  }                                                     // constructor Site(2)
  
  protected void Init_Variables() {
    // Method to initialise the class variables because SimAnim does not 
    // destroy and recreate the objects.
      super.Init_Variables();

      this.Create_Lists();
      super.Set_Status(SLEEPING);
  }                                                  // void Init_Variables(0)

  public void body() {
      // Body for participant at site
      int start_state_num = 1;
      boolean stop;

      this.Init_Variables();

      stop = this.Handle_Start_Txn();

      while (!stop) {
  	  if (Is_Participant()) {
              super.Show_Window();
              Info("-> Started Participant "+Id());
              super.Set_Status(RUN_OK);
              this.Begin_Txn_PART(start_state_num);
          }
          else {
              super.Set_Status(SLEEPING);
              super.Hide_Window();
          }
	  stop = this.Handle_End_Txn();
      }
      if (super.Status() == FAILED) {
          Info("<- Participant "+Id()+" recovered after TXN completed");
      }    

      Info("<- Global decision: "+Global.Name_CorA(super.Global_Decision()));

      if (super.Local_Decision() != ABORT) {
          Info("<- Terminated Participant "+Id()+": "
                                  +Global.Name_CorA(super.Global_Decision()));
      }
      else {
          Info("<- Terminated Participant "+Id()+": "
                                   +Global.Name_CorA(super.Local_Decision()));
      }
  }                                                            // void body(0)

  private void Create_Lists() {
      // Method to create the lists for a participant
      this.Lists = new Site_Lists(PART, this.Id(), this.Site_Window());
  }                                                    // void Create_Lists(0)

  private boolean Handle_Start_Txn() {
      // Method to handle the start of a txn - initiates Part if required
      // Returns true if any error occured and txn not started
      int     info;
      boolean stop = true;
      Message msg;

      msg = this.Receive_Message();
      info = msg.Info();

      if ((msg.Round() == -1) && (msg.Name().compareTo("PARTICIPATE") == 0)) {
 	  if ((info == COMMIT) || (info == ABORT)) {
              super.Set_Role_Of_Site(PART);
              super.Set_Local_Decision(info);
          }
          else {
              super.Hide_Window();
          }
          stop = false;
      }
      else {
          Error("First message does not start txn");
          stop = true;
      }
      super.Store_Current_Msg(null);
      return stop;
  }                                             // boolean Handle_Start_Txn(0)

  private boolean Handle_End_Txn() {
      // Method to handle the end of a txn - either terminate entity
      // Should never be called unless a Participant is set up but excluded
      // from txn.
      // Returns true if site terminated
      boolean stop = false;
      Message msg, new_msg; 

      while (!stop) {
          msg = this.Receive_Message();

          if ((msg.Round() == -1) && (msg.Name().compareTo("END-TXN") == 0)) {
              // Transaction terminated
              super.Set_Global_Decision(msg.Info());
              stop = true;
              if ((super.Global_Decision() != UNKNOWN)
                                      || (super.Local_Decision() == ABORT)) {
                  Info("<- TXN completed.");
              }
              Info("<- Local decision: "
                                   +Global.Name_CorA(super.Local_Decision()));
          }
          else {
              super.Store_Current_Msg(null);
          }
      }
      return stop;
  }                                               // boolean Handle_End_Txn(0)


  private void Begin_Txn_PART(int state_num) {
      // Wait for/send messages from/to coordinator as detailed in table,
      // starting in state given.
      int     next_state_num;
      boolean stop;
      States  cur_state;

      stop = false;

      while ((state_num <= Table().Total_States(PART)) && (!stop)) {
          super.Set_State_Num(state_num);
          if (state_num == Table().Recover_State(PART)) {
              // hold causes unpredictable behaviour in SimAnim - smr
              super.Set_Status(RUN_OK);
	      sim_hold(RECOVER_DELAY);
          }

          cur_state = Table().State(PART, state_num);
          next_state_num = this.Decide_Choice_PART(cur_state);

          Debug("in state "+state_num); 
          Debug("next state "+next_state_num); 
          if (next_state_num == 0) {
	      stop = true;                      // No state to move to so stop
          }
          else if (next_state_num == FAIL) {
              Debug("failed choices in state "+state_num);
              Error("<- TERMINATED BEFORE COMPLETION");
              stop = true;
          }
          else {
	      if (state_num != next_state_num) {
	          // New State so reset number to fail in old and next state
		  //   Global.Reset_Num_Have_Failed(state_num);
                  //  Global.Reset_Num_Have_Failed(next_state_num);
		  // Must add new way as this will reset the number when
  	  	  // every part leaves state - not when all have left
	      }
   	      state_num = next_state_num;
          }
      }
  }                                                  // void Begin_Txn_Part(1)

  private int Decide_Choice_PART(States state) {
      // Method to select a choice of actions given a state.
      // If there are no choices or only one then there is no decision to
      // be made, otherwise the choice will be made depending on the first 
      // action of each choice.  It may be simpler to try the first choice,
      // then if its first action fails the second choice could be tried.
      // If a first action succeeds then it is committed to that choice.
      // The next state will be returned (from the choice selected) or zero
      // to stop the txn.
      int next_state_num;
      int total_choices;
      int choice_num;

      Debug("Deciding choice for part"+this.Id());
      total_choices = state.Total_Choices();

      if (total_choices == 0) {                        // No choices available
          Error("no choices, stop");
          next_state_num = FAIL; 
      }
      else {                  // This will try each choice until it can do one
          next_state_num = RETRY;
          choice_num = 1;
	  while (next_state_num == RETRY) { 
	      if (choice_num > total_choices) {          // If no choices left
                  next_state_num = FAIL;
              }
              else {                                             // Try choice
                  Debug("trying choice "+choice_num);
                  super.Set_Choice_Num(choice_num);
                  next_state_num = 
                            this.Handle_Choice_PART(state.Choice(choice_num));
              }
              choice_num++;
          }                       
      }
      return next_state_num;               // return next state number or fail
  }                                               // int Decide_Choice_Part(1)

  private int Handle_Choice_PART(Choices choice) {
      // Method to handle the choice of actions and return the next state. 
      int action_num;
      int success;
      int next_state_num;
      
      action_num = 1;
      success = OK;

      while ((action_num <= choice.Total_Actions()) && (success == OK)) {

          success = Handle_Action_PART(choice.Action(action_num));

          if (super.Status() == FAILED) {
              Debug("action "+action_num+" not completed due to failure");
              success = Handle_Failure_PART();
          } 
          else { 
              if ((success == OK)) {
                               // Firt action okay so committed to this choice
                  Debug("action "+action_num+" completed successfully");
                  action_num++;
              }
              else if (action_num == 1) {
  	          // Could not handle first action so return to try 
		  // another choice
		  // Leave stored message for use in trying next choice
                  success = RETRY;
              }     
              else {            // Handled first action so committed to choice
                  success = FAIL;
              }
          }
      }
      if ((success == OK) && (super.Status() != FAILED)) {
          next_state_num = choice.Next_State();
      }
      else { 
          next_state_num = success;
      }
      return next_state_num;                              // Return next state
  }                                               // int Handle_Choice_PART(1)

  private int Handle_Action_PART(Actable action) {
      // Method to handle the Participant's action depending on what it is,
      // i.e. message to be sent/received, control action, log r/w. 
      int success;

      success = FAIL;

      Debug("handling action - "+action.Display());

      if (action instanceof Msg) {                        // Action is message
          success = Handle_Action_Msg_PART((Msg) action);
      }

      else if (action instanceof Ctrl) {
          success = Handle_Action_Ctrl_PART((Ctrl) action);
      } 

      else if (action instanceof Log) {
          success = Handle_Action_Log_PART((Log) action);
      }

      else {
          Error("unknown class");
      }
      Debug("returning success "+success); 
      return success;
  }                                               // int Handle_Action_Part(1)

  private int Handle_Action_Msg_PART(Msg msg_act) {
      // Method to handle a Participant's message action.
      // i.e. protocol 'real' message to be sent or received.
      int       success;
      boolean   fail_point;
      Message   msg;

      success = FAIL;
      fail_point = false;

      if (msg_act.Send()) {                                    // Send message
          
          fail_point = super.Determine_Failure();

          if (fail_point) {
              Info("  Adversary, should I fail?");
              Info("      (before sending "+msg_act.Name()
                   +" in state "+super.State_Num()+":"
                   +super.State_Name()+")");
              Info("  Adversary: YES");
              super.Set_Status(FAILED);
              // hold causes unpredictable behaviour in SimAnim - smr
	      // sim_hold(CRASH_DELAY);
          }
          else {
              Debug("  Adversary, should I fail?");
              Debug("      (before sending "+msg_act.Name()
                   +" in state "+super.State_Num()+":"
                   +super.State_Name()+")");
              Debug("  Adversary: NO");
          }
          this.Send_Message(msg_act);

          success = OK;
      }

      else if (msg_act.Receive()) {                         // Receive message
          Debug("expecting message "+msg_act.Name()+","+msg_act.Round());

	  msg = this.Receive_Message();
 
          if (msg.Src_Type() == COORD) {
              if ((msg.Name().compareTo(msg_act.Name()) == 0) 
                                        && (msg_act.Round() == msg.Round())) {
                  // Matching message pair - as expected
                  success = OK;
                  if (msg.Round() == 1) {  
                      // Set Coord ID from first message
                      super.Set_Coord_Id_Of_Part(msg.Src_Id());
                      Debug("Coordinator's Id: "+this.Coord_Id_Of_Part());
                  }
                  if (msg.List()) {                   // Set Parts In Txn List
                      this.Lists.Copy_Parts_In_Txn_List(msg.Part_List());
                  }
                  super.Set_Global_Decision(msg.Info());
              } 
              else {              // Not a pair - rec message was not expected
                  Debug("expecting message "
                                         +msg_act.Name()+","+msg_act.Round());
                  Debug("received message "+msg.Name()+","+msg.Round());
              }
          }
          else if (msg.Src_Type() == PART) {
              Error("cannot handle Part->Part messages");
          }
          else if (msg.Src_Type() == SCHED) {
	      if (msg.Round() == -1) {
		  // Message from Scheduler because simulation blocked
	  	  // Do nothing
 	      }
              else {
                  Error("Unknown message from Scheduler");
              }
          }
          else {
              Error("Unknown Src_Type Handle_Action_Msg_PART(1)");
          }

          if (success == OK) { 
	      // Finished with message if it was expected
	      // Otherwise, must store for remaining choices
              super.Store_Current_Msg(null); 
          }

      }
      else {
          Error("can't handle message");
      }
        
      return success;
  }                                           // int Handle_Action_Msg_PART(1)

  private int Handle_Action_Ctrl_PART(Ctrl ctrl_act) {
      // Method to handle a Participant's Control action.
      // i.e. vote, timeout, commit or abort (end)
      int       success;
      Msg       msg_tmp;
      Message   msg;
   
      success = FAIL;

      if (ctrl_act.Name().compareTo("VOTE") == 0) {   
          // Control action is to vote, done by checking info field
          if (ctrl_act.Info() == super.Local_Decision()) {
              success = OK;
              Info("Voting "+Global.Name_CorA(super.Local_Decision()));
          }
          else {
              success = FAIL;
              Debug("failing vote ctrl message");
          }
      }

      else if (ctrl_act.Name().compareTo("TIMEOUT") == 0) {
  	  // Control action is to expect a Timeout message
          msg = Receive_Message();

          if (msg.Name().compareTo("TIMEOUT") == 0) {
	      if (ctrl_act.Msg().Round() == msg.Round()) {
                  success = OK;
                  Info("TIMEOUT in state "+super.State_Name());
	      }
	      else {
                  Error("failing timeout, ctrl msg rnd "
                         +ctrl_act.Msg().Round()+", msg rnd "
                         +msg.Round()+" - not current");
                  Error("local message round is "+super.Local_Msg_Round());
	      }
          }
	  else {
              Debug("Message received not TIMEOUT");
	  }
          if (success == OK) { 
              super.Store_Current_Msg(null); 
          }
      }

      else if (   (ctrl_act.Name().compareTo("COMMIT") == 0) 
               || (ctrl_act.Name().compareTo("ABORT")  == 0)) {

          if (ctrl_act.Is_Msg()) {
	      // A commit or abort control action must involve
	      // sending or receiving a message
              msg_tmp = ctrl_act.Msg();

              if (msg_tmp.Receive()) {
                  msg = Receive_Message();

                  if ((msg.Name().compareTo("END-TXN") == 0) 
                                      && (msg.Round() == -1) 
                                      && (msg.Info() == ctrl_act.Info())) {
                      success = OK;
                  }
                  else {
  	  	      // Message does not end txn - may be for another choice
                  }
              }
              else {
                  Error("can't handle send END control message");
              }
          }
          else {
              Error("invalid Ctrl action for commit or abort, no message");
         }      
      }
      else {
          Error("unknown ctrl message");
      }

      return success;
  }                                          // int Handle_Action_Ctrl_PART(1)

  private int Handle_Action_Log_PART(Log log_act) {
      // Method to handle a Participant's log actions.
      // i.e. log read or write.
      int       success, info;
      String    name;
      Log       log_entry;
   
      success = FAIL;

      if (log_act.RW() == W) {
	  // Write log information to DT log
          this.Lists.Write_DT_Log(log_act);
          Info("DT Log write: "+log_act.Name()
               +" ("+Global.Name_CorA(log_act.Info())+")");
          success = OK;
      }
      else if (log_act.RW() == R) {
	  // Read last DT log entry and match with expected log action
          log_entry = this.Lists.Read_DT_Log();
          name = log_entry.Name();
          info = log_entry.Info();
          if ((log_act.Name().compareTo(name) == 0) 
                                               && (log_act.Info() == info)) {
              Info("DT Log read: "+name+" ("+Global.Name_CorA(info)+")");
              Global.Inc_DT_Log_Reads();
              success = OK;
          }
          else {
              Debug("log entry ("+log_act.Name()+", "+log_act.Info()
                    +") expected, read ("+name+", "+info+")");
          }
      }
      else {
          Error("Unknown log access");
      }
      return success;
  }                                           // int Handle_Action_Log_PART(1)


  private int Handle_Failure_PART() {
      // Method to handle failure by waiting for a message and trying to
      // recover.
      int     last_msg_count;
      int     next_state; 
      boolean recover_point;
      Message msg, last_msg = null;

      recover_point = false;
      next_state = -1;
      last_msg_count = 0;

      if (super.Status() != FAILED) {
          Error("Status is not failed - should not be handling failure");
      }

      while (next_state == -1) {

          msg = Receive_Message();
          if (last_msg_count == 0) {
              Debug("Message is first, reset last message");
              last_msg = msg.Clone();
              last_msg_count++;
          }
          else {
              Debug("Name: "+last_msg.Name()+", "+msg.Name()+" - Type: "
                    +last_msg.Src_Type()+", "+msg.Src_Type()+" - Id: "
                    +last_msg.Src_Id()+", "+msg.Src_Id());
              if (   (last_msg.Name().compareTo(msg.Name()) == 0) 
                  && (last_msg.Src_Type() == msg.Src_Type()) 
  	  	  && (last_msg.Src_Id() == msg.Src_Id())         ) { 
  	  	  // Message is same as last
                  last_msg_count++;
                  Debug("Message is same, count "+last_msg_count);
                  last_msg = msg.Clone();
              }
              else {
                  last_msg_count = 0;
                  Debug("Message is different, reset last message");
                  last_msg = null;
              }
          }  
        
          if (last_msg_count >= DETECT_MESSAGE_LOOP) {
 	      // If message cycle detected then force recovery
              Info("");
              Info("*** Detected a dependency loop with "
                   +Global.Short_Type(msg.Src_Type())+msg.Src_Id()+" \t***");
              Info("*** Possibly resulting from non-recovery \t***");
              Info("*** Message "+msg.Name()+" received "+last_msg_count
                   +" \t***");
              Info("*** Protocol has probably blocked \t***\n");
              Info("*** Forcing recovery to break cycle \t***\n");
              Global.Inc_Timeout_Cycles();
              recover_point = true;
	  }
          else {
  	      // Message does not indicate cycle so allow choice
              recover_point = super.Determine_Recovery();
          }
  
          if (recover_point) {
              Info("  Adversary, should I recover?");
              Info("      (before receiving "+msg.Name()+")");
              Info("  Adversary: YES");
              // DO NOT CHANGE STATUS - HANDLED LATER
              next_state = Option.Table().Recover_State(PART);
          }
          else {
              Debug("  Adversary, should I recover?");
              Debug("      (before receiving "+msg.Name()+")");
              Debug("  Adversary: NO");

              if (!msg.Control()) {
	          // Received a protocol message so send a timeout back.

	          // Will only send a timeout to coordinator.
	          // Assumes that participant only sent HELP-ME, no reply sent
 	          if ((msg.Src_Type() == COORD) 
                              && (msg.Src_Id() == super.Coord_Id_Of_Part())) {
                      msg = new Message("TIMEOUT", 
                                        COORD, super.Coord_Id_Of_Part(), null,
                                        PART, this.Id(), msg.Round(), 
                                        msg.Info(), null, true);
                      this.Schedule_Message(msg);
                      // Current message is lost otherwise loop occurs
                      // May need the message after recovery - unsure 
                      super.Store_Current_Msg(null);
	          }
	          else {
                      Debug("Protocol message "+msg.Name()+" from "
                            +Global.Short_Type(msg.Src_Type())+" ignored");
                      super.Store_Current_Msg(null);
                  }
              }
              else {
  	          if (msg.Name().compareTo("END-TXN") == 0) {
                      next_state = 0;
	          }
	          else {
                      Debug("Control message "+msg.Name()+" from "
                        +Global.Short_Type(msg.Src_Type())+" ignored");
                      super.Store_Current_Msg(null);
                  }
              }
          }          
      }
      return next_state; 
  }                                              // int Handle_Failure_PART(0)

  private Message Receive_Message() { 
      // Method to wait for a message (event) or return the first queued
      // in buffer.  The buffered message will be returned first.
      Message   msg = null, reply_msg = null;
      Sim_event ev = new Sim_event();

      msg = super.Stored_Message();

      if (msg != null) {            // Only wait for message if not one stored
          Debug("using stored message");
      }
      else {
          Debug("No message currently stored"); 
          Debug("number of events queued: "+sim_waiting());
          Debug("waiting for event to arrive");

          sim_get_next(ev);

          msg = (Message) ev.get_data();

          Trace_Msg_Received(msg);

          super.Store_Current_Msg(msg);            // Store current message

          if (msg.Control()) {
              Global.Inc_Ctrl_Msgs_Rec();
              Debug("control msgs received "+Global.Ctrl_Msgs_Rec());
          }
          else {
              Info("Received "+msg.Name());
              Global.Inc_Real_Msgs_Rec();
              Debug("real msgs received "+Global.Real_Msgs_Rec());
          }
      }
      if (msg.Round() == (super.Local_Msg_Round() + 1)) {
          super.Inc_Local_Msg_Round();
      }
      return msg;
  }                                              // Message Receive_Message(0)

  private void Send_Message(Msg msg_act) {
      // Method to send a message given the source type and action
      // describing message.
      // Assumes coord messages are for part only and vice versa.
      int[]   part_list = new int[MAX_SITES+1];
      int[]   dest_list = new int[MAX_SITES+1];
      Message msg;

      Debug("about to send message");
   
      if (msg_act.Round() == (Global.Message_Round() + 1)) {
          Global.Inc_Msg_Round();
      }
      if (msg_act.Round() == (super.Local_Msg_Round() + 1)) {
          super.Inc_Local_Msg_Round();
      }

      if (super.Status() == RUN_OK) {
          if (msg_act.Unicast()) {
              msg = new Message(msg_act.Name(), 
                                COORD, super.Coord_Id_Of_Part(), null,
                                PART, this.Id(), msg_act.Round(), 
                                msg_act.Info(), null);
              this.Schedule_Message(msg);
          }
          else {
	      Debug("only multicast msg is HELP-ME");
              if (super.Stored_Message() != null) {
                  Debug("Assuming can recover from stored msg");
                  // Otherwise have to ask Coord, assuming it hasn't failed
	      } 
              else {
                  Error("must multicast HELP-ME to Parts"); 
                  // Either to Parts only or also Coord
	      } 
          }
      }
      else if (super.Status() == FAILED) {
          msg = new Message("TIMEOUT", 
                            COORD, super.Coord_Id_Of_Part(), null,
                            PART, this.Id(), msg_act.Round(), 
                            msg_act.Info(), null, true);
          this.Schedule_Message(msg);
      }
      else {
          Error("Can't send messages in any other status");
      }
  }                                                    // void Send_Message(1)

  private void Schedule_Message(Message snd_msg) {
      // Method to actually send a message to intended recipient. 
      Message msg;

      msg = snd_msg.Clone();
      sim_schedule(port_site, LINK_DELAY, SITE, msg);

      Trace_Msg_Sent(msg, port_site.get_pname());

      if (msg.Control()) {
          Global.Inc_Ctrl_Msgs_Sent();
          Debug("control msgs sent "+Global.Ctrl_Msgs_Sent());
      }
      else {
          Info("Sent "+msg.Name()+" (state "+super.State_Num()
               +":"+super.State_Name()+")");
          Global.Inc_Real_Msgs_Sent();
          Debug("real msgs sent "+Global.Real_Msgs_Sent());
          // Assume if message sent then no fail/recover cycle exists
	  super.Reset_Times_Failed();                          // New state
      }

  }                                                // void Schedule_Message(1)

}                                                                // class Part

/*---------------------------- End of Part.java -----------------------------*/

