
/*
** Author : Steven Ramsay <smr@dcs.ed.ac.uk>, 19/03/97
** Files  : Pr_Dist.java, Pr_Dist.class
** Update : 11/06/97 
** Package: simdcp
**
** Description:
** All probability distributions and methods for handling them
*/

package eduni.simdcp;

import eduni.simjava.*;

/* Methods defined in class Pr_Dist
** PUBLIC
** -- no constructor -- 
**
** PROTECTED
** static double  Sample_Failure(int)
** static double  Sample_Recovery(int)
** static void    Set_Pr_Dist_Failure(int)
** static int     Pr_Dist_Failure()
** static void    Set_Pr_Dist_Recovery(int)
** static int     Pr_Dist_Recovery()
** static int     Seed();
** static void    Set_Min_Uni_Failure(double)
** static void    Set_Max_Uni_Failure(double)
** static void    Set_Mean_Norm_Failure(double)
** static void    Set_Var_Norm_Failure(double)
** static void    Set_Min_Uni_Recovery(double)
** static void    Set_Max_Uni_Recovery(double)
** static void    Set_Mean_Norm_Recovery(double)
** static void    Set_Var_Norm_Recovery(double)
** static double  Min_Uni_Failure()
** static double  Max_Uni_Failure()
** static double  Mean_Norm_Failure()
** static double  Var_Norm_Failure()
** static double  Min_Uni_Recovery()
** static double  Max_Uni_Recovery()
** static double  Mean_Norm_Recovery()
** static double  Var_Norm_Recovery()
** static String  Name_Pr_Dist(int)
** static void    Display_Pr_F_Stats()
** static void    Display_Pr_R_Stats()
**
** PRIVATE
** static void    Set_Seed();
** static void    Info_Pub();
*/
class Pr_Dist implements Constants {
  // Class for all probability distributions, etc.

  // Need to consider choice of distribution values
  // Is it necessary to be able to change the values?

  private static double  MIN_UNI_F   = 0.0, MAX_UNI_F  = 1.0; 
  private static double  MIN_UNI_R   = 0.0, MAX_UNI_R  = 1.0;
  private static double  MEAN_NORM_F = 0.5, VAR_NORM_F = 1.0;
  private static double  MEAN_NORM_R = 0.5, VAR_NORM_R = 1.0;

  private static int     PR_DIST_FAILURE  = NONE;
  private static int     PR_DIST_RECOVERY = NONE;
  private static int     SEED             = 1;

  private static Sim_uniform_obj uni_f_dist, uni_r_dist;
  private static Sim_normal_obj  norm_f_dist, norm_r_dist;

  static {
    Set_Seed(556);              // May need to look at seed generation/choice

    uni_f_dist  = new Sim_uniform_obj("Uni_F", Min_Uni_Failure(),
                                               Max_Uni_Failure(),
                                               Seed());
    norm_f_dist = new Sim_normal_obj("Norm_F", Mean_Norm_Failure(),
                                               Var_Norm_Failure(),
                                               Seed());
    uni_r_dist  = new Sim_uniform_obj("Uni_R", Min_Uni_Recovery(),
                                               Max_Uni_Recovery(),
                                               Seed());
    norm_r_dist = new Sim_normal_obj("Norm_R", Mean_Norm_Recovery(),
                                               Var_Norm_Recovery(),
                                               Seed());
  }

  protected static double Sample_Failure(int dist) {
      // Method to return the next sample in failure distribution
      // if not valid distribution then it returns 0.0
      double sample = 0.0;

      if (dist == UNIFORM) {
          sample = uni_f_dist.sample();
      }
      else if (dist == NORMAL) {
          sample = norm_f_dist.sample();
      }
      return sample;
  }                                                // double Sample_Failure(1)

  protected static double Sample_Recovery(int dist) {
      // Method to return the next sample in recovery distribution
      // if not valid distribution then it returns 0.0
      double sample = 0.0;

      if (dist == UNIFORM) {
          sample = uni_r_dist.sample();
      }
      else if (dist == NORMAL) {
          sample = norm_r_dist.sample();
      }
      return sample;
  }                                               // double Sample_Recovery(1)
 
  protected static void Set_Pr_Dist_Failure(int pr_dist) {  
      // Method to set the failure probability distribution
      PR_DIST_FAILURE = pr_dist;
  }                                             // void Set_Pr_Dist_Failure(1)

  protected static int Pr_Dist_Failure() {  
      // Method to return the failure probability distribution
      return PR_DIST_FAILURE;
  }                                              // boolean Pr_Dist_Failure(0)

  protected static void Set_Pr_Dist_Recovery(int pr_dist) {  
      // Method to set the recovery probability distribution
      PR_DIST_RECOVERY = pr_dist;
  }                                             // void Set_Pr_Dist_Recover(1)

  protected static int Pr_Dist_Recovery() {  
      // Method to return the recovery probability distribution
      return PR_DIST_RECOVERY;
  }                                              // boolean Pr_Dist_Recover(0)

  private static void Set_Seed(int seed) {
      // Method to set the seeds used
      SEED = seed;
  }                                                        // void Set_Seed(0)

  protected static int Seed() {
      // Method to return the seed used
      return SEED;
  }                                                             // int Seed(0)

  protected static void Set_Min_Uni_Failure(double d) {
      // Method to set the min value for uniform dist. for failures
      MIN_UNI_F = d;
  }                                             // void Set_Min_Uni_Failure(1)

  protected static void Set_Max_Uni_Failure(double d) {
      // Method to set the max value for uniform dist. for failures
      MAX_UNI_F = d;
  }                                             // void Set_Max_Uni_Failure(1)

  protected static void Set_Mean_Norm_Failure(double d) {
      // Method to set the mean for normal dist. for failures
      MEAN_NORM_F = d;
  }                                           // void Set_Mean_Norm_Failure(1)

  protected static void Set_Var_Norm_Failure(double d) {
      // Method to set the variance for normal dist. for failures
      VAR_NORM_F = d;
  }                                            // void Set_Var_Norm_Failure(1)

  protected static void Set_Min_Uni_Recovery(double d) {
      // Method to set the min value for uniform dist. for recovery
      MIN_UNI_R = d;
  }                                            // void Set_Min_Uni_Recovery(1)

  protected static void Set_Max_Uni_Recovery(double d) {
      // Method to set the max value for uniform dist. for recovery
      MAX_UNI_R = d;
  }                                            // void Set_Max_Uni_Recovery(1)

  protected static void Set_Mean_Norm_Recovery(double d) {
      // Method to set the mean for normal dist. for recovery
      MEAN_NORM_R = d;
  }                                          // void Set_Mean_Norm_Recovery(1)

  protected static void Set_Var_Norm_Recovery(double d) {
      // Method to set the variance for normal dist. for recovery
      VAR_NORM_R = d;
  }                                           // void Set_Var_Norm_Recovery(1)

  protected static double Min_Uni_Failure() {
      // Method to return the min value for uniform dist. for failures
      return MIN_UNI_F;
  }                                               // double Min_Uni_Failure(0)

  protected static double Max_Uni_Failure() {
      // Method to return the max value for uniform dist. for failures
      return MAX_UNI_F;
  }                                               // double Max_Uni_Failure(0)

  protected static double Mean_Norm_Failure() {
      // Method to return the mean for normal dist. for failures
      return MEAN_NORM_F;
  }                                             // double Mean_Norm_Failure(0)

  protected static double Var_Norm_Failure() {
      // Method to return the variance for normal dist. for failures
      return VAR_NORM_F;
  }                                              // double Var_Norm_Failure(0)

  protected static double Min_Uni_Recovery() {
      // Method to return the min value for uniform dist. for recovery
      return MIN_UNI_R;
  }                                              // double Min_Uni_Recovery(0)

  protected static double Max_Uni_Recovery() {
      // Method to return the max value for uniform dist. for recovery
      return MAX_UNI_R;
  }                                              // double Max_Uni_Recovery(0)

  protected static double Mean_Norm_Recovery() {
      // Method to return the mean for normal dist. for recovery
      return MEAN_NORM_R;
  }                                            // double Mean_Norm_Recovery(0)

  protected static double Var_Norm_Recovery() {
      // Method to return the variance for normal dist. for recovery
      return VAR_NORM_R;
  }                                             // double Var_Norm_Recovery(0)

  protected static String Name_Pr_Dist(int pr_dist) {
      // Method to return a string with the name of the probability dist. 
      String name = null;
 
      if (pr_dist == UNIFORM) {
          name = new String("Uniform");
      }
      else if (pr_dist == NORMAL) {
          name = new String("Normal");
      }
      else {
          name = new String("None");
      }
      return name;
  }                                                   // String Name_Pr_Dist(1)

  protected static void Display_Pr_F_Stats() {
      // Method to display the statistics relating to the probability
      // distributions used in simulation for failure

      Info_Pub("Prob. dist. for failure   \t: "
                                            +Name_Pr_Dist(Pr_Dist_Failure()));
      if (Pr_Dist_Failure() == UNIFORM) {
	  // Uniform distribution
          Info_Pub("\t\t\tmin: "+Min_Uni_Failure()
                   +", max: "+Max_Uni_Failure());
      }
      else if (Pr_Dist_Failure() == NORMAL) {
	  // Normal distribution
          Info_Pub("\t\t\tmean: "+Mean_Norm_Failure()
                   +", var: "+Var_Norm_Failure());
      }
  }                                              // void Display_Pr_F_Stats(0)


  protected static void Display_Pr_R_Stats() {
      // Method to display the statistics relating to the probability
      // distributions used in simulation for recovery

      Info_Pub("Prob. dist. for recovery \t: "
                                           +Name_Pr_Dist(Pr_Dist_Recovery()));
      if (Pr_Dist_Recovery() == UNIFORM) {
          // Uniform distribution
          Info_Pub("\t\t\tmin: "+Min_Uni_Recovery()
                   +", max: "+Max_Uni_Recovery());
      }
      else if (Pr_Dist_Recovery() == NORMAL) {
          // Normal distribution
          Info_Pub("\t\t\tmean: "+Mean_Norm_Recovery()
                    +", var: "+Var_Norm_Recovery());
      }
  }                                              // void Display_Pr_R_Stats(0)

  private static void Info_Pub(String note) {
      // Method for displaying info, mainly in stats.
      // Interface to Scheduler.Info_Pub()
      Scheduler.Info_Pub(note);
  }                                                        // void Info_Pub(1)

}                                                             // class Pr_Dist

/*---------------------------- End of Pr_Dist.java --------------------------*/
