
/*
** Author : Steven Ramsay <smr@dcs.ed.ac.uk>, 15/01/97
** Files  : Site.java, Site.class
** Update : 11/06/97 
** Package: simdcp
**
** Description:
** The site class contains methods and data common to the coordinator and
** participant as well as the filter for each site.
*/

package eduni.simdcp;

import java.awt.Frame;
import eduni.simjava.*;
import eduni.simanim.*;

/* Methods defined in class Site 
** PUBLIC
** Site(String, int, type, int, int, String)
**
** -- ABSTRACT -- these must be defined in any subclass
** public  void    body()
** private void    Send_Message(Message)
** private Message Receive_Message()
**
** PROTECTED
**        void    finalize()
**        void    Init_Variables()
** final  int     Id()
** final  int     Type()
** final  void    Set_State_Num(int)
** final  int     State_Num()
** final  String  State_Name()
** final  void    Set_Choice_Num(int)
** final  int     Choice_Num()
** final  void    Set_Status(int)
** final  int     Status()
** final  void    Store_Current_Msg(Message) 
** final  Message Stored_Message()
** final  boolean Is_Coordinator()
** final  boolean Is_Participant()
** final  void    Table()
** final  void    Set_Role_Of_Site(int)
** final  void    Display_Role_Of_Site()
** final  void    Set_Coord_Id_Of_Part(int)
** final  int     Coord_Id_Of_Part()
** final  void    Inc_Local_Msg_Round()
** final  int     Local_Msg_Round()
** final  void    Set_Local_Decision(int)
** final  int     Local_Decision()
** final  void    Set_Global_Decision(int)
** final  int     Global_Decision()
**        void    Reset_Times_Failed()
** final  boolean Determine_Failure()
** final  boolean Determine_Recovery()
** final  void    Trace_Msg_Sent(Message, String)
** final  void    Trace_Msg_Received(Message)
**        void    Show_Window()
**        void    Hide_Window()
**        W_Box   Site_Window()
** final  void    Debug(String)
** final  void    Error(String)
** final  void    Info(String)
** final  void    Route(String)
** final  void    Reset_Window()
** final  void    Trace_S(String, String)
** final  void    Trace_P_icon(String)
** final  void    Trace_P_state()
** final  void    Trace(String)
**
** PRIVATE
**        void    Set_Id(int)
**        void    Set_Type(int)
**        void    Inc_Times_Failed()
**        int     Times_Failed()   
**        boolean Check_F_Always()
**        boolean Prompt_F_User()
**        boolean Check_F_Interval()
**        boolean Check_F_State()
**        boolean Check_F_Dist()
**        boolean Prompt_R_User()
**        boolean Check_R_Interval()
**        boolean Check_R_Dist()
**        void    Setup_Window()
**        void    Close_Window()
**        void    Debug_GUI(String)
**        void    Error_GUI(String)
**        void    Info_GUI(String)
*/
class Site extends Sim_entity implements Constants {
  private int     site_id;                            // Site identifer number
  private int     site_type;                                   // Type of site
  private int     site_state_num;                      // State number of site
  private int     site_choice_num;                    // Choice number of site
  private int     site_status = SLEEPING;                    // Status of site
  private Message stored_msg;               // Stores current message received

  private boolean coordinator;                        // Is site a coordinator
  private boolean participant;                        // Is site a participant

  private int   global_decision;                          // Global txn status
  private int   local_decision;                           // Local part status
  private int   local_msg_round;                        // Local message round

  private int   PART_coord_id;                       // Participant's Coord Id

  private int   times_failed;             // Number of times failed in a state

  private W_Box site_window;


  //  private Frame prompt_frame;  - smr
  //  private P_Box site_prompt;   - smr

  private int x, y;                                   // Sim_anim icon co-ords
  private String i_gif, p_gif;         // Sim_entity icon name, port icon name

  public Site(String name, int id, int type, int x, int y, 
                                   String i_gif, String p_gif) {
      // Constructor for Coord and Part at site
      super(name, i_gif, x, y);
      this.x = x; this.y = y;
      this.i_gif = i_gif; this.p_gif = p_gif;

      this.Set_Id(id);                         // Set Site's Identifier number
      this.Set_Type(type);                                 // Set type of site
  }                                                     // constructor Site(2)
  
  protected void finalize() throws Throwable {
      // Finalize method to clean up windows opened etc when site terminates
      this.Close_Window();
      super.finalize();
  }                                                        // void finalize(0)

  protected void Init_Variables() {
    // Method to initialise the class variables because SimAnim does not 
    // destroy and recreate the objects.

      site_state_num = 0;
      site_choice_num = 0;
      stored_msg = null;

      coordinator = false; 
      participant = false;

      global_decision = UNKNOWN;
      local_decision  = UNKNOWN;
      local_msg_round = 0;

      PART_coord_id = UNKNOWN;

      times_failed = 0;

      this.Setup_Window();

  }                                                  // void Init_Variables(0)


  private void Set_Id(int id) {
      // Method to set the id number of the site
      if ((id >= 1) && (id <= Option.Num_Sites())) {
          this.site_id = id;
      }
      else {
          Error("id "+id+" invalid in Set_Id(1)");
      }
  }                                                          // void Set_Id(1)

  protected final int Id() {
      // Method to return the id number of the site
      return this.site_id;
  }                                                               // int Id(0)

  private void Set_Type(int type) {
      // Method to set the type of the site
      if ((type == COORD) || (type == PART) || (type == SITE)) {
          this.site_type = type;
      }
      else {
          Error("type "+type+" invalid in Set_Type(1)");
      }
  }                                                        // void Set_Type(1)

  protected final int Type() {
      // Method to return the type of the site
      return this.site_type;
  }                                                             // int Type(0)

  protected final void Set_State_Num(int num) {
      // Method to set the state site is in
      if ((Type() == COORD) || (Type() == PART)) {
          this.site_state_num = num;
          Trace_P();
      }
  }                                                   // void Set_State_Num(1)

  protected final int State_Num() {
      // Method to return the state a site is in
      return this.site_state_num;
  }                                                        // int State_Num(0)
  
  protected final String State_Name() {
      // Method to return the name of a state which the site is in
      String name = new String(".");
      if (State_Num() > 0) {
          name = new String(this.Table().State_Name(Type(), State_Num()));
      }
      return name;
  }                                                    // String State_Name(0)

  protected final void Set_Choice_Num(int num) {
      // Method to set the choice site is in
      if ((Type() == COORD) || (Type() == PART)) {
          this.site_choice_num = num;
      }
  }                                                  // void Set_Choice_Num(1)

  protected final int Choice_Num() {
      // Method to return the choice site is in
      return this.site_choice_num;
  }                                                       // int Choice_Num(0)

  protected final void Set_Status(int status) {
      // Method to set the status of site
      boolean valid = false; 

      if (status == SLEEPING) {
          if ((this.Status() == RUN_OK) || (this.Status() == SLEEPING)) {
                  Trace_P("sleep");
                  valid = true;
          }
          else if (this.Status() == FAILED) {
                  Error("cannot become inactive if status is FAILED");
          }
      }
      else if (status == RUN_OK) {
	  if (this.Status() == SLEEPING) {
              Trace_P("runok");
              valid = true;
          }
          else if (this.Status() == FAILED) {
              Info(Global.Full_Type(Type())+Id()+" is recovering");
              if (Type() == PART) {
                  Global.Inc_Recovered_PART();
              }
              else {
		  // Only allowing Participants to recover at present
		  // Later, pass PART or COORD argument to function, 
		  // rather than having one specific to _PART
                  Error("Only Participant recovery handled"); 
              }
              Trace_P("runok");
              valid = true;
          }
      } 
      else if (status == FAILED) {
	  if (this.Status() == RUN_OK) {
              Info(Global.Full_Type(Type())+Id()+" is crashing");
	      if (Type() == PART) {
                  Global.Inc_Failed_PART();
                  this.Inc_Times_Failed();
	      }
              else {
		  // Only allowing Participants to fail at present
		  // Later, pass PART or COORD argument to function, 
		  // rather than having one specific to _PART
                  Error("Only Participant failure handled"); 
              }
              Trace_P("fail");
              valid = true;
 	  }
          else if (this.Status() == SLEEPING) {
              Error("cannot fail if status is inactive SLEEPING");
          }
      }
      else {
          Error("Invalid status in Set_Status(1)");
      }

      if (valid) {
          this.site_status = status;
          Info("Status: "+Global.Name_Status(this.Status()));
      }
      else {
          Debug("status: "+Global.Name_Status(this.Status())
                                                     +" - remains unchanged");
      }
  }                                                      // void Set_Status(1)

  protected final int Status() {
      // Method to return the status of site
      return this.site_status;
  }                                                           // int Status(0)

  protected final void Store_Current_Msg(Message msg) {
      // Method to store the current message being used
      if (msg == null) {
          Debug("storing null message");
          this.stored_msg = null;
      }
      else {
          Debug("storing message - "+msg.Name());
          this.stored_msg = msg.Clone();          
      }
  }                                               // void Store_Current_Msg(1)
 
  protected final Message Stored_Message() {
      // Method to return the current message stored.
      Message msg = null;
 
      if (this.stored_msg != null) {
          msg = this.stored_msg.Clone();
      }
      return msg;
  }                                               // Message Stored_Message(0)
 
  protected final boolean Is_Coordinator() {
      // Method to determine if a site is a coordinator.
      if (this.coordinator) {
          return true;
      }
      else {
          return false;
      }
  }                                               // boolean Is_Coordinator(0)

  protected final boolean Is_Participant() {
      // Method to determine if a site is a participant.
      if (this.participant) {
          return true;
      }
      else {
          return false;
      }
  }                                               // boolean Is_Participant(0)

  protected final Table Table() {
      // Method to return the current table
      return Option.Table();
  }                                                          // Table Table(0)

  protected final void Set_Role_Of_Site(int role) {
      // Method to set a site to have a participant and/or coordinator.
      Debug("Role to be changed to "+Global.Full_Type(role));
      if ((Type() == COORD) || (Type() == SITE)) {
  	  if ((role == COORD) || (role == BOTH)) {
              if (this.Is_Coordinator()) {
                  Error("Role is already as an active coordinator");
              }
              else {
                  this.coordinator = true;
              }
	  }
 	  else if (role == NONE) {
              if (this.Is_Coordinator()) {
                  this.coordinator = false;
                  Debug("Role is no longer as an active coordinator");
              }
              else {
                  Debug("Role was not as an active coordinator anyway");
              }
          }
      }
      if ((Type() == PART) || (Type() == SITE)) {
  	  if ((role == PART) || (role == BOTH)) {
              if (this.Is_Participant()) {
                  Error("Role is already as an active participant");
              }
              else {
                  this.participant = true;
              }
          }
 	  else if (role == NONE) {
              if (this.Is_Participant()) {
                  this.participant = false;
                  Debug("Role is no longer as an active participant");
              }
              else {
                 Debug("Role was not as a participant anyway");
              }
          }
      }
      this.Display_Role_Of_Site();
  }                                                // void Set_Role_Of_Site(0)

  protected final void Display_Role_Of_Site() {
      // Method to output if a site has a participant, coordinator or both. 
      if (this.Is_Coordinator()) {
          Debug("Role is as an active coordinator");
      }
      if (this.Is_Participant()) {
          Debug("Role is as an active participant");
      }
  }                                            // void Display_Role_Of_Site(0)

  protected final void Set_Coord_Id_Of_Part(int id) {
      // Method to set the ID of the participant's coordinator
      if (id < 1) { 
          // Consider error checking. 
          Error("Coord ID "+id+" out of range - Set_Coord_Id_Of_Part(1)");
      }
      else if (this.Is_Participant()) {
          this.PART_coord_id = id;
          Debug("Set Participant's Coord Id: "+id);
      } 
      else {
          Error("Cannot set Coord Id - Set_Coord_Id_Of_Part(1)");
      }
  }                                            // void Set_Coord_Id_Of_Part(1)

  protected final int Coord_Id_Of_Part() {
      // Method to return the ID of the participant's coordinator.
      int c_id = UNKNOWN;
      if (this.Is_Participant()) {
          c_id = this.PART_coord_id;
      }
      else {
          Error("has no coordinator to return id of - Coord_Id_Of_Part(0)");
      }
      return c_id;
  }                                                 // int Coord_Id_Of_Part(0)

  protected final void Inc_Local_Msg_Round() {
      // Method to increment the site's local message round - not global
      this.local_msg_round++;
      Debug("\t\t\tLocal message round "+this.Local_Msg_Round());
      Debug("\t\t\tGlobal message round "+Global.Message_Round());
  }                                             // void Inc_Local_Msg_Round(0)

  protected final int Local_Msg_Round() {
      // Method to return the site's local message round - not global
      return this.local_msg_round;
  }                                                  // int Local_Msg_Round(0)

  protected final void Set_Local_Decision(int decision) {
      // Method to set the local decision of participant/site
      if ((Type() == PART) || (Type() == SITE)) {
 	  if (this.local_decision == UNKNOWN) {
              this.local_decision = decision;
              Debug("setting local decision to "+Global.Name_CorA(decision));
          }
          else {
              Error("Local decsion already set to "
                                     +Global.Name_CorA(this.local_decision));
          }
      }
      else {
          Error("cannot set local decision");
      }
  }                                             // void Set_Local_Decision(1)

  protected final int Local_Decision() {
      // Method to return the local decision of participant/site
      int decision = UNKNOWN;

      if ((Type() == PART) || (Type() == SITE)) {
          decision = this.local_decision;
      }
      else {
          Error("cannot set local decision");
      }
      return decision;
  }                                                  // int Local_Decision(0)

  protected final void Set_Global_Decision(int decision) {
      // Method to set the global decision of coordinator/participant/site
      if (this.global_decision == UNKNOWN) {
          this.global_decision = decision;
          Debug("setting global decision to "+Global.Name_CorA(decision));
      }
      else if (global_decision == ABORT) {
          if (decision == ABORT) {
              Debug("Global decision already set to ABORT");
          } 
          else {
              Error("Global decision already set to ABORT");
              Error("Why change to "+Global.Name_CorA(decision)+"??");
          }
      }
      else {
          Debug("Global decision changed from "
               +Global.Name_CorA(this.global_decision)
               +", to "+Global.Name_CorA(decision));
      }
  }                                             // void Set_Global_Decision(1)

  protected final int Global_Decision() {
      // Method to return the global decision of coordinator/participant/site
      int decision;

      decision = this.global_decision;

      return decision;
  }                                                  // int Global_Decision(0)


  private void Inc_Times_Failed() {
      // Method to increment the number of times a failure has occured 
      // in the same state.
      if (this.Type() == PART) {
          this.times_failed++;
          Debug("Incrementing times failed");
      }
      else {
          Error("Can only increment for Participant failures"); 
      }
  }                                                // void Inc_Times_Failed(0)

  protected void Reset_Times_Failed() {
      // Method to reset the number of times a failure has occured 
      // in the same state.
      if (this.Type() == PART) {
          this.times_failed = 0;
          Debug("Reset times failed");
      }
      else {
          Error("Can only increment for Participant failures"); 
      }
  }                                              // void Reset_Times_Failed(0)

  private int Times_Failed() {
      // Method to return the number of times a failure has occured 
      // in the same state.
      int times = 0;
      if (this.Type() == PART) {
          times = this.times_failed;
      }
      else {
          Error("Can only increment for Participant failures"); 
      }
      return times;
  }                                                     // int Times_Failed(0)

  protected final boolean Determine_Failure() {
      // Method to determine if the Part (or coord) should fail
      boolean fail_point = false;

      if (this.Type() == PART) {

          switch (Option.F_Method()) {
              case ALWAYS : fail_point = Check_F_Always();
                            break;
              case PROMPT : fail_point = Prompt_F_User();
                            break;
              case INTERVAL : fail_point = Check_F_Interval();
                              break;
              case STATES : fail_point = Check_F_State();
                            break;
              case RANDOM : fail_point = Check_F_Dist();
                            break;	
              default:
              case NEVER : fail_point = false;
                           break;
          }
      }
      else {
          Error("Can only handle Participant failures");
      }

      Debug("Times failed in this state: "+this.Times_Failed());
      if (this.Times_Failed() >= DETECT_FAILURE_LOOP) {
 	  // If fail/recover cycle detected then force non-failure
          Info("");
          Info("*** Detected a fail/recover cycle \t***");
          Info("*** Possibly from continual failure \t***");
          Info("*** Model has blocked due to cycle \t***"); 
          Info("*** Forcing non-failure to break cycle \t***\n");
              
          Global.Inc_FailR_Cycles();
          fail_point = false;
      }
 
      return fail_point;
  }                                            // boolean Determine_Failure(0)

  protected final boolean Determine_Recovery() {
      // Method to determine if the Part (or coord) should recover
      boolean recover_point = false;

      if (this.Type() == PART) {
 
          switch (Option.R_Method()) {
              case NEVER : recover_point = false;
                           break;
              case PROMPT : recover_point = Prompt_R_User();
                            break;
              case INTERVAL : recover_point = Check_R_Interval();
                              break;
              case RANDOM : recover_point = Check_R_Dist();
                            break;	
              default:
              case ALWAYS : recover_point = true;
                            break;
          }
      }
      else {
          Error("Can only handle Participant recovery");
      }
      return recover_point;
  }                                           // boolean Determine_Recovery(0)

  private boolean Check_F_Always() {
      // Method to check that failure can occur in state
      // true if should fail
      // REMEMBER: CANNOT FAIL IN SOME STATES
      boolean fail_point = false;

      if (this.Type() == PART) {
	  if (Table().Fail_Possible(PART, this.State_Num())) {
              fail_point = true;
	  }
	  else {
              Debug("Always - cannot fail in this state");
	  }
      }
      else {
          Error("Can only handle Participant failure");
      }
      return fail_point;
  }                                               // boolean Check_F_Always(0)

  private boolean Prompt_F_User() {
      // Method to prompt the user to specify failures
      // true if should fail

      // REMEMBER: CANNOT FAIL IN SOME STATES

      if (this.Type() == PART) {
          Debug("Failure Prompt: - DO NOT FAIL");
      }
      else {
          Error("Can only handle Participant failure");
      }
      return false;      // CHANGE LATER
  }                                                // boolean Prompt_F_User(0)

  private boolean Check_F_Interval() {
      // Method to check the failure message interval and return 
      // true if should fail
      boolean f_point = false;

      // REMEMBER: CANNOT FAIL IN SOME STATES

      if (this.Type() == PART) {
	  if (Table().Fail_Possible(PART, this.State_Num())) {
              if (Global.Num_Sent_Msgs() >= Option.F_Interval()) {
                  f_point = true;
                  Debug("Failure Interval: number sent "+Global.Num_Sent_Msgs()
                        +", interval "+Option.F_Interval()+" - FAIL");
                  Global.Reset_Sent_Msgs();
              } 
              else {
                  Debug("Failure Interval: number sent "+Global.Num_Sent_Msgs()
                        +", interval "+Option.F_Interval()+" - DO NOT FAIL");
              }
          }
          else {
              Debug("Interval - cannot fail in this state");
	  }
      }
      else {
          Error("Can only handle Participant failure");
      }

      return f_point;
  }                                             // boolean Check_F_Interval(0)

  private boolean Check_F_State() {
      // Method to check the failure in state and return 
      // true if should fail
      int     state = 0;
      boolean f_point = false;

      if (this.Type() == PART) {
          state = this.State_Num();
          if (Global.Num_Have_Failed(state) < Option.Num_To_Fail(state)) {
              f_point = true;
              Debug("Failure States: - FAIL ("
                    +Global.Num_Have_Failed(state)+" < "
                    +Option.Num_To_Fail(state)+" in state "+state+")");
              Global.Inc_Num_Have_Failed(state);
          }  
	  else {
              Debug("Failure States: - DO NOT FAIL ("
                    +Global.Num_Have_Failed(state)+" >= "
                    +Option.Num_To_Fail(state)+" in state "+state+")");
          }
      }
      else {
          Error("Can only handle Participant failure");
      }
      return f_point;   // CHANGE LATER
  }                                                // boolean Check_F_State(0)

  private boolean Check_F_Dist() {
      // Method to check the failure by distribution and return 
      // true if should fail

      // REMEMBER: CANNOT FAIL IN SOME STATES

      if (this.Type() == PART) {
          Debug("Failure Dist: - DO NOT FAIL");
      }
      else {
          Error("Can only handle Participant failure");
      }
      return false;      // CHANGE LATER
  }                                                 // boolean Check_F_Dist(0)

  private boolean Prompt_R_User() {
      // Method to prompt the user to specify recovery
      // true if should fail

      if (this.Type() == PART) {
          Debug("Recovery Prompt: - RECOVER");
      }
      else {
          Error("Can only handle Participant recovery");
      }
      return false;      // CHANGE LATER
  }                                                // boolean Prompt_R_User(0)

  private boolean Check_R_Interval() {
      // Method to check the recovery message interval and return 
      // true if should recover
      boolean r_point = false;

      if (this.Type() == PART) {
          if (Global.Num_Rec_Msgs() >= Option.R_Interval()) {
              r_point = true;
              Debug("Recpvery Interval: number rec "+Global.Num_Rec_Msgs()
                    +", interval "+Option.R_Interval()+" - RECOVER");
              Global.Reset_Rec_Msgs();
          } 
          else {
              Debug("Recovery Interval: number rec "+Global.Num_Rec_Msgs()
                    +", interval "+Option.R_Interval()+" - DO NOT RECOVER");
          }
      }
      else {
          Error("Can only handle Participant recovery");
      }
      return r_point;
  }                                             // boolean Check_R_Interval(0)

  private boolean Check_R_Dist() {
      // Method to check the recovery by distribution and return 
      // true if should recover

      if (this.Type() == PART) {
          Debug("Recovery Dist: - RECOVER");
      }
      else {
          Error("Can only handle Participant recovery");
      }
      return true;      // CHANGE LATER
  }                                                 // boolean Check_R_Dist(0)


  protected final void Trace_Msg_Sent(Message msg, String port_name) {
      // Method to output trace that message was sent from site,
      // on the port named.
      String note;

      note = msg.Sent_Data(Type());

      if ((!msg.Control()) || ((msg.Control()) && (Option.Show_Debug()))) {
          if (msg.Src_Type() == Type()) {
              Trace("sent "+note); 
              Debug("sent "+note);
          }
          else {
	      // Message sent through site but did not originate here 
              Trace("(sent) forwarding "+note);
	      if (Type() == SITE) {
                  Route("(sent) forwarding "+note);
              }
	      else {
                  Debug("(sent) forwarding "+note);
	      }
          }
      }
      // Added for SimAnim
      if (msg.Control()) {
  	 if (Option.Show_Ctrl_Messages()) {   
             Trace_S(port_name, msg.Name().toLowerCase());
         }
      }
      else{
         Trace_S(port_name, msg.Name().toUpperCase());
      }
  }                                                  // void Trace_Msg_Sent(2)

  protected final void Trace_Msg_Received(Message msg) {
      // Method to output trace that message was received at site.
      String note;

      note = msg.Rec_Data(Type());

      if ((!msg.Control()) || ((msg.Control()) && (Option.Show_Debug()))) {
          if (msg.Dest_Type() == Type()) {
              Trace("received "+note);
              Debug("received "+note);
          }
          else {
  	      // Message sent through site but did not originate here 
              Trace("(rec) forwarding "+note);
	      if (Type() == SITE) {
                  Route("(rec) forwarding "+note);
              }
	      else {
                  Debug("(rec) forwarding "+note);
              }
          }
      }
  }                                              // void Trace_Msg_Received(1)

  private void Setup_Window() {
      // Method to setup a site's window on screen 
      if (Option.Display_GUI()) {
          Debug("Setting up GUI window");

          if ((Type() == COORD) || (Type() == PART)) {
              this.site_window = new W_Box(Id(), Global.Full_Type(Type()));
          }
          else if (Type() == SITE) {
              this.site_window = new W_Box(Global.Full_Type(Type())+Id());
          }
          else {
              Error("Invalid type setting up GUI");
          }
      } 
  }                                                    // void Setup_Window(0)

  protected final void Reset_Window() {
      // Method to reset the window without killing it
      if (this.site_window != null) {
          this.Hide_Window();
          this.site_window.Clear();      
      }
  }                                                    // void Reset_Window(0) 

  protected final void Show_Window() {
      // Method to hide a site's window on screen 
      if ((Type() == SITE) && (!Option.Show_Debug())) {
      }
      else {
          Debug("Showing GUI window");
          Global.Window_Visible(this.site_window, true);
      }
  }                                                     // void Show_Window(0)

  protected final void Hide_Window() {
      // Method to hide a site's window on screen 
      Debug("Hiding GUI window");
      Global.Window_Visible(this.site_window, false); 
  }                                                     // void Hide_Window(0)

  private void Close_Window() {
      // Method to close a site's window on screen 
      if (this.site_window != null) {
          this.site_window.dispose();
      } 
      /* Removed - smr                                  (used for user prompt)
      ** if (this.site_prompt != null) {
      **     this.site_prompt.dispose();
      ** }
      */
  }                                                    // void Close_Window(0)

  protected final W_Box Site_Window() {
      // Method to return the site's gui 
      W_Box s_window = null; 
      if ((Type() == COORD) || (Type() == PART)) {
          s_window = this.site_window;
      }
      else {
          Error("cannot return site window");
      }
      return s_window;
  }                                                    // W_Box Site_Window(0)

  protected final void Debug(String note) {
      // Method to output debugging comments
      Global.Debug_CLI(Global.Short_Type(Type())+Id()+": "+note);
      this.Debug_GUI(note);
  }                                                           // void Debug(1)

  private void Debug_GUI(String note) {
      // Method to output debugging comments via graphical user interface
      if ((Option.Show_Debug()) && (Option.Display_GUI())) {
   	  if (site_window != null) {
              site_window.Add_Output_Entry(note);  
          }
      }
  }                                                       // void Debug_GUI(1)
 
  protected final void Error(String note) {
      // Method to output error messages
      Global.Error_CLI(Global.Short_Type(Type())+Id()+": "+note);
      this.Error_GUI(note);
  }                                                           // void Error(1)

  private void Error_GUI(String note) {
      // Method to output error messages via graphical user interface
      if ((Option.Show_Error()) && (Option.Display_GUI())) {
	  if (site_window != null) {
              site_window.Add_Error_Entry(note);  
          }
      }
  }                                                       // void Error_GUI(1)

  protected final void Info(String note) {
      // Method to output information if required
      Global.Info_CLI(Global.Short_Type(Type())+Id()+": "+note);
      this.Info_GUI(note);
  }                                                            // void Info(1)

  private void Info_GUI(String note) {
      // Method to output info via graphical user interface
      if ((Option.Show_Info()) && (Option.Display_GUI())) {
	  if (site_window != null) {
              site_window.Add_Output_Entry(note);  
          }
      }
  }                                                        // void Info_GUI(1)

  protected final void Route(String note) {
      // Method to output route debugging comments
      if (Type() == SITE) {
	  if (Option.Show_Routing()) {
              Debug(note);
         }
      }
      else {
          Error("Cannot use Route(1) method");
      }
  }                                                           // void Route(1) 

  protected final void Trace_S(String port, String note) {
      // Method to output sending of event traces in tracefile
      // port : name of port event sent from
      // note : name of event

      if ((SIMANIM) && (Option.Show_Messages())) {
  	  // Only for SimAnim version
          sim_trace(1, "S "+port+" "+note);
          Debug("Update port, note: "+port+", "+note);
      }
  }                                                         // void Trace_S(2)

  protected final void Trace_P(String icon) {
      // Method to output entity parameter traces in tracefile
      // icon : name of icon being displayed
      String state = new String(State_Name());

      if (icon.compareTo("fail") == 0) {
	  state = new String("CRASHED");
      }

      if ((SIMANIM) && (Type() != SITE)) {
  	  // Only for SimAnim version
          sim_trace(1, "P "+state+" "+icon);
          Debug("Update state, icon: "+state+", "+icon);
      }
  }                                                         // void Trace_P(1)

  protected final void Trace_P() {
      // Method to output entity parameter traces in tracefile
      // icon : name of icon being displayed
      String icon = new String("sleep");

      if (Status() == RUN_OK) {
          icon = new String("runok");
      }
      else if (Status() == FAILED) {
          icon = new String("fail");
      }
      Trace_P(icon);
  }                                                         // void Trace_P(0)

  protected final void Trace(String note) {
      // Method to output trace in tracefile
      if ((!SIMANIM) && (Option.Show_Trace())) {
  	  // Must not add extra traces if SimAnim version
          sim_trace(1, Global.Short_Type(Type())+Id()+": "+note);
      }
  }                                                           // void Trace(1)

}                                                       // abstract class Site

/*---------------------------- End of Site.java -----------------------------*/

