
/*
** Author : Steven Ramsay <smr@dcs.ed.ac.uk>, 31/01/97
** Files  : Site_Lists.java, Site_Lists.class
** Update : 11/06/97 
** Package: simdcp
**
** Description:
** Maintains all lists for coordinator and participant at a site.
*/

package eduni.simdcp;

/* Methods defined in final class Site_Lists
** PUBLIC
** Site_Lists(int, int, W_Box)
**
** PROTECTED
**        void    Reset_Replies_From_Parts_List() 
**        void    Copy_Parts_In_Txn_List(int[]) 
**        int[]   Parts_In_Txn_List() 
**        int[]   Replies_From_Parts_List() 
**        int[]   Reply_To_Parts_List() 
**        int     Num_Replies_From_Parts()
**        int     Num_Reply_To_Parts()
**        void    Set_Part_Replied(int, int)
**        void    Update_Reply_To_Parts_List(int[])
**        int     Has_Part_Replied(int)
**        boolean Was_Part_Sent_Msg(int)
**        void    Display_Parts_In_Reply_List() 
**        void    Write_DT_Log(Log)
**        Log     Read_DT_Log()
**
** PRIVATE
**        int     Id() 
**        int     Type() 
**        void    Reset_Reply_To_Parts_List() 
**        void    Reset_DT_Log()
**        int     Num_Parts_In_Txn()
**        void    Add_DT_Log_Entry(Log)
**        Log     DT_Log_Entry()
**        boolean Is_Part_In_Txn(int)
**        void    Display_Parts_In_Txn_List() 
**        void    Log_Window(String)
**        void    Debug(String)
**        void    Debug_GUI(String)
**        void    Error(String)
**        void    Error_GUI(String)
**        void    Info(String)
**        void    Info_GUI(String)
*/
final class Site_Lists implements Constants {
  private int   SITE_id   = UNKNOWN;
  private int   SITE_type = UNKNOWN;
  private W_Box SITE_window = null;

  private int[] SITE_parts_in_txn_list        = null;
  private int[] COORD_reply_to_parts_list     = null;
  private int[] COORD_replies_from_parts_list = null;

  private Log[] SITE_dt_log = null;


  public Site_Lists(int type, int id, W_Box site_window) {
      // Constructor for creating lists for coord or part on a site.
      this.SITE_id                           = id;
      this.SITE_parts_in_txn_list            = new int[MAX_SITES+1]; 
      this.SITE_dt_log                       = new Log[MAX_LOG_ENTRIES+1];
      this.SITE_window                       = site_window;
      if (type == COORD) {
          this.SITE_type                     = COORD;
          this.COORD_reply_to_parts_list     = new int[MAX_SITES+1];
          this.COORD_replies_from_parts_list = new int[MAX_SITES+1]; 
      }
      else if (type == PART) {
          this.SITE_type                     = PART;
      }
      else {
          Error("Unknown type in constructor Site_Lists(3)");
      }
  }                                               // constructor Site_Lists(2)

  private int Id() {
      // Method to return the id number of the site (participant/coordinator)
      return this.SITE_id;
  }                                                               // int Id(0)

  private int Type() {
      // Method to return the type of the site
      return this.SITE_type;
  }                                                             // int Type(0)

  protected void Reset_Replies_From_Parts_List() {
      // Method to reset the replied from parts list ready for next round
      int i;

      if (this.Type() == COORD) {
          for (i=0; i <= Option.Num_Sites(); i++) {
               this.COORD_replies_from_parts_list[i] = 0;
          }
          Debug("Reset replies from parts list");
      }
      else {
          Error("type not yet handled in Reset_Replies_From_Parts_List(0)");
      }
  }                                   // void Reset_Replies_From_Parts_List(0)

  private void Reset_Reply_To_Parts_List() {
      // Method to reset the reply to parts list ready for next message round
      int i;

      if (this.Type() == COORD) {
          for (i=0; i <= Option.Num_Sites(); i++) {
               this.COORD_reply_to_parts_list[i] = 0;
          }
          Debug("Reset reply to parts list");
      }
      else {
          Error("type not yet handled in Reset_Reply_To_Parts_List(1)");
      }
  }                                        // int Reset_Reply_To_Parts_List(0)

  private void Reset_DT_Log() {
      // Method to reset the Distributed Transaction Log for a site
      int i;

      for (i=0; i <= MAX_LOG_ENTRIES+1; i++) {
           this.SITE_dt_log[i] = null;
      }
      Debug("Reset DT Log");
  }                                                    // void Reset_DT_Log(0)


  protected void Copy_Parts_In_Txn_List(int[] msg_p_list) {
      // Method to copy the part list from message to coord/part's list
      System.arraycopy(msg_p_list, 0, this.SITE_parts_in_txn_list, 
                       0, Option.Num_Sites()+1);
      this.Display_Parts_In_Txn_List();        // Display list of Participants
  }                                          // void Copy_Parts_In_Txn_List(1)


  protected int[] Parts_In_Txn_List() {
      // Method to return the list of participants in txn
      int[] p_list = new int[MAX_SITES+1];

      System.arraycopy(this.SITE_parts_in_txn_list, 0, 
                           p_list, 0, Option.Num_Sites()+1);

      return p_list;
  }                                              // int[] Parts_In_Txn_List(0)

  protected int[] Replies_From_Parts_List() {
      // Method to return the list of replied participants
      int[] replies_from_list = new int[MAX_SITES+1];

      if (this.Type() == COORD) {
          System.arraycopy(this.COORD_replies_from_parts_list, 0, 
                                 replies_from_list, 0, Option.Num_Sites()+1);
      }
      else {
          Error("type not yet handled in Replies_From_Parts_List(0)");
      }
      return replies_from_list;
  }                                        // int[] Replies_From_Parts_List(0)

  protected int[] Reply_To_Parts_List() {
      // Method to return the list of participants sent messages
      int[] reply_to_list = new int[MAX_SITES+1];

      if (this.Type() == COORD) {
          System.arraycopy(this.COORD_reply_to_parts_list, 0, 
                               reply_to_list, 0, Option.Num_Sites()+1);
      }
      else {
          Error("type not yet handled in Reply_To_Parts_List(0)");
      }
      return reply_to_list;
  }                                            // int[] Reply_To_Parts_List(0)


  private int Num_Parts_In_Txn() {
      // Method to return the number of participants in txn list,
      // number held in index zero.
      int num = 0;

      num = this.SITE_parts_in_txn_list[0];

      return num;
  }                                                 // int Num_Parts_In_Txn(0)
 
  protected int Num_Replies_From_Parts() {
      // Method to return the number of participants who have replied in
      // current message round, number held in index zero.
      int num = 0;

      if (this.Type() == COORD) {
          num = this.COORD_replies_from_parts_list[0];
      }
      else {
          Error("type not yet handled in Num_Replies_From_Parts(0)");
      }
      return num;
  }                                           // int Num_Replies_From_Parts(0)

  protected int Num_Reply_To_Parts() {
      // Method to return the number of participants sent a message
      // current message round, number held in index zero.
      int num = 0;

      if (this.Type() == COORD) {
          num = this.COORD_reply_to_parts_list[0];
      }
      else {
          Error("type not yet handled in Num_Reply_To_Parts(0)");
      }
      return num;
  }                                               // int Num_Reply_To_Parts(0)


  protected void Set_Part_Replied(int id, int reply_val) {
      // Method to keep track of participants who have replied.
      if (this.Type() == COORD) {
          if (this.Was_Part_Sent_Msg(id)) {
	      if ((this.Has_Part_Replied(id) == UNKNOWN) 
	                         || (this.Has_Part_Replied(id) == FAILED)) {
 	  	  if ((reply_val == COMMIT) || (reply_val == ABORT)) {
                      this.COORD_replies_from_parts_list[0]++;
		  }
                  this.COORD_replies_from_parts_list[id] = reply_val;
                  Debug("set part"+id+" replied: "+reply_val); 
              }
              else {
                 Debug("Part"+id+" already replied - ignoring");
              }
          }
          else {
              Error("Part"+id+" was not sent message");
          }
      }
      else {
          Error("type not yet handled in Set_Part_Replied(2)");
      }
  }                                                 // int Set_Part_Replied(2)

  protected void Update_Reply_To_Parts_List(int[] p_list) {
      // Method to update the reply to list with the current list of parts
      // who have replied.  If a reply is to ABORT then must be excluded.
      int id, reply;

      if (this.Type() == COORD) {
          this.Reset_Reply_To_Parts_List();
          for (id=1; id <= Option.Num_Sites(); id++) {
              reply = p_list[id];
	      if ((reply == COMMIT) || (reply == FAILED)) { 
                  this.COORD_reply_to_parts_list[id] = reply;
                  this.COORD_reply_to_parts_list[0]++;  
                  Debug("Part"+id+" replied "+reply+" - so included");
	      }
              else {
                  Debug("Part"+id+" replied "+reply+" - so excluded");
              }
          }
          Debug("Parts in list to send to "+this.Num_Reply_To_Parts());
      }
      else {
          Error("type not yet handled in Update_Reply_To_Parts_List(1)");
      }
  }                                      // void Update_Reply_To_Parts_List(1)

  private void Add_DT_Log_Entry(Log log) {
      // Method to write an entry in the site's distributed transaction log
      int     i    = 1;
      boolean stop = false;

      while ((i <= MAX_LOG_ENTRIES) && (!stop)) {
	  if (this.SITE_dt_log[i] == null) {
              this.SITE_dt_log[i] = log;
              stop = true;
          }
          i++;
      }
  }                                                // void Add_DT_Log_Entry(1)

  private Log DT_Log_Entry() {
      // Method to read an entry in the site's distributed transaction log
      int    i   = 1;
      Log    log;

      while (this.SITE_dt_log[i] != null) {
          i++;
      }
      log = this.SITE_dt_log[i-1];
      return log;
  }                                                     // Log DT_Log_Entry(0)


  private boolean Is_Part_In_Txn(int id) {
      // Method to return whether or not a participant is in the txn,
      // i.e. to be sent initial messages.
      boolean in_txn = false;                                       // default

      if (id <= 0) {
  	  // Error checking - no part. has id 0.
          Error("Part. ID "+id+" out of range.");
      }
      else {
          if (this.SITE_parts_in_txn_list[id] == YES) {
              in_txn = true;
          }
      }
      return in_txn;
  }                                                   // int Is_Part_In_Txn(1)

  protected int Has_Part_Replied(int id) {
      // Method to return a participants reply from current message round,
      // or UNKNOWN if it hasn't replied.
      int reply = UNKNOWN;                                          // default

      if (id <= 0) {
  	  // Error checking - no part. has id 0 - at present.
          Error("Part. ID "+id+" out of range.");
      }
      else {
  	  if (this.Type() == COORD) {
              if (this.Is_Part_In_Txn(id)) {
                  reply = this.COORD_replies_from_parts_list[id];
              }
          }
          else {
              Error("type not yet handled in Has_Part_Replied(1)");
          }
      }
      return reply;
  }                                                 // int Has_Part_Replied(1)

  protected boolean Was_Part_Sent_Msg(int id) {
      // Method to return a participants inclusion/exclusion status from 
      // being sent a message in current message round.
      boolean sent_msg = false;                                     // default

      if (id <= 0) {
  	  // Error checking - no part. has id 0 - at present.
          Error("Part. ID "+id+" out of range.");
      }
      else {
  	  if (this.Type() == COORD) {
  	      if (this.Is_Part_In_Txn(id)) {
	          if ((this.COORD_reply_to_parts_list[id] == COMMIT) 
                           || (this.COORD_reply_to_parts_list[id] == ABORT)) {
                      sent_msg = true;
                  }
              }
          }
          else {
              Error("type not yet handled in Was_Part_Sent_Msg(1)");
          }
      }
      return sent_msg;
  }                                            // boolean Was_Part_Sent_Msg(1)


  private void Display_Parts_In_Txn_List() {
      // Method to display the list of participants in txn for a Coord/Part.
      int id;

      for (id=1; id <= Option.Num_Sites(); id++) {
          if (this.Is_Part_In_Txn(id)) {
              Debug("\tPart"+id+" is in list");
          }
      }
      Debug("Num. of parts in list "+Num_Parts_In_Txn());
  }                                       // void Display_Parts_In_Txn_List(0)

  protected void Display_Parts_In_Reply_List() {
      // Method to display the list of participants in reply list
      int id, count = 0;

      for (id=1; id <= Option.Num_Sites(); id++) {
  	  if (this.COORD_replies_from_parts_list[id] != UNKNOWN) {
              Debug("\tPart"+id+" - "
                  +Global.Name_CorA(this.COORD_replies_from_parts_list[id])
                  +" is in reply list");
              count++;
          }
      }
      Debug("Num. of parts in reply list "+count);
  }                                     // void Display_Parts_In_Reply_List(0)



  protected void Write_DT_Log(Log log) {
      // Method to write an entry in the site's distributed transaction log
      this.Add_DT_Log_Entry(log);
      Debug("DT Log write - "+log.Name());
      this.Log_GUI(log.Name());
      Global.Inc_DT_Log_Writes();
  }                                                    // void Write_DT_Log(1)

  protected Log Read_DT_Log() {
      // Method to write an entry in the site's distributed transaction log
      Log    log_entry;
      String name;
      log_entry = this.DT_Log_Entry();
      name = log_entry.Name();
      Debug("read "+name+" from DT Log");
      return log_entry;
  }                                                      // Log Read_DT_Log(0)

  private void Log_GUI(String note) {
      // Method to output log entry via GUI
      if ((Option.Show_DTLog()) && (Option.Display_GUI())) {
	  if (this.SITE_window != null) {
              this.SITE_window.Add_Log_Entry(note);  
          }
      }
  }                                                         // void Log_GUI(2)

  private void Debug(String note) {
      // Method to output debugging comments
      Global.Debug_CLI(Global.Short_Type(this.Type())+Id()+": "+note);
      this.Debug_GUI(note);
  }                                                           // void Debug(0)

  private void Debug_GUI(String note) {
      // Method to output debugging comments via graphical user interface
      if ((Option.Show_Debug()) && (Option.Display_GUI())) {
	  if (this.SITE_window != null) {
              this.SITE_window.Add_Output_Entry(note);  
          }
      }
  }                                                       // void Debug_GUI(1)

  private void Error(String note) {
      // Method to output error messages
      Global.Error_CLI(Global.Short_Type(this.Type())+Id()+": "+note);
      this.Error_GUI(note);
  }                                                           // void Error(1)


  private void Error_GUI(String note) {
      // Method to output error messages via graphical user interface
      if ((Option.Show_Error()) && (Option.Display_GUI())) {
	  if (this.SITE_window != null) {
              this.SITE_window.Add_Error_Entry(note);  
          }
      }
  }                                                       // void Error_GUI(1)

  private void Info(String note) {
      // Method to output info messages
      Global.Info_CLI(Global.Short_Type(this.Type())+Id()+": "+note);
      this.Info_GUI(note);
  }                                                            // void Info(1)


  private void Info_GUI(String note) {
      // Method to output info messages via graphical user interface
      if ((Option.Show_Info()) && (Option.Display_GUI())) {
	  if (this.SITE_window != null) {
              this.SITE_window.Add_Output_Entry(note);  
          }
      }
  }                                                        // void Info_GUI(1)

}                                                          // class Site_Lists

/*---------------------------- End of Site_Lists.java -----------------------*/

