
/*
** Author : Steven Ramsay <smr@dcs.ed.ac.uk>, 15/03/97
** Files  : States.java, States.class
** Package: simdcp
** Update : 11/06/97 
**
** Description:
** The structure representing a state in protocol
*/

package eduni.simdcp;

/* Methods defined in final class States
** PUBLIC
** States(int, String)
**
** PROTECTED
**        String  Name()
**        int     Total_Choices()
**        int     Number()
**        void    Add_x_Choices(int)
**        Choices Choice(int)
**        void    Display()
**
** PRIVATE
** static void    Error(String)
** static void    Info(String)
*/
final class States implements Constants {
  // This class represents one state in the table.  It has a starting state, 
  // i.e. the current state and it performs one possible choice of actions 
  // before making the transition to the next state.

  private int       state_num;                               // state's number 
  private int       total_choices;         // Total choices available in state
  private Choices[] choices;               // the different choices of actions
  private String    state_name;                               // name of state
    
  public States(int number, String name) {
      this.state_num     = number;
      this.state_name    = name;
      this.choices       = new Choices[MAX_CHOICES + 1];  
      this.total_choices = 0;
  }                                                   // constructor States(2)

  protected String Name() { 
      return this.state_name;
  }                                                          // String Name(0)
  
  protected int Total_Choices() {
      return this.total_choices;
  }                                                    // int Total_Choices(0)

  protected int Number() {
      return this.state_num;
  }                                                           // int Number(0)

  protected void Add_x_Choices(int num_choices) {
      int i;

      for (i=1; i <= num_choices; i++) {
          if (this.total_choices < MAX_CHOICES) {
              this.total_choices++;
              this.choices[this.total_choices] = new Choices(total_choices);
          }
          else {
              Error("Cannot add another choice to state.");
          }
      }
  }                                                   // void Add_x_Choices(1)

  protected Choices Choice(int number) {
      Choices tmp_choice = null;

      if ((number > 0) && (number <= this.Total_Choices())) {
          tmp_choice = this.choices[number];
      }
      else {
          Error("No choice exists at "+number);
      }
      return tmp_choice;
  }                                                       // Choices Choice(1)

  protected void Display() {
      int     i;
      String  note1 = new String();
      String  note2 = new String();
      Choices tmp_choice;

      note1 = new String("\t\t ( ");
      for (i=1; i <= this.Total_Choices(); i++) {
           tmp_choice = this.Choice(i);
           if (tmp_choice != null) {
               note2 = new String(note1+tmp_choice.Display());
               if (i < this.Total_Choices()) {
                   note1 = new String(note2+"\n\t\t + ");
               }
               else {
                   note1 = note2;
               }
           }
      }
      Info(note1+"\n\t\t )");
  }                                                         // void Display(0)

  private static void Error(String note) {
      // Method to output error message if required
      Scheduler.Error_Pub(note+" (class State)");
  }                                                           // void Error(1)

  private static void Info(String note) {
      // Method to output information if required
      Scheduler.Info_Pub(note);
  }                                                            // void Info(1)

}                                                              // class States

/*------------------------------ End of States.java -------------------------*/

