
/*
** Author : Steven Ramsay <smr@dcs.ed.ac.uk>, 08/01/97
** Files  : Table_2PC.java, Table_2PC.class
** Package: simdcp
** Update : 11/06/97 
**
** Description: Specific protocol represented in the table structure.
** Name       : Basic 2-Phase Commit Protocol
** Comments   : Complete - coordinator does not handle help-me messages 
*/

package eduni.simdcp;

/* Methods defined in final class Table_2PC
** PUBLIC
** Table_2PC(String)
**
** PROTECTED
**        void    Display_Stats(int)
**
** PRIVATE
**        void    SetUp_Table()
**        void    Set_State_Names()
**        void    Set_Message_Names()
**        void    Set_Log_Names()
**        void    Add_Actions()
*/
final class Table_2PC extends Table implements Constants {

  public Table_2PC(String name) {
      super.Init_Table(name);                  // Must be first in constructor
      this.Setup_Table();
  }                                                // constructor Table_2PC(1)

  private void Setup_Table() {
      Info("Constructing table: "+super.Name()+"...");
      this.Set_State_Names();
      this.Set_Message_Names();
      this.Set_Log_Names();
      super.Add_States();
      super.Set_Recover_State(COORD, 6);
      super.Set_Recover_State(PART, 6);
      this.Add_Actions();
      Info("...Finished "+super.Name());
  }                                                     // void Setup_Table(0)
      			
  private void Set_State_Names() {
      Set_State_Name(COORD, 1, "INITIAL", false);
      Set_State_Name(COORD, 2, "WAIT",    false);
      Set_State_Name(COORD, 3, "COMMIT",  false);
      Set_State_Name(COORD, 4, "ABORT",   false);
      Set_State_Name(COORD, 5, "FINAL",   false);
      Set_State_Name(COORD, 6, "RECOVER", false);

      Set_State_Name(PART, 1, "INITIAL", true );
      Set_State_Name(PART, 2, "READY",   true );
      Set_State_Name(PART, 3, "COMMIT",  true );
      Set_State_Name(PART, 4, "ABORT",   true );
      Set_State_Name(PART, 5, "FINAL",   false);
      Set_State_Name(PART, 6, "RECOVER", false);
  }                                                 // void Set_State_Names(0) 

  private void Set_Message_Names() {
      Set_Message_Name(1, "START");
      Set_Message_Name(2, "PREPARE");
      Set_Message_Name(3, "VOTE-COMMIT");
      Set_Message_Name(4, "VOTE-ABORT");
      Set_Message_Name(5, "GLOBAL-COMMIT");
      Set_Message_Name(6, "GLOBAL-ABORT");
      Set_Message_Name(7, "ACK");
      Set_Message_Name(8, "HELP-ME");
  }                                               // void Set_Message_Names(0)

  private void Set_Log_Names() {
      Set_Log_Name(1, "start-2PC");
      Set_Log_Name(2, "ready");
      Set_Log_Name(3, "commit");
      Set_Log_Name(4, "abort");
      Set_Log_Name(5, "end_of_transaction");
  }                                                   // void Set_Log_Names(0)

  private void Add_Actions() {
      States  tmp_state;
      Choices tmp_choice;

      // Coordinator 
      tmp_state = State(COORD, 1);                                  // State 1
      tmp_state.Add_x_Choices(1); 
      tmp_choice = tmp_state.Choice(1);                   // State 1, Choice 1
      tmp_choice.Add_Action(new Msg(Message_Name(1), 0, REC, UNI, NONE));
      tmp_choice.Add_Action(new Log(Log_Name(1), W, NONE));
      tmp_choice.Add_Action(new Msg(Message_Name(2), 1, SND, MLT, NONE));
      tmp_choice.Set_Next_State(2);

      tmp_state = State(COORD, 2);                                  // State 2
      tmp_state.Add_x_Choices(5);
      tmp_choice = tmp_state.Choice(1);                   // State 2, Choice 1 
      tmp_choice.Add_Action(new Ctrl(Ctrl_Name(4), COMMIT));
      tmp_choice.Add_Action(new Log(Log_Name(3), W, COMMIT));
      tmp_choice.Add_Action(new Msg(Message_Name(5), 3, SND, MLT, COMMIT));
      tmp_choice.Set_Next_State(3);
      tmp_choice = tmp_state.Choice(2);                   // State 2, Choice 2 
      tmp_choice.Add_Action(new Ctrl(Ctrl_Name(4), ABORT));
      tmp_choice.Add_Action(new Log(Log_Name(4), W, ABORT));
      tmp_choice.Add_Action(new Msg(Message_Name(6), 3, SND, MLT, ABORT));
      tmp_choice.Set_Next_State(4);
      tmp_choice = tmp_state.Choice(3);                   // State 2, Choice 3 
      tmp_choice.Add_Action(new Ctrl(Ctrl_Name(5), 2, REC, UNI, NONE));
      tmp_choice.Add_Action(new Log(Log_Name(4), W, ABORT));
      tmp_choice.Add_Action(new Msg(Message_Name(6), 3, SND, MLT, ABORT));
      tmp_choice.Set_Next_State(4);
      tmp_choice = tmp_state.Choice(4);                   // State 2, Choice 4 
      tmp_choice.Add_Action(new Msg(Message_Name(3), 2, REC, UNI, COMMIT));
      tmp_choice.Set_Next_State(2);      
      tmp_choice = tmp_state.Choice(5);                   // State 2, Choice 5 
      tmp_choice.Add_Action(new Msg(Message_Name(4), 2, REC, UNI, ABORT));
      tmp_choice.Set_Next_State(2);

      tmp_state = State(COORD, 3);                                  // State 3
      tmp_state.Add_x_Choices(3);
      tmp_choice = tmp_state.Choice(1);                   // State 3, Choice 1 
      tmp_choice.Add_Action(new Ctrl(Ctrl_Name(4), COMMIT));
      tmp_choice.Add_Action(new Log(Log_Name(5), W, COMMIT));
      tmp_choice.Add_Action(new Ctrl(Ctrl_Name(1), 0, SND, UNI, COMMIT));
      tmp_choice.Set_Next_State(5);      
      tmp_choice = tmp_state.Choice(2);                   // State 3, Choice 2 
      tmp_choice.Add_Action(new Ctrl(Ctrl_Name(5), 4, REC, UNI, COMMIT));
      tmp_choice.Add_Action(new Msg(Message_Name(5), 3, SND, UNI, COMMIT));
      tmp_choice.Set_Next_State(3);
      tmp_choice = tmp_state.Choice(3);                   // State 3, Choice 3
      tmp_choice.Add_Action(new Msg(Message_Name(7), 4, REC, UNI, COMMIT));
      tmp_choice.Set_Next_State(3);      

      tmp_state = State(COORD, 4);                                  // State 4
      tmp_state.Add_x_Choices(5); 
      tmp_choice = tmp_state.Choice(1);                   // State 4, Choice 1
      tmp_choice.Add_Action(new Ctrl(Ctrl_Name(4), ABORT));
      tmp_choice.Add_Action(new Log(Log_Name(5), W, ABORT));
      tmp_choice.Add_Action(new Ctrl(Ctrl_Name(2), 0, SND, UNI, ABORT));
      tmp_choice.Set_Next_State(5);
      tmp_choice = tmp_state.Choice(2);                   // State 4, Choice 2 
      tmp_choice.Add_Action(new Ctrl(Ctrl_Name(5), 4, REC, UNI, ABORT));
      tmp_choice.Add_Action(new Msg(Message_Name(6), 3, SND, UNI, ABORT));
      tmp_choice.Set_Next_State(4);
      tmp_choice = tmp_state.Choice(3);                   // State 4, Choice 3 
      tmp_choice.Add_Action(new Msg(Message_Name(7), 4, REC, UNI, ABORT));
      tmp_choice.Set_Next_State(4);      
      tmp_choice = tmp_state.Choice(4);                   // State 4, Choice 4 
      tmp_choice.Add_Action(new Msg(Message_Name(3), 2, REC, UNI, COMMIT));
      tmp_choice.Set_Next_State(4);
      tmp_choice = tmp_state.Choice(5);                   // State 4, Choice 5 
      tmp_choice.Add_Action(new Msg(Message_Name(4), 2, REC, UNI, ABORT));
      tmp_choice.Set_Next_State(4);

      tmp_state = State(COORD, 5);                                  // State 5
      tmp_state.Add_x_Choices(1);
      tmp_choice = tmp_state.Choice(1);                   // State 5, Choice 1 
      tmp_choice.Set_Next_State(0);

      tmp_state = State(COORD, 6);                                  // State 6
      tmp_state.Add_x_Choices(5);
      tmp_choice = tmp_state.Choice(1);                   // State 6, Choice 1 
      tmp_choice.Add_Action(new Log(Log_Name(1), R, NONE));
      tmp_choice.Add_Action(new Log(Log_Name(4), W, ABORT));
      tmp_choice.Add_Action(new Msg(Message_Name(6), 3, SND, MLT, ABORT));   
      tmp_choice.Set_Next_State(4);
      tmp_choice = tmp_state.Choice(2);                   // State 6, Choice 2 
      tmp_choice.Add_Action(new Log(Log_Name(3), R, COMMIT));
      tmp_choice.Add_Action(new Msg(Message_Name(5), 3, SND, MLT, COMMIT));
      tmp_choice.Set_Next_State(3);
      tmp_choice = tmp_state.Choice(3);                   // State 6, Choice 3 
      tmp_choice.Add_Action(new Log(Log_Name(4), R, ABORT));
      tmp_choice.Add_Action(new Msg(Message_Name(6), 3, SND, MLT, ABORT));
      tmp_choice.Set_Next_State(4);
      tmp_choice = tmp_state.Choice(4);                   // State 6, Choice 4 
      tmp_choice.Add_Action(new Log(Log_Name(5), R, COMMIT));
      tmp_choice.Add_Action(new Ctrl(Ctrl_Name(1), 0, SND, UNI, COMMIT));
      tmp_choice.Set_Next_State(5);
      tmp_choice = tmp_state.Choice(5);                   // State 6, Choice 5 
      tmp_choice.Add_Action(new Log(Log_Name(5), R, ABORT));
      tmp_choice.Add_Action(new Ctrl(Ctrl_Name(2), 0, SND, UNI, ABORT));
      tmp_choice.Set_Next_State(5);


   
      // Participant 
      tmp_state = State(PART, 1);                                   // State 1
      tmp_state.Add_x_Choices(4);
      tmp_choice = tmp_state.Choice(1);                   // State 1, Choice 1 
      tmp_choice.Add_Action(new Ctrl(Ctrl_Name(5), 1, REC, UNI, NONE));
      tmp_choice.Add_Action(new Log(Log_Name(4), W, ABORT));
      tmp_choice.Set_Next_State(4);
      tmp_choice = tmp_state.Choice(2);                   // State 1, Choice 2 
      tmp_choice.Add_Action(new Msg(Message_Name(8), 0, REC, UNI, NONE));
      tmp_choice.Add_Action(new Log(Log_Name(4), W, ABORT));
      tmp_choice.Add_Action(new Msg(Message_Name(4), 2, SND, UNI, ABORT));
      tmp_choice.Add_Action(new Msg(Message_Name(6), 4, SND, UNI, ABORT));
      tmp_choice.Set_Next_State(4);
      tmp_choice = tmp_state.Choice(3);                   // State 1, Choice 3 
      tmp_choice.Add_Action(new Ctrl(Ctrl_Name(3), COMMIT));
      tmp_choice.Add_Action(new Msg(Message_Name(2), 1, REC, UNI, NONE));
      tmp_choice.Add_Action(new Log(Log_Name(2), W, COMMIT));
      tmp_choice.Add_Action(new Msg(Message_Name(3), 2, SND, UNI, COMMIT));
      tmp_choice.Set_Next_State(2);
      tmp_choice = tmp_state.Choice(4);                   // State 1, Choice 4 
      tmp_choice.Add_Action(new Ctrl(Ctrl_Name(3), ABORT));
      tmp_choice.Add_Action(new Msg(Message_Name(2), 1, REC, UNI, NONE));
      tmp_choice.Add_Action(new Log(Log_Name(4), W, ABORT));
      tmp_choice.Add_Action(new Msg(Message_Name(4), 2, SND, UNI, ABORT));
      tmp_choice.Set_Next_State(4);

      tmp_state = State(PART, 2);                                   // State 2
      tmp_state.Add_x_Choices(6);
      tmp_choice = tmp_state.Choice(1);                   // State 2, Choice 1 
      tmp_choice.Add_Action(new Ctrl(Ctrl_Name(5), 3, REC, UNI, COMMIT));
      tmp_choice.Add_Action(new Msg(Message_Name(8), 0, SND, MLT, NONE));
      tmp_choice.Set_Next_State(2);
      tmp_choice = tmp_state.Choice(2);                   // State 2, Choice 2 
      tmp_choice.Add_Action(new Msg(Message_Name(5), 3, REC, UNI, COMMIT));
      tmp_choice.Add_Action(new Log(Log_Name(3), W, COMMIT));
      tmp_choice.Add_Action(new Msg(Message_Name(7), 4, SND, UNI, COMMIT));
      tmp_choice.Set_Next_State(3); 
      tmp_choice = tmp_state.Choice(3);                   // State 2, Choice 3
      tmp_choice.Add_Action(new Msg(Message_Name(6), 3, REC, UNI, ABORT));
      tmp_choice.Add_Action(new Log(Log_Name(4), W, ABORT));
      tmp_choice.Add_Action(new Msg(Message_Name(7), 4, SND, UNI, ABORT));
      tmp_choice.Set_Next_State(4);
      tmp_choice = tmp_state.Choice(4);                   // State 2, Choice 4 
      tmp_choice.Add_Action(new Msg(Message_Name(8), 0, REC, UNI, NONE));
      tmp_choice.Set_Next_State(2);
      tmp_choice = tmp_state.Choice(5);                   // State 2, Choice 5 
      tmp_choice.Add_Action(new Msg(Message_Name(5), 4, REC, UNI, COMMIT));
      tmp_choice.Add_Action(new Log(Log_Name(3), W, COMMIT));
      tmp_choice.Set_Next_State(3); 
      tmp_choice = tmp_state.Choice(6);                   // State 2, Choice 6
      tmp_choice.Add_Action(new Msg(Message_Name(6), 4, REC, UNI, ABORT));
      tmp_choice.Add_Action(new Log(Log_Name(4), W, ABORT));
      tmp_choice.Set_Next_State(4);

      tmp_state = State(PART, 3);                                   // State 3
      tmp_state.Add_x_Choices(4);
      tmp_choice = tmp_state.Choice(1);                   // State 3, Choice 1 
      tmp_choice.Add_Action(new Ctrl(Ctrl_Name(1), 0, REC, UNI, COMMIT));
      tmp_choice.Set_Next_State(5);
      tmp_choice = tmp_state.Choice(2);                   // State 3, Choice 2 
      tmp_choice.Add_Action(new Msg(Message_Name(8), 0, REC, UNI, NONE));
      tmp_choice.Add_Action(new Msg(Message_Name(5), 4, SND, UNI, COMMIT));
      tmp_choice.Set_Next_State(3);
      tmp_choice = tmp_state.Choice(3);                   // State 3, Choice 3 
      tmp_choice.Add_Action(new Msg(Message_Name(5), 4, REC, UNI, COMMIT));
      tmp_choice.Set_Next_State(3);
      tmp_choice = tmp_state.Choice(4);                   // State 3, Choice 4 
      tmp_choice.Add_Action(new Msg(Message_Name(5), 3, REC, UNI, COMMIT));
      tmp_choice.Add_Action(new Msg(Message_Name(7), 4, SND, UNI, COMMIT));
      tmp_choice.Set_Next_State(3);

      tmp_state = State(PART, 4);                                   // State 4
      tmp_state.Add_x_Choices(4);
      tmp_choice = tmp_state.Choice(1);                   // State 4, Choice 1 
      tmp_choice.Add_Action(new Ctrl(Ctrl_Name(2), 0, REC, UNI, ABORT));
      tmp_choice.Set_Next_State(5);
      tmp_choice = tmp_state.Choice(2);                   // State 4, Choice 2 
      tmp_choice.Add_Action(new Msg(Message_Name(8), 0, REC, UNI, NONE));
      tmp_choice.Add_Action(new Msg(Message_Name(6), 4, SND, UNI, ABORT));
      tmp_choice.Set_Next_State(4);
      tmp_choice = tmp_state.Choice(3);                   // State 4, Choice 3 
      tmp_choice.Add_Action(new Msg(Message_Name(6), 4, REC, UNI, ABORT));
      tmp_choice.Set_Next_State(4);
      tmp_choice = tmp_state.Choice(4);                   // State 4, Choice 4 
      tmp_choice.Add_Action(new Msg(Message_Name(6), 3, REC, UNI, ABORT));
      tmp_choice.Add_Action(new Msg(Message_Name(7), 4, SND, UNI, ABORT));
      tmp_choice.Set_Next_State(4);

      tmp_state = State(PART, 5);                                   // State 5
      tmp_state.Add_x_Choices(1);
      tmp_choice = tmp_state.Choice(1);                   // State 5, Choice 1 
      tmp_choice.Set_Next_State(0);

      tmp_state = State(PART, 6);                                   // State 6
      tmp_state.Add_x_Choices(4);
      tmp_choice = tmp_state.Choice(1);                   // State 6, Choice 1 
      tmp_choice.Add_Action(new Log(Log_Name(0), R, NONE));
      tmp_choice.Add_Action(new Log(Log_Name(4), W, ABORT));
      tmp_choice.Set_Next_State(4);
      tmp_choice = tmp_state.Choice(2);                   // State 6, Choice 2 
      tmp_choice.Add_Action(new Log(Log_Name(2), R, COMMIT));
      tmp_choice.Add_Action(new Msg(Message_Name(8), 0, SND, MLT, NONE));
      tmp_choice.Set_Next_State(2);
      tmp_choice = tmp_state.Choice(3);                   // State 6, Choice 3 
      tmp_choice.Add_Action(new Log(Log_Name(3), R, COMMIT));
      tmp_choice.Set_Next_State(3);
      tmp_choice = tmp_state.Choice(4);                   // State 6, Choice 4 
      tmp_choice.Add_Action(new Log(Log_Name(4), R, ABORT));
      tmp_choice.Set_Next_State(4);
  }                                                     // void Add_Actions(0)

  protected void Display_Stats(int n) {
      // Method to output the expected statistics for protocol based on 
      // n participants
      super.Display_Statistics(n);

      Info("\tTotal messages (no failures)  \t: 4n = "+(4*n));

      // Removed - smr 
      // Info("\tTotal messages (inc. failures)\t: n(4n+7)/2 = "
      //                                                    +(n*((4*n)+7)/2));
      Info("\tMessage rounds                \t: 4"); 
  }                                                   // void Display_Stats(1)
 
}                                                           // class Table_2PC

/*--------------------------- End of Table_2PC.java -------------------------*/

