
/*
** Author : Steven Ramsay <smr@dcs.ed.ac.uk>, 03/02/97
** Files  : W_Box.java, W_Box.class
** Update : 11/06/97 
** Package: simdcp
**
** Description:
** Maintains all Site GUI methods 
*/

package eduni.simdcp;

import java.awt.*;

/* Methods defined in class W_Box
** PUBLIC
** W_Box(String)
** W_Box(int, String)
**
** PROTECTED
**        void    finalize()
**        void    Add_Output_Entry(String)
**        void    Add_Error_Entry(String)
**        void    Add_Log_Entry(String)
**
** PRIVATE
**        void    Setup_Window_No_Log()
**        void    Setup_Window_With_Log()
**        void    constrain(6)
**        void    constrain(14)
*/
final class W_Box extends Frame {

  int      id;
  String   name;
  TextArea log_area = null;
  TextArea info_area = null;
  TextArea error_area = null;

  Panel panel1, panel2, panel3;

  GridBagLayout gridbag = new GridBagLayout();

  public W_Box(String name) {
      super(name+" Window");
      this.name = name;
      this.id = 0;
      this.Setup_Window_No_Log();
      this.pack();
  }                                                    // constructor W_Box(1) 
 
  public W_Box(int id, String name) {
      super(name+id+" Window");
      this.name = name; 
      this.id = id;
      this.Setup_Window_With_Log();
      this.pack();
  }                                                    // constructor W_Box(2)

  private void Setup_Window_No_Log() {

      info_area = new TextArea(10, 50);
      info_area.setEditable(false);      
      error_area = new TextArea("", 2, 50, TextArea.SCROLLBARS_VERTICAL_ONLY);
      error_area.setEditable(false);      
      panel1 = new Panel();
      panel1.setLayout(gridbag);
     
      constrain(panel1, new Label(this.name+"  Output"), 0, 0, 1, 1);
      constrain(panel1, info_area, 0, 1, 1, 1,
                GridBagConstraints.BOTH, GridBagConstraints.NORTHWEST, 
                1.0, 1.0, 0, 0, 2, 0);

      constrain(panel1, new Label(this.name+"  Errors"), 0, 2, 1, 1);
      constrain(panel1, error_area, 0, 3, 1, 1,
                GridBagConstraints.HORIZONTAL, GridBagConstraints.NORTHWEST, 
                1.0, 0.0, 2, 0, 0, 0);

      this.setLayout(gridbag);

      constrain(this, panel1, 0, 0, 1, 1, 
                GridBagConstraints.BOTH, GridBagConstraints.NORTHWEST, 
                1.0, 1.0, 5, 5, 2, 5);
  }                                                   // void Window_No_Log(0) 

  private void Setup_Window_With_Log() {
      log_area = new TextArea("", 4, 25, TextArea.SCROLLBARS_VERTICAL_ONLY);
      log_area.setEditable(false);      
      info_area = new TextArea(10, 50);
      info_area.setEditable(false);      
      error_area = new TextArea("", 2, 50, TextArea.SCROLLBARS_VERTICAL_ONLY);
      error_area.setEditable(false);      

      panel1 = new Panel();
      panel1.setLayout(this.gridbag);
      panel2 = new Panel();
      panel2.setLayout(this.gridbag);
     
      constrain(panel1, new Label(this.name+this.id+"  DT Log"), 0, 0, 1, 1);
      constrain(panel1, log_area, 0, 1, 1, 1);
      
     
      constrain(panel2, new Label(this.name+this.id+"  Output"), 0, 0, 1, 1);
      constrain(panel2, info_area, 0, 1, 1, 1,
                GridBagConstraints.BOTH, GridBagConstraints.NORTHWEST, 
                1.0, 1.0, 0, 0, 2, 0);
      constrain(panel2, new Label(this.name+this.id+"  Errors"), 0, 2, 1, 1);
      constrain(panel2, error_area, 0, 3, 1, 1,
                GridBagConstraints.HORIZONTAL, GridBagConstraints.NORTHWEST, 
                1.0, 0.0, 2, 0, 0, 0);

      this.setLayout(gridbag);
      constrain(this, panel1, 0, 0, 1, 1, 
                GridBagConstraints.HORIZONTAL, GridBagConstraints.NORTHWEST, 
                1.0, 0.0, 5, 5, 2, 5);
      constrain(this, panel2, 0, 1, 1, 1, 
                GridBagConstraints.BOTH, GridBagConstraints.NORTHWEST, 
                1.0, 1.0, 2, 5, 5, 5);
       
  }                                           // void Setup_Window_With_Log(0)

  protected void finalize() throws Throwable {
      // Finalize method to clean up windows etc on termination
      super.finalize();
  }                                                        // void finalize(0)

  protected void Clear() {
      // Method to clear the output text areas
      int start, end;

      this.info_area.setText("");
      this.error_area.setText("");

      if (id != 0) {
        this.log_area.setText("");
      }

  }                                                           // void Clear(0)
 
  protected void Add_Output_Entry(String note) {
      // Method to add a debug/info message to the window 
      this.info_area.append("\n"+note);       // JDK 1.1 code

      /* For JDK 1.0.2 compatability of deprecated methods - smr
      ** try {
      **     this.info_area.append("\n"+note);       // JDK 1.1 code
      ** } catch (NoSuchMethodError e) {
      **     this.info_area.appendText("\n"+note);   // JDK 1.0.2 code
      ** }
      */
  }                                                // void Add_Output_Entry(1)

  protected void Add_Error_Entry(String note) {
      // Method to add an error message to the error window 
      String text = note;

      if (Option.Show_Debug() || Option.Show_Info()) {
          text = new String("ERROR: "+note);
          this.Add_Output_Entry(text);
      }
      this.error_area.append("\n"+note);      // JDK 1.1 code

      /* For JDK 1.0.2 compatability of deprecated methods - smr
      ** try {
      **     this.error_area.append("\n"+note);      // JDK 1.1 code
      ** } catch (NoSuchMethodError e) {
      **     this.error_area.appendText("\n"+note);  // JDK 1.0.2 code
      ** }
      */
  }                                                 // void Add_Error_Entry(1)

  protected void Add_Log_Entry(String note) {
      // Method to add a log entry the correct log window 
      if ((Option.Show_DTLog()) && (this.log_area != null)) {
  	  this.log_area.append("\n"+note);      // JDK 1.1 code
 
          /* For JDK 1.0.2 compatability of deprecated methods - smr
	  ** try {
	  **     this.log_area.append("\n"+note);        // JDK 1.1 code
	  ** } catch (NoSuchMethodError e) {
	  **     this.log_area.appendText("\n"+note);    // JDK 1.0.2 code
	  ** }
          */
      }
  }                                                   // void Add_Log_Entry(2)

  /* The following constrain methods are from "Java in a Nutshell" Section 5
  ** examples on GUIs.  Used to position components in containers.
  */  
  private void constrain(Container container, Component component, 
                        int grid_x, int grid_y, int grid_w, int grid_h,
                        int fill, int anchor, 
                        double weight_x, double weight_y,
                        int top, int left, int bottom, int right) 
  {

      GridBagConstraints c = new GridBagConstraints();
      c.gridx = grid_x; 
      c.gridy = grid_y;
      c.gridwidth = grid_w; 
      c.gridheight = grid_h;
      c.fill = fill; 
      c.anchor = anchor;
      c.weightx = weight_x; 
      c.weighty = weight_y;

      if (top + bottom + left + right > 0) {
          c.insets = new Insets(top, left, bottom, right);
      }
      ((GridBagLayout)container.getLayout()).setConstraints(component, c);
      container.add(component);
  }                                                      // void constrain(14)
    
  private void constrain(Container container, Component component, 
                         int grid_x, int grid_y, int grid_w, int grid_h) 
  {
      constrain(container, component, grid_x, grid_y, grid_w, grid_h, 
                GridBagConstraints.NONE, GridBagConstraints.NORTHWEST, 
                0.0, 0.0, 0, 0, 0, 0);
  }                                                       // void constrain(6)
    
}                                                               // class W_Box

/*------------------------------- End of W_Box.java -------------------------*/

